% Data script for the construction of an d-stable descriptor system.
%
% DESCRIPTION:
%   This data script generates the matrices of an d-stable descriptor LTI
%   system of the form
%
%       Ex(t+1) = Ax(t) + Bu(t),
%          y(t) = Cx(t) + Du(t),
%
%   with E and A (n x n) matrices, B an (n x m) matrix, C a (p x n)
%   matrix, and D a (p x m) matrix.
%
%   The construction has the following dimensions:
%       n = 100,
%       m = 3,
%       p = 2,
%   and the matrix pencil s*E - A is constructed such that all eigenvalues
%   lie inside the open unit disk.
%   In the datainfo struct the following numbers are saved:
%       - n, number of states,
%       - ni, number of infinite eigenvalues of s*E - A,
%       - np, number of finite eigenvalues of s*E - A inside the open unit
%         disk,
%       - nu, number of finite eigenvalues of s*E - A outside the open unit
%         disk.
%
% NOTES:
%   * The data, generated under Matlab 9.1.0.441655 (R2016b) GLNXA64, is
%     saved in the same-named .MAT file.
%   * This script cannot be used under Octave.
%
% See also morlab_data_desc_infunstab.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%

% System dimensions.
n = 100;
m = 3;
p = 2;

% Random seed.
s = 1.0;

% System generation.
rng(s);
B = rand(n, m);

rng(s);
C = [rand(p, n - 10), zeros(p, 8), eye(p, 2)];

rng(s);
D = rand(p, m);

% Construction of the matrix pencil.
rng(s);
A = randn(n);
E = eye(n);

% System transformation.
rng(s)
U = orth(rand(n));

A = U' * A * U;
E = U' * E * U;
B = U' * B;
C = C * U;

A = A / 6;
E = 2 * E;

% Set sampling time to 1.
Ts = 1;

% Info about the generated data structure.
x = abs(eig(A, E));

datainfo = struct(...
    'n' , n, ...
    'ni', 0, ...
    'np', sum(x < 1), ...
    'nu', sum(x > 1));

% Clear unwanted variables.
clear n m p s U x;
