% Data script for the construction of an unstable descriptor system.
%
% DESCRIPTION:
%   This data script generates the matrices of an unstable descriptor LTI
%   system of the form
%
%       Ex'(t) = Ax(t) + Bu(t),
%         y(t) = Cx(t) + Du(t),
%
%   with E and A (n x n) matrices, B an (n x m) matrix, C a (p x n)
%   matrix, and D a (p x m) matrix.
%
%   The construction has the following dimensions:
%       n = 100,
%       m = 3,
%       p = 2,
%   and the matrix pencil s*E - A is constructed such that 10 eigenvalues
%   are infinite with index 3, 10 eigenvalues lie in the open right
%   half-plane and 80 in the open left half-plane.
%   Also, the system is constructed to get a proper transfer function.
%   In the datainfo struct the following numbers are saved:
%       - n, number of states,
%       - ni, number of infinite eigenvalues of s*E - A,
%       - np, number of finite eigenvalues of s*E - A in the open left
%         half-plane,
%       - nu, number of finite eigenvalues of s*E - A in the open right
%         half-plane.
%
% NOTES:
%   * The data, generated under Matlab 8.0.0.783 (R2012b) GLNXA64, is saved
%     in the same-named .MAT file.
%   * This script cannot be used under Octave.
%
% See also morlab_data_std_unstab.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%

% System dimensions.
n = 100;
m = 3;
p = 2;

% Random seed.
s = 1.0;

% System generation.
rng(s);
B = rand(n, m);

rng(s);
C = [rand(p, n - 10), zeros(p, 8), eye(p, 2)];

rng(s);
D = rand(p, m);

% Construction of the matrix pencil.
rng(s);
A = blkdiag(rand(n - 10), eye(10));

E         = blkdiag(eye(n - 10), zeros(10));
E(91, 92) = 1;
E(92, 93) = 1;
E(95, 96) = 1;

% System transformation.
rng(s)
U = orth(rand(n));

A = U' * A * U;
E = U' * E * U;
B = U' * B;
C = C * U;

A = A - 2 * E;

% Info about the generated data structure.
x = real(eig(A));

datainfo = struct(...
    'n' , n, ...
    'ni', 10, ...
    'np', sum(x < 0), ...
    'nu', n - 10 - sum(x < 0));

% Clear unwanted variables.
clear n m p s U x;
