function [X, info] = ml_cabe_sgn(A, Q, E, opts)
%ML_CABE_SGN Continuous-time algebraic Bernoulli equation solver.
%
% SYNTAX:
%   [X, info] = ML_CABE_SGN(A, Q)
%   [X, info] = ML_CABE_SGN(A, Q, [])
%   [X, info] = ML_CABE_SGN(A, Q, [], opts)
%
%   [X, info] = ML_CABE_SGN(A, Q, E)
%   [X, info] = ML_CABE_SGN(A, Q, E, opts)
%
% DESCRIPTION:
%   Computes the solution matrix of the standard algebraic Bernoulli
%   equation
%
%       A'*X + X*A - X*Q*X = 0,                                         (1)
%
%   or of the generalized algebraic Bernoulli equation
%
%       A'*X*E + E'*X*A - E'*X*Q*X*E = 0,                               (2)
%
%   using the sign function iteration. It is assumed that the eigenvalues
%   of A (or s*E - A) lie in the open right half-plane.
%
% INPUT:
%   A    - matrix with dimensions n x n in (1) or (2)
%   Q    - matrix with dimensions n x n in (1) or (2)
%   E    - matrix with dimensions n x n in (2), if empty the standard
%          equation (1) is solved
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | AbsTol          | nonnegative scalar, tolerance for the absolute    |
%   |                 | error in the last iteration step                  |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Info            | {0, 1}, used to disable/enable display of verbose |
%   |                 | status information during iteration steps         |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | MaxIter         | positive integer, maximum number of iteration     |
%   |                 | steps                                             |
%   |                 | (default 100)                                     |
%   +-----------------+---------------------------------------------------+
%   | RelTol          | nonnegative scalar, tolerance for the relative    |
%   |                 | error in the last iteration step                  |
%   |                 | (default 1.0e+01*n*eps)                           |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   X    - solution matrix of (1) or (2)
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | AbsErr          | vector, containing the absolute error of the      |
%   |                 | iteration matrix in each iteration step           |
%   +-----------------+---------------------------------------------------+
%   | IterationSteps  | number of performed iteration steps               |
%   +-----------------+---------------------------------------------------+
%   | RelErr          | vector, containing the relative error of the      |
%   |                 | iteration matrix in each iteration step           |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   S. Barrachina, P. Benner, E. S. Quintana-Orti, Efficient algorithms for
%   generalized algebraic Bernoulli equations based on the matrix sign
%   function, Numer. Algorithms 46 (4) (2007) 351--368.
%   https://doi.org/10.1007/s11075-007-9143-x
%
% See also ml_lyap_sgn, ml_sylv_sgn.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(2, 4);

if (nargin < 4) || isempty(opts)
    opts = struct();
end

% Check input matrices.
n = size(A, 1);

assert(isequal(size(A), [n n]), ...
    'MORLAB:data', ...
    'The matrix A has to be square!');

assert(isequal(size(Q), [n n]), ...
    'MORLAB:data', ...
    'The matrix Q must have the same dimensions as A!');

if issparse(A), A = full(A); end
if issparse(Q), Q = full(Q); end

if (nargin >= 3) && not(isempty(E))
    assert(isequal(size(E), [n n]), ...
        'MORLAB:data', ...
        'The matrix E must have the same dimensions as A!');
    
    if issparse(E), E = full(E); end
    
    hasE = 1;
else
    E    = eye(n);
    hasE = 0;
end

% Check and assign optional parameters.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');

if ml_field_set_to_value(opts, 'AbsTol')
    ml_assert_nonnegscalar(opts.AbsTol, 'opts.AbsTol');
else
    opts.AbsTol = 0;
end

if ml_field_set_to_value(opts, 'Info')
    ml_assert_boolean(opts.Info, 'opts.Info');
else
    opts.Info = false;
end

if ml_field_set_to_value(opts, 'MaxIter')
    ml_assert_posinteger(opts.MaxIter, 'opts.MaxIter');
else
    opts.MaxIter = 100;
end

if ml_field_set_to_value(opts, 'RelTol')
    ml_assert_nonnegscalar(opts.RelTol, 'opts.RelTol');
else
    opts.RelTol = 1.0e+01 * n * eps;
end

% Case of empty data.
if isempty(A)
    X    = [];
    info = struct([]);
    return;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INITIALIZATION.                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

niter            = 0;
[abserr, relerr] = deal(zeros(1, opts.MaxIter));

if hasE
    normE = norm(E, 'fro');
else
    normE = sqrt(n);
end

abserr(1) = norm(A + E, 'fro');
relerr(1) = abserr(1) / normE;

% Method is converged if absolute or relative errors are small enough.
converged = (abserr(1) <= opts.AbsTol) || (relerr(1) <= opts.RelTol);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIGN FUNCTION ITERATION.                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

while (niter < opts.MaxIter) && not(converged)
    EAinv  = E / A;
    if hasE
        EAinvE = EAinv * E;
    else
        EAinvE = EAinv;
    end
    
    % Scaling factor for convergence acceleration.
    if ((niter == 0) && (relerr(1) > 1.0e-02)) ...
            || ((niter >= 1) && (relerr(niter) > 1.0e-02))
        c = sqrt(norm(A, 'fro') / norm(EAinvE, 'fro'));
    else
        c = 1.0;
    end
    c1 = 1.0 / (2.0 * c);
    c2 = 0.5 * c;
    
    % Construction of next solution matrix.
    Q = c1 * Q + c2 * (EAinv * (Q * EAinv'));
    
    % Update of iteration matrix.
    A = c1 * A + c2 * EAinvE;
    
    niter         = niter + 1;
    abserr(niter) = norm(A - E, 'fro');
    relerr(niter) = abserr(niter) / normE;
    
    % Information about current iteration step.
    if opts.Info
        fprintf(1, ['CABE_SGN step: %4d absolute error: %e' ...
            ' relative error: %e \n'], ...
            niter, abserr(niter), relerr(niter));
    end
    
    % Method is converged if absolute or relative errors are small enough.
    converged = (abserr(niter) <= opts.AbsTol) || ...
        (relerr(niter) <= opts.RelTol);
end

if hasE
    X = [Q; E' - A'] \ [A / E + eye(n); zeros(n)];
else
    X = [Q; E' - A'] \ [A + eye(n); zeros(n)];
end

% Warning if iteration not converged.
if (niter == opts.MaxIter) && not(converged)
    warning('MORLAB:eqn_solver', ...
        ['No convergence in %d iteration steps!\n' ...
        'Abs. tolerance: %e | Abs. error: %e\n' ...
        'Rel. tolerance: %e | Rel. error: %e\n' ...
        'Try to increase the tolerances or number of ' ...
        'iteration steps!'], ...
        niter, opts.AbsTol, min(abserr(niter)), ...
        opts.RelTol, min(relerr(niter)));
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN INFORMATION.                                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign information about iteration.
info = struct( ...
    'AbsErr'        , abserr(1:niter), ...
    'IterationSteps', niter, ...
    'RelErr'        , relerr(1:niter));
