function [varargout] = ml_ct_dss_hinfbt(varargin)
%ML_CT_DSS_HINFBT H-infinity balanced truncation for descriptor systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_HINFBT(A, B, C, D, E)
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_HINFBT(A, B, C, D, E, opts)
%
%   [rom, info] = ML_CT_DSS_HINFBT(sys)
%   [rom, info] = ML_CT_DSS_HINFBT(sys, opts)
%
% DESCRIPTION:
%   This function computes the H-infinity balanced truncation for a
%   descriptor system of the form
%
%       E*x'(t) = A*x(t) + B*u(t),                                      (1)
%          y(t) = C*x(t) + D*u(t).                                      (2)
%
%   Therefore, first an additive decomposition of the system is performed
%   using the matrix disk function, such that
%
%            [ Ei  0 ]       [ Ai  0 ]                        [ Ci ]
%       E2 = [       ], A2 = [       ], B2 = [ Bi, Bp ], C2 = [    ],
%            [ 0  Ep ]       [ 0  Ap ]                        [ Cp ]
%
%   with (Ei, Ai, Bi, Ci, D) belonging to the polynomial part and
%   (Ep, Ap, Bp, Cp, 0) belonging to the strictly proper part.
%   Now, the two generalized continuous-time Riccati equations
%
%       Ap*Pp*Ep' + Ep*Pp*Ap' + Bp*Bp' - beta
%           * (Ep*Pp*Cp' + B*M') * inv(Rb) * (Ep*Pp*Cp' + B*M')' = 0,
%       Ap'*Qp*Ep + Ep'*Qp*Ap + Cp'*Cp - beta
%           * (Bp'*Qp*Ep + M'*C)' * inv(Rc) * (Bp'*Qp*Ep + M'*C) = 0
%
%   are solved for the reduction of the strictly proper part, with
%   beta = (1 - GAM^(-2)) and
%
%       Rb = I + M*M',
%       Rc = I + M'*M,
%
%   where M = D - Ci * inv(Ai) * Bi. Also, the two generalized
%   discrete-time Lyapunov equations
%
%       Ai*Pi*Ai' - Ei*Pi*Ei' - Bi*Bi' = 0,
%       Ai'*Qi*Ai - Ei'*Qi*Ei - Ci'*Ci = 0
%
%   are solved for the reduction of the polynomial part. As result, a
%   reduced-order system of the form
%
%       Er*x'(t) = Ar*x(t) + Br*u(t),                                   (3)
%           y(t) = Cr*x(t) + Dr*u(t)                                    (4)
%
%   is computed, such that for the original transfer function G with the
%   (right) coprime factorization sqrt(beta)*G = N*inv(M) and the
%   reduced-order transfer function Gr with an r-th order strictly proper
%   part and the (right) coprime factorization sqrt(beta)*Gr = Nr*inv(Mr)
%   it holds
%
%       1/sqrt(beta) *||[N; M] - [Nr; Mr]||_{\infty} <=
%           2 * (Hsvp(r+1)/sqrt(1 + (beta*Hsvp(r+1))^2) + ...
%           + Hsvp(n)/sqrt(1 + (beta*Hsvp(n))^2)),
%
%   with Hsvp a vector containing the characteristic H-infinity singular
%   values of the system.
%
%   Note:
%     1) The parameter GAM refers to the optimal cost gamma computed during
%        the H-infinity controller construction of the LQG normalized
%        system with feed-through term
%
%           [ s*E - A | B 0 B ]
%           [ --------------- ]
%           [       C | 0 0 0 ].
%           [       0 | 0 0 I ]
%           [       C | 0 I D ]
%
%        Still, there is no known implementation of the Gamma-iteration
%        for descriptor systems in MATLAB.
%        In case of estimating/computing an optimal cost parameter GAM_OPT,
%        the GAM > GAM_OPT can be chosen arbitrarily.
%        By default, the maximum of 1 and the given GAM is disturbed by
%        0.01% before used for computations.
%     2) In the case of GAM == Inf, the H-infinity balanced truncation is
%        identical to the LQG balanced truncation.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   sys  - structure or state-space object, containing the descriptor
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Beta            | nonnegative scalar, used as shift of the          |
%   |                 | in Bass' algorithm for better conditioning if     |
%   |                 | StabMethod == 'lyap' is chosen only used if       |
%   |                 | RiccatiSolver = 'newton'                          |
%   |                 | (default 0.1)                                     |
%   +-----------------+---------------------------------------------------+
%   | caredlopts      | structure, containing the optional parameters for |
%   |                 | the Riccati equation sign function solver, only   |
%   |                 | used if RiccatiSolver = 'sign', see               |
%   |                 | ml_caredl_sgn_fac                                 |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | careopts        | structure, containing the optional parameters for |
%   |                 | the computation of the continuous-time algebraic  |
%   |                 | Riccati equations, only used if                   |
%   |                 | RiccatiSolver = 'newton', see ml_care_nwt_fac     |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | DecompEig       | positive scalar, overestimation of the absolute   |
%   |                 | value of the largest finite eigenvalue of s*E - A,|
%   |                 | if set, replaces the computation with DecompTol   |
%   |                 | (default [])                                      |
%   +-----------------+---------------------------------------------------+
%   | DecompTol       | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest singular value of E to determine the      |
%   |                 | smallest non-quasi-zero singular value of E       |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, used for the scaling of the      |
%   |                 | Riccati equations, see GAM above                  |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | gdlyapdlopts    | structure, containing the optional parameters for |
%   |                 | the computation of the generalized discrete-time  |
%   |                 | Lyapunov equations, see ml_gdlyapdl_smith_fac     |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | ImproperTrunc   | nonnegative scalar, tolerance multiplied with the |
%   | {!}             | largest proper Hankel singular value of the       |
%   |                 | system to truncate the improper part, if 0 no     |
%   |                 | improper balanced truncation is performed         |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | Index           | nonnegative integer, index of the descriptor      |
%   |                 | system used to set an upper bound on the size of  |
%   |                 | the reduced improper part, Inf if unknown         |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | infdecopts      | structure, containing the optional parameters for |
%   |                 | the decomposition of the finite and infinite parts|
%   |                 | of the system using the disk function and subspace|
%   |                 | extraction method, see ml_disk and ml_getqz       |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Method          | character array, determining algorithm for the    |
%   | {!}             | computation of the reduced-order model            |
%   |                 |  'sr'   - square-root method                      |
%   |                 |  'bfsr' - balancing-free square-root method       |
%   |                 | (default 'sr')                                    |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   | {!}             | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsvp)) + Nu + Ni)          |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using absolute error bound         |
%   |                 | (default 'tolerance')                             |
%   +-----------------+---------------------------------------------------+
%   | RiccatiSolver   | character array, determining the solver for the   |
%   |                 | dual Riccati equations                            |
%   |                 |  'newton' - Newton iteration                      |
%   |                 |  'sign'   - dual sign function method             |
%   |                 | (default 'sign')                                  |
%   +-----------------+---------------------------------------------------+
%   | stabdecopts     | structure, containing the optional parameters for |
%   |                 | the decomposition of the stable and unstable parts|
%   |                 | of the system using the sign function and subspace|
%   |                 | extraction method, , only used if                 |
%   |                 | RiccatiSolver = 'newton'see ml_signm and ml_getqz |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | StabMethod      | character array, determining the method of        |
%   |                 | stabilization of the system, only used if         |
%   |                 | RiccatiSolver = 'newton'                          |
%   |                 |   'cabe'  - partial stabilization with the        |
%   |                 |            algebraic Bernoulli equation           |
%   |                 |   'lyap' - partial stabilization with Bass'       |
%   |                 |            algorithm                              |
%   |                 | (default 'cabe')                                  |
%   +-----------------+---------------------------------------------------+
%   | stabmethodopts  | structure, containing the optional parameters for |
%   |                 | the sign function based Lyapunov or Bernoulli     |
%   |                 | equation solver used for the stabilization, only  |
%   |                 | used if RiccatiSolver = 'newton', see ml_cabe_sgn |
%   |                 | or ml_lyap_sgn                                    |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | StoreProjection | {0, 1}, used to disable/enable storing of the     |
%   |                 | computed projection matrices W and T              |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 | by an absolute error bound if 'tolerance' is set  |
%   |                 | for OrderComputation                              |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   Er   - matrix of (3) with dimensions r x r
%   rom  - structure or state-space object, containing the reduced-order
%          descriptor system:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | AbsErrBound     | computed error bound for the absolute error of the|
%   | {!}             | (right) coprime factorization in H-infinity norm  |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, the disturbed scaling parameter  |
%   |                 | of the computations                               |
%   +-----------------+---------------------------------------------------+
%   | Hsvi            | a vector, containing the computed Hankel singular |
%   |                 | values of the improper part of the system         |
%   +-----------------+---------------------------------------------------+
%   | Hsvp            | a vector, containing the computed characteristic  |
%   |                 | H-infinity singular values of the proper part of  |
%   |                 | the system                                        |
%   +-----------------+---------------------------------------------------+
%   | infoCARE_C      | structure, containing information about the       |
%   |                 | Newton solver for the regulator Riccati equation, |
%   |                 | see ml_care_nwt_fac                               |
%   +-----------------+---------------------------------------------------+
%   | infoCARE_O      | structure, containing information about the       |
%   |                 | Newton solver for the filter Riccati equation,    |
%   |                 | see ml_care_nwt_fac                               |
%   +-----------------+---------------------------------------------------+
%   | infoCAREDL      | structure, containing information about the       |
%   |                 | sign function solver for the dual Riccati         |
%   |                 | equations, see ml_caredl_sgn_fac                  |
%   +-----------------+---------------------------------------------------+
%   | infoADTF        | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | infinite, finite stable and finite anti-stable    |
%   |                 | parts, see ml_ct_dss_adtf                         |
%   +-----------------+---------------------------------------------------+
%   | infoGDLYAPDL    | structure, containing information about the       |
%   |                 | generalized discrete-time Lyapunov equation solver|
%   |                 | for the improper Gramians,                        |
%   |                 | see ml_gdlyapdl_smith_fac                         |
%   +-----------------+---------------------------------------------------+
%   | infoPARTSTAB_C  | structure, containing information about the       |
%   |                 | partial stabilization used for the controllability|
%   |                 | Riccati equation, see ml_ct_dss_partstab          |
%   +-----------------+---------------------------------------------------+
%   | infoPARTSTAB_O  | structure, containing information about the       |
%   |                 | partial stabilization used for the observability  |
%   |                 | Riccati equation, see ml_ct_dss_partstab          |
%   +-----------------+---------------------------------------------------+
%   | Nf              | Dimension of the finite reduced part in the       |
%   | {!}             | reduced-order model                               |
%   +-----------------+---------------------------------------------------+
%   | Ni              | Dimension of the improper part in the reduced-    |
%   | {!}             | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | T               | projection matrix used as right state-space       |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%   | W               | projection matrix used as left state-space        |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%
% See also ml_ct_ss_hinfbt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_dss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 6);
else
    nargoutchk(0, 2);
end

% Fill the D matrix.
if isempty(sys.D)
    sys.D = zeros(size(sys.C, 1), size(sys.B, 2));
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'caredlopts')
    assert(isa(opts.caredlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.careopts has to be a struct!');
else
    opts.caredlopts = struct();
end

if ml_field_set_to_value(opts, 'careopts')
    assert(isa(opts.careopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.careopts has to be a struct!');
else
    opts.careopts = struct();
end

if ml_field_set_to_value(opts, 'Gamma')
    ml_assert_posscalar(opts.Gamma, 'opts.Gamma');
else
    opts.Gamma = Inf;
end

if ml_field_set_to_value(opts, 'gdlyapdlopts')
    assert(isa(opts.gdlyapdlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.gdlyapdlopts has to be a struct!');
else
    opts.gdlyapdlopts = struct();
end

if ml_field_set_to_value(opts, 'infdecopts')
    assert(isa(opts.infdecopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.infdecopts has to be a struct!');
else
    opts.infdecopts = struct();
end

opts       = ml_check_cell_param(opts, 'Method', @ml_assert_char, 'sr');
numMethods = length(opts.Method);
mor_method = cell(1, numMethods);
for k = 1:numMethods
    method = opts.Method{k};
    if strcmpi(method, 'bfsr')
        mor_method{k} = @ml_dss_bfsr;
    elseif strcmpi(method, 'sr')
        mor_method{k} = @ml_dss_sr;
    else
        error('MORLAB:data', ...
            'The given computation method is not implemented!');
    end
end

opts = ml_check_cell_param(opts, 'OrderComputation', ...
    @ml_assert_char, 'tolerance');
numOrderComp = length(opts.OrderComputation);
tselect      = cell(1, numOrderComp);
for k = 1:numOrderComp
    if strcmpi(opts.OrderComputation{k}, 'order')
        tselect{k} = 0;
    elseif strcmpi(opts.OrderComputation{k}, 'tolerance')
        tselect{k} = 6;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
end

if ml_field_set_to_value(opts, 'RiccatiSolver')
    assert(strcmpi(opts.RiccatiSolver, 'newton') ...
        || strcmpi(opts.RiccatiSolver, 'sign'), ...
        'MORLAB:data', ...
        'The desired Riccati equation solver is not implemented!');
else
    opts.RiccatiSolver = 'sign';
end

if ml_field_set_to_value(opts, 'stabdecopts')
    assert(isa(opts.stabdecopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.stabdecopts has to be a struct!');
else
    opts.stabdecopts = struct();
end

if ml_field_set_to_value(opts, 'StoreProjection')
    ml_assert_boolean(opts.StoreProjection, 'opts.StoreProjection');
else
    opts.StoreProjection = false;
end

% Handling of anti-stable dimension for decomposition or stabilization.
if ml_field_set_to_value(opts.stabdecopts, 'Dimension')
    ml_assert_integer(opts.stabdecopts.Dimension, ...
        'opts.stabdecopts.Dimension');
else
    opts.stabdecopts.Dimension = -1;
end

nu                         = opts.stabdecopts.Dimension;
opts.stabdecopts.Dimension = 0;

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into finite and infinite parts.
[sys, infoADTF] = ml_ct_dss_adtf(sys, opts);

ninf = size(sys.Ainf, 1);

% Assign information about additive decomposition.
info.infoADTF = rmfield(infoADTF, {'T'; 'W'});


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARTIAL STABILIZATION AND SOLVE MATRIX EQUATIONS.                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Disturbance of scaling parameter.
opts.Gamma = 1.0001 * max(1.0, opts.Gamma);
gam        = sqrt(1 - 1 / opts.Gamma^2);

% Computation of the constant polynomial part.
if ninf > 0
    M = sys.D - sys.Cinf * (sys.Ainf \ sys.Binf);
else
    M = sys.D;
end

if size(sys.A, 1) > 0
    % Computation of matrices for Riccati equations.
    Rb = eye(size(sys.C, 1)) + M * M';
    Lb = chol(Rb);
    Rc = eye(size(sys.B, 2)) + M' * M;
    Lc = chol(Rc);
    
    A = sys.A - sys.B * M' * (Rb \ sys.C);
    B = sys.B / Lc;
    C = Lb' \ sys.C;
    
    switch lower(opts.RiccatiSolver)
        case 'newton'
            % Partial stabilization.
            opts.stabdecopts.Dimension = nu;
            opts.infdecopts.Dimension  = 0;
            
            [K1, infoPARTSTAB_C] = ml_ct_dss_partstab(A', gam * C', ...
                sys.E', opts);
            [K2, infoPARTSTAB_O] = ml_ct_dss_partstab(A, gam * B, ...
                sys.E, opts);
            
            % Computation of the full-rank factors of the proper Gramians.
            opts.careopts.K0 = K1;
            [R, infoCARE_C]  = ml_care_nwt_fac(A', gam * C', B', ...
                sys.E', opts.careopts);
            
            opts.careopts.K0 = K2;
            [L, infoCARE_O]  = ml_care_nwt_fac(A, gam * B, C, ...
                sys.E, opts.careopts);
            
            info.infoCARE_C     = infoCARE_C;
            info.infoCARE_O     = infoCARE_O;
            info.infoPARTSTAB_C = infoPARTSTAB_C;
            info.infoPARTSTAB_O = infoPARTSTAB_O;
            
        case 'sign'
            % Compute the two system Gramians.
            [L, R, infoCAREDL] = ml_caredl_sgn_fac(A, B, C, sys.E, ...
                -gam^2, opts.caredlopts);
            
            info.infoCAREDL = infoCAREDL;
            
    end
else
    [R, L] = deal([]);
end

% Computation of the full-rank factors of the improper Gramians.
if ninf > 0
    [Rinf, Linf, infoGDLYAPDL] = ml_gdlyapdl_smith_fac( ...
        sys.Ainf, sys.Binf, sys.Cinf, sys.Einf, opts.gdlyapdlopts);
    
    % Assign information about improper Lyapunov equation solver.
    info.infoGDLYAPDL = infoGDLYAPDL;
else
    [Rinf, Linf] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE REDUCED-ORDER MODEL.                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if numMethods == 1
    [roms, hsvp, hsvi, prj] = mor_method{1}(sys, R, Rinf, L, Linf, ...
        tselect, opts);
    
    T    = prj.T;
    Tinf = prj.Tinf;
    W    = prj.W;
    Winf = prj.Winf;
    
    if not(isa(roms, 'cell'))
        roms = {roms};
        T    = {T};
        Tinf = {Tinf};
        W    = {W};
        Winf = {Winf};
    end
else
    roms = cell(1, numMethods);
    T    = cell(1, numMethods);
    Tinf = cell(1, numMethods);
    W    = cell(1, numMethods);
    Winf = cell(1, numMethods);
    for k = 1:numMethods
        opts.vecInd = k;
        opts.vecMax = numMethods;
        [roms{k}, hsvp, hsvi, prj] = mor_method{k}(sys, R, Rinf, L, ...
            Linf, tselect, opts);
        
        T{k}    = prj.T;
        Tinf{k} = prj.Tinf;
        W{k}    = prj.W;
        Winf{k} = prj.Winf;
    end
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign output information.
numSys      = length(roms);
nf          = cell(1, numSys);
absErrBound = cell(1, numSys);
for k = 1:numSys
    nf{k}          = size(roms{k}.A, 1);
    absErrBound{k} = 2 * sum(hsvp(nf{k}+1:end) ...
        ./ sqrt(1 + (gam * hsvp(nf{k}+1:end)).^2));
end

if (ml_field_set_to_value(opts, 'ImproperTrunc') ...
        && isa(opts.ImproperTrunc, 'cell'))
    ni = cell(1, numSys);
    for k = 1:numSys
        ni{k} = size(roms{k}.Ainf, 1);
    end
else
    ni = size(roms{1}.Ainf, 1);
end

info.AbsErrBound = absErrBound;
info.Gamma       = opts.Gamma;
info.Hsvi        = hsvi;
info.Hsvp        = hsvp;
info.Nf          = nf;
info.Ni          = ni;

if opts.StoreProjection
    for k = 1:numSys
        info.T{k} = infoADTF.T * blkdiag(T{k}, Tinf{k});
        info.W{k} = blkdiag(W{k}, Winf{k}) * infoADTF.W;
    end
else
    info.T = [];
    info.W = [];
end

[varargout{1:nargout}] = ml_dss_format_output(roms, ioformat, info);
