function [varargout] = ml_ct_dss_hna(varargin)
%ML_CT_DSS_HNA Hankel-norm approximation for descriptor systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_HNA(A, B, C, D, E)
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_HNA(A, B, C, D, E, opts)
%
%   [rom, info] = ML_CT_DSS_HNA(sys)
%   [rom, info] = ML_CT_DSS_HNA(sys, opts)
%
% DESCRIPTION:
%   This function computes the generalized Hankel-norm approximation of a
%   descriptor system
%
%       E*x'(t) = A*x(t) + B*u(t),                                      (1)
%          y(t) = C*x(t) + D*u(t).                                      (2)
%
%   Therefore, first a balanced realization is computed by using the
%   generalized balanced truncation square-root method with an appropriate
%   tolerance for the minimal realization of the given system. Then the
%   strictly proper part of the system is transformed using the formulas
%   for all-pass systems. As result, a reduced-order system of the form
%
%       Er*x'(t) = Ar*x(t) + Br*u(t),                                   (3)
%           y(t) = Cr*x(t) + Dr*u(t)                                    (4)
%
%   is computed, such that for the original transfer function G and the
%   reduced-order transfer function Gr with an r-th order strictly proper
%   part it holds
%
%       ||G - Gr||_{H}       = Hsvp(r+1),
%       ||G - Gr||_{\infty} <= 2 * (Hsvp(r+1) + ... + Hsvp(n)),
%
%   with Hsvp, a vector containing the proper Hankel singular values of the
%   system.
%
%   Note: For unstable systems, an additional additive decomposition into
%         the stable and anti-stable parts is performed and then only the
%         stable part will be reduced. That does not change the error
%         formulas.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   E    - matrix from (1) with dimensions n x n
%   sys  - structure or state-space object, containing the descriptor
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | DecompEig       | positive scalar, overestimation of the absolute   |
%   |                 | value of the largest finite eigenvalue of s*E - A,|
%   |                 | if set, replaces the computation with DecompTol   |
%   |                 | (default [])                                      |
%   +-----------------+---------------------------------------------------+
%   | DecompTol       | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest singular value of E to determine the      |
%   |                 | smallest non-quasi-zero singular value of E       |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | gdlyapdlopts    | structure, containing the optional parameters for |
%   |                 | the computation of the generalized discrete-time  |
%   |                 | Lyapunov equations, see ml_gdlyapdl_smith_fac     |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | hankeldecopts   | structure, containing the optional parameters for |
%   |                 | the disk function used for the decomposition      |
%   |                 | after the transformation to an all-pass system    |
%   |                 | see ml_disk and ml_getqz                          |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | ImproperTrunc   | nonnegative scalar, tolerance multiplied with the |
%   | {!}             | largest proper Hankel singular value of the       |
%   |                 | system to truncate the improper part, if 0 no     |
%   |                 | improper balanced truncation is performed         |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | Index           | nonnegative integer, index of the descriptor      |
%   |                 | system used to set an upper bound on the size of  |
%   |                 | the reduced improper part, Inf if unknown         |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | infdecopts      | structure, containing the optional parameters for |
%   |                 | the decomposition of the finite and infinite parts|
%   |                 | of the system using the disk function and subspace|
%   |                 | extraction method, see ml_disk and ml_getqz       |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | lyapdlopts      | structure, containing the optional parameters for |
%   |                 | the computation of the generalized continuous-time|
%   |                 | Lyapunov equations, see ml_lyapdl_sgn_fac         |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | MinRelTol       | nonnegative scalar, tolerance multiplied with the |
%   | {!}             | largest characteristic value to determine a       |
%   |                 | minimal realization                               |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   | {!}             | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsvp)) + Nu + Ni)          |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using absolute error bound         |
%   |                 | (default 'tolerance')                             |
%   +-----------------+---------------------------------------------------+
%   | stabdecopts     | structure, containing the optional parameters for |
%   |                 | the decomposition of the stable and unstable parts|
%   |                 | of the system using the sign function and subspace|
%   |                 | extraction method, see ml_signm and ml_getqz      |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 | by an absolute error bound if 'tolerance' is set  |
%   |                 | for OrderComputation                              |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   Er   - matrix of (3) with dimensions r x r
%   rom  - structure or state-space object, containing the reduced-order
%          descriptor system:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | AbsErrBound     | computed error bound for the absolute error of the|
%   | {!}             | reduced-order model in H-infinity norm            |
%   +-----------------+---------------------------------------------------+
%   | Hsvi            | a vector, containing the computed Hankel singular |
%   |                 | values of the improper part of the system         |
%   +-----------------+---------------------------------------------------+
%   | Hsvp            | a vector, containing the computed Hankel singular |
%   |                 | values of the proper part of the system           |
%   +-----------------+---------------------------------------------------+
%   | infoADTF        | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | infinite, finite stable and finite anti-stable    |
%   |                 | parts, see ml_ct_dss_adtf                         |
%   +-----------------+---------------------------------------------------+
%   | infoGDLYAPDL    | structure, containing information about the       |
%   |                 | generalized discrete-time Lyapunov equation solver|
%   |                 | for the improper Gramians,                        |
%   |                 | see ml_gdlyapdl_smith_fac                         |
%   +-----------------+---------------------------------------------------+
%   | infoHAADTF      | structure, containing information about the       |
%   |                 | disk function method, see ml_ct_dss_adtf          |
%   +-----------------+---------------------------------------------------+
%   | infoLYAPDL      | structure, containing information about the       |
%   |                 | continuous-time dual Lyapunov equations solver,   |
%   |                 | see ml_lyapdl_sgn_fac                             |
%   +-----------------+---------------------------------------------------+
%   | Ni              | Dimension of the improper part in the reduced-    |
%   | {!}             | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | Np              | Dimension of the proper part in the reduced-order |
%   | {!}             | model                                             |
%   +-----------------+---------------------------------------------------+
%   | Nu              | Dimension of the unstable part in the reduced-    |
%   |                 | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | Sigma           | Chosen proper Hankel singular value, exact        |
%   | {!}             | approximation error in the Hankel-norm            |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   S. Werner, Hankel-norm approximation of descriptor systems, Master's
%   thesis, Otto von Guericke University, Magdeburg, Germany (2016).
%   http://nbn-resolving.de/urn:nbn:de:gbv:ma9:1-8845
%
% See also ml_ct_ss_hna, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_dss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 6);
else
    nargoutchk(0, 2);
end

% Fill the D matrix.
if isempty(sys.D)
    sys.D = zeros(size(sys.C, 1), size(sys.B, 2));
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'gdlyapdlopts')
    assert(isa(opts.gdlyapdlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.gdlyapdlopts has to be a struct!');
else
    opts.gdlyapdlopts = struct();
end

if ml_field_set_to_value(opts, 'hankeldecopts')
    assert(isa(opts.hankeldecopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.hankeldecopts has to be a struct!');
else
    opts.hankeldecopts = struct();
end

if ml_field_set_to_value(opts, 'lyapdlopts')
    assert(isa(opts.lyapdlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.lyapdlopts has to be a struct!');
else
    opts.lyapdlopts = struct();
end

opts = ml_check_cell_param(opts, 'MinRelTol', ...
    @ml_assert_nonnegscalar, log(size(sys.A, 1)) * eps);

opts = ml_check_cell_param(opts, 'OrderComputation', ...
    @ml_assert_char, 'tolerance');
numOrderComp = length(opts.OrderComputation);
tselect      = cell(1, numOrderComp);
for k = 1:numOrderComp
    if strcmpi(opts.OrderComputation{k}, 'order')
        tselect{k} = 0;
    elseif strcmpi(opts.OrderComputation{k}, 'tolerance')
        tselect{k} = 3;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into unstable, finite and infinite parts.
[sys, infoADTF] = ml_ct_dss_adtf(sys, opts);

ninf = size(sys.Ainf, 1);

% Assign information about additive decomposition.
info.infoADTF = infoADTF;


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE MINIMAL, BALANCED REALIZATION.                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computation of the full-rank factors of the proper Gramians.
if size(sys.A, 1) > 0
    [R, L, infoLYAPDL] = ml_lyapdl_sgn_fac(sys.A, sys.B, sys.C, sys.E, ...
        opts.lyapdlopts);
    
    % Assign information about Lyapunov equation solver.
    info.infoLYAPDL = infoLYAPDL;
else
    [R, L] = deal([]);
end

% Computation of the full-rank factors of the improper Gramians.
if ninf > 0
    [Rinf, Linf, infoGDLYAPDL] = ml_gdlyapdl_smith_fac( ...
        sys.Ainf, sys.Binf, sys.Cinf, sys.Einf, opts.gdlyapdlopts);
    
    % Assign information about improper Lyapunov equation solver.
    info.infoGDLYAPDL = infoGDLYAPDL;
else
    [Rinf, Linf] = deal([]);
end

if ml_field_set_to_value(opts, 'Tolerance')
    tmp = opts.Tolerance;
else
    tmp = [];
end
opts.Tolerance    = opts.MinRelTol;
[sys, hsvp, hsvi] = ml_dss_sr(sys, R, Rinf, L, Linf, 1, opts);
opts.Tolerance    = tmp;

if not(isa(sys, 'cell'))
    sys = {sys};
end

opts.noUnpack = 1;
nh            = ml_order(hsvp, size(sys{1}.Au, 1), ...
    size(sys{1}.Ainf, 1), tselect, opts);
[sysFull, nh] = ml_extend_cell(sys, nh);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TRANSFORMATION TO REDUCED-ORDER MODEL.                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

numSys    = length(nh);
roms      = cell(1, numSys);
sigmaFull = cell(1, numSys);
nhFull    = nh;

for r = 1:numSys
    nh  = nhFull{r};
    sys = sysFull{r};
    nf  = size(sys.A, 1);
    
    if nh < nf
        sigmaFull{r} = hsvp(nh + 1);
        sigma        = sigmaFull{r};
        
        % Get multiplicity of the chosen Hankel singular value.
        k = 1;
        
        while k < nf - nh
            if abs(sigma - hsvp(nh + 1 + k)) > sigma * log(nf) * sqrt(eps)
                break;
            end
            
            k = k + 1;
        end
        
        % Permutation of the system.
        if nh + k < nf
            p = [1:nh, nh+k+1:nf, nh+1:nh+k];
            
            sys.A = sys.A(p,p);
            sys.B = sys.B(p, :);
            sys.C = sys.C(: , p);
            hsvh  = hsvp(p);
        else
            hsvh = hsvp(1:nf);
        end
        
        % Transformation to an all-pass error system.
        nk = nf - k;
        
        U = sigma * pinv(sys.C(: , nk+1:nf)') * sys.B(nk+1:nf, :);
        S = sparse(1:nk, 1:nk, hsvh(1:nk));
        A = sys.A(1:nk, 1:nk);
        B = sys.B(1:nk, :);
        C = sys.C(: , 1:nk);
        
        rom = struct( ...
            'A', sigma^2 * A' + S * A * S + C' * U * B', ...
            'B', S * B - C' * U, ...
            'C', C * S - U * B', ...
            'D', sys.D + U, ...
            'E', diag(hsvh(1:nk).^2 - sigma^2));
        
        % Additive decomposition.
        hankelopts                       = struct();
        hankelopts.infdecopts.Dimension  = 0;
        hankelopts.stabdecopts           = opts.hankeldecopts;
        hankelopts.stabdecopts.Dimension = nk - nh;
        
        [rom, infoHAADTF] = ml_ct_dss_adtf(rom, hankelopts);
        
        % Assign information about additive decomposition.
        info.infoHAADTF = infoHAADTF;
    else
        sigmaFull{r} = 0;
        rom          = sys;
    end
    roms{r} = rom;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign output information.
absErrBound = cell(1, numSys);
for k = 1:numSys
    absErrBound{k} = 2 * sum(hsvp(nhFull{k}+1:end));
    % unstable parts of the original decomposition should be merged
    roms{k}.Au     = sysFull{k}.Au;
    roms{k}.Bu     = sysFull{k}.Bu;
    roms{k}.Cu     = sysFull{k}.Cu;
    roms{k}.Eu     = sysFull{k}.Eu;
    roms{k}.Ainf   = sysFull{k}.Ainf;
    roms{k}.Binf   = sysFull{k}.Binf;
    roms{k}.Cinf   = sysFull{k}.Cinf;
    roms{k}.Einf   = sysFull{k}.Einf;
end

if (ml_field_set_to_value(opts, 'ImproperTrunc') ...
        && isa(opts.ImproperTrunc, 'cell'))
    ni = cell(1, numSys);
    for k = 1:numSys
        ni{k} = size(roms{k}.Ainf, 1);
    end
else
    ni = size(roms{1}.Ainf, 1);
end

info.AbsErrBound = absErrBound;
info.Hsvi        = hsvi;
info.Hsvp        = hsvp;
info.Ni          = ni;
info.Np          = nhFull;
info.Nu          = size(roms{1}.Au, 1);
info.Sigma       = sigmaFull;

[varargout{1:nargout}] = ml_dss_format_output(roms, ioformat, info);
