function [varargout] = ml_ct_dss_mt(varargin)
%ML_CT_DSS_MT Modal truncation for descriptor systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_MT(A, B, C, D, E)
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_MT(A, B, C, D, E, opts)
%
%   [rom, info] = ML_CT_DSS_MT(sys)
%   [rom, info] = ML_CT_DSS_MT(sys, opts)
%
% DESCRIPTION:
%   This function computes the generalized modal truncation for a
%   descriptor system of the form
%
%       E*x'(t) = A*x(t) + B*u(t),                                      (1)
%          y(t) = C*x(t) + D*u(t).                                      (2)
%
%   Therefore, a block diagonalization of the matrix pencil s*E - A is
%   performed using the matrix disk function, such that
%
%            [ Ei 0  0  ]       [ Ai 0  0  ]
%       E2 = [ 0  E0 0  ], A2 = [ 0  A0 0  ],
%            [ 0  0  E1 ]       [ 0  0  A1 ]
%
%                                 [ Ci ]
%       B2 = [ Bi, B0, B1 ], C2 = [ C0 ],
%                                 [ C1 ]
%
%   where the matrix pencil s*Ei - Ai contains all infinite eigenvalues
%   and s*E0 - A0 the finite eigenvalues with the real part larger than a
%   given alpha. As result, the reduced-order system is given by
%
%       Er*x'(t) = Ar*x(t) + Br*u(t),                                   (3)
%           y(t) = Cr*x(t) + Dr*u(t),                                   (4)
%
%   with
%
%            [ E0  0 ]       [ A0  0 ]                        [ C0 ]
%       Er = [       ], A2 = [       ], B2 = [ B0, Bi ], C2 = [    ].
%            [ 0  Ei ]       [ 0  Ai ]                        [ Ci ]
%
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   E    - matrix from (1) with dimensions n x n
%   sys  - structure or state-space object, containing the descriptor
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Alpha           | real scalar, such that all finite eigenvalues with|
%   | {!}             | the real part smaller than Alpha are truncated    |
%   |                 | (default -1.0)                                    |
%   +-----------------+---------------------------------------------------+
%   | DecompEig       | positive scalar, overestimation of the absolute   |
%   |                 | value of the largest finite eigenvalue of s*E - A,|
%   |                 | if set, replaces the computation with DecompTol   |
%   |                 | (default [])                                      |
%   +-----------------+---------------------------------------------------+
%   | DecompTol       | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest singular value of E to determine the      |
%   |                 | smallest non-quasi-zero singular value of E       |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | signmopts       | structure, containing the optional parameters for |
%   |                 | the sign function method, see ml_signm            |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | infdecopts      | structure, containing the optional parameters for |
%   |                 | the decomposition of the finite and infinite parts|
%   |                 | of the system using the disk function and subspace|
%   |                 | extraction method, see ml_disk and ml_getqz       |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | RankTol         | nonnegative scalar, tolerance used for the        |
%   |                 | determination of deflating subspaces              |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | StoreProjection | {0, 1}, used to disable/enable storing of the     |
%   |                 | computed projection matrices W and T              |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%
%   Note: Parameters marked with {!} may also be a cell array containing
%         multiple arguments. In this case an cell array of the same size
%         is returned with one entry computed for each input argument and
%         the marked fields of the info struct are cells as well.
%         When multiple arguments are given as cells, they are expected to
%         have the same length.
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   Er   - matrix of (3) with dimensions r x r
%   rom  - structure or state-space object, containing the reduced-order
%          descriptor system:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | infoSIGNM       | structure, containing information about the sign  |
%   | {!}             | function method used for the modal truncation of  |
%   |                 | the system, see ml_signm                          |
%   +-----------------+---------------------------------------------------+
%   | infoADTF        | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | infinite and finite parts, see ml_ct_dss_adtf     |
%   +-----------------+---------------------------------------------------+
%   | N               | Dimension of the finite part in the reduced-order |
%   | {!}             | model                                             |
%   +-----------------+---------------------------------------------------+
%   | Ni              | Dimension of the infinite part in the reduced-    |
%   |                 | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | T               | projection matrix used as right state-space       |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%   | W               | projection matrix used as left state-space        |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   P. Benner, E. S. Quintana-Orti, Model reduction based on spectral
%   projection methods, in: P. Benner, V. Mehrmann, D. Sorensen (Eds.),
%   Dimension Reduction of Large-Scale Systems, Vol. 45 of Lect. Notes
%   Comput. Sci. Eng., Springer, Berlin/Heidelberg, Germany, 2005,
%   pp. 5--45. https://doi.org/10.1007/3-540-27909-1_1
%
% See also ml_ct_ss_mt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_dss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 6);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
opts = ml_check_cell_param(opts, 'Alpha', ...
    @ml_assert_scalar, -1.0);
[opts.Alpha, Inds] = sort(cell2mat(opts.Alpha));
numAlpha = length(opts.Alpha);

if ml_field_set_to_value(opts, 'signmopts')
    assert(isa(opts.signmopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.signmopts has to be a struct!');
else
    opts.signmopts = struct();
end

if ml_field_set_to_value(opts, 'StoreProjection')
    ml_assert_boolean(opts.StoreProjection, 'opts.StoreProjection');
else
    opts.StoreProjection = false;
end

% Initial info structure.
info           = struct();
info.N         = cell(1, numAlpha);
info.infoSIGNM = cell(1, numAlpha);

if opts.StoreProjection
    info.T = cell(1, numAlpha);
    info.W = cell(1, numAlpha);
else
    info.T = [];
    info.W = [];
end

roms = cell(1, numAlpha);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into finite and infinite parts.
opts.stabdecopts.Dimension = 0;
[sys, infoADTF]            = ml_ct_dss_adtf(sys, opts);

% Assign information about additive decomposition.
info.infoADTF = rmfield(infoADTF, {'T'; 'W'});

if opts.StoreProjection
    Iinf = eye(size(sys.Ainf));
end

for k = 1:numAlpha
    n = size(sys.A, 1);
    
    % Decomposition of desired and undesired eigenvalues.
    if n > 0
        % In vectorized mode this field could be set from a previous
        % iteration.
        opts.Dimension      = [];
        [Aspace, infoSIGNM] = ml_signm(sys.A - opts.Alpha(k) * sys.E, ...
            sys.E, opts.signmopts);
        AE1                 = Aspace + sys.E;
        AE2                 = Aspace - sys.E;
        [Q, ~, n1]          = ml_getqz(sys.A, sys.E, AE1, AE2, opts);
        
        if (n1 > 0) && (n1 < n)
            opts.Dimension = n - n1;
            [~, V, ns]     = ml_getqz(sys.A, sys.E, AE2, AE1, opts);
            
            sys.A = Q(: , n1+1:end)' * sys.A * V(: , 1:ns);
            sys.E = Q(: , n1+1:end)' * sys.E * V(: , 1:ns);
            sys.B = Q(: , n1+1:end)' * sys.B;
            sys.C = sys.C * V(: , 1:ns);
            
            if opts.StoreProjection
                if k == 1
                    info.T{k} = infoADTF.T * blkdiag(V(: , 1:ns), Iinf);
                    info.W{k} = blkdiag(Q(: , n1+1:end)', Iinf) ...
                        * infoADTF.W;
                else
                    info.T{k} = info.T{k-1} * blkdiag(V(: , 1:ns), Iinf);
                    info.W{k} = blkdiag(Q(: , n1+1:end)', Iinf) ...
                        * info.W{k-1};
                end
            end
        elseif n1 == n
            [sys.A, sys.B, sys.C, sys.E] = deal([]);
            
            if opts.StoreProjection
                info.T{k} = infoADTF.T * blkdiag(zeros(n, 0), Iinf);
                info.W{k} = blkdiag(zeros(0, n), Iinf) * infoADTF.W;
            end
        else
            if opts.StoreProjection
                info.T{k} = infoADTF.T;
                info.W{k} = infoADTF.W;
            end
        end
        
        % Assign information about the disk function.
        info.infoSIGNM{Inds(k)} = infoSIGNM;
    else
        if opts.StoreProjection
            info.T{k} = infoADTF.T * blkdiag(zeros(n, 0), Iinf);
            info.W{k} = blkdiag(zeros(0, n), Iinf) * infoADTF.W;
        end
    end
    
    roms{Inds(k)} = sys;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign output information.
for k = 1:numAlpha
    info.N{k} = size(roms{k}.A, 1);
end
info.Ni = size(sys.Ainf, 1);

[varargout{1:nargout}] = ml_dss_format_output(roms, ioformat, info);
