function [varargout] = ml_ct_ss_bst(varargin)
%ML_CT_SS_BST Balanced stochastic truncation for standard systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_BST(A, B, C, D)
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_BST(A, B, C, D, opts)
%
%   [rom, info] = ML_CT_SS_BST(sys)
%   [rom, info] = ML_CT_SS_BST(sys, opts)
%
% DESCRIPTION:
%   This function computes the balanced stochastic truncation for a
%   standard system of the form
%
%       x'(t) = A*x(t) + B*u(t),                                        (1)
%        y(t) = C*x(t) + D*u(t).                                        (2)
%
%   Therefore, first the Lyapunov equation
%
%       A*P  + P*A' + B*B' = 0,
%
%   is solved for the controllability Gramian P and then, the corresponding
%   Riccati equation
%
%       A'*Q + Q*A + (C - Bw' * Q)' * inv(D*D') * (C - Bw' * Q) = 0
%
%   is solved for the Gramian Q, with
%
%       Bw = B*D' + P*C'.
%
%   As result, a reduced-order system of the form
%
%       x'(t) = Ar*x(t) + Br*u(t),                                      (3)
%        y(t) = Cr*x(t) + Dr*u(t)                                       (4)
%
%   is computed, such that for the original transfer function G and and the
%   r-th order transfer function Gr it holds
%
%       ||G - Gr||_{\infty} / ||G||_{\infty}
%       <= ((1 + Hsv(r+1))/(1 - Hsv(r+1)) * ...
%          * (1 + Hsv(n))/(1 - Hsv(n))) + 1,
%
%   with Hsv, a vector containing the characteristic stochastic singular
%   values of the system. If the transfer function is invertible it holds
%
%       ||inv(G)*(G - Gr)||_{\infty}
%       <= ((1 + Hsv(r+1))/(1 - Hsv(r+1)) * ...
%          * (1 + Hsv(n))/(1 - Hsv(n))) + 1.
%
%   Notes:
%     1) The equations above are defined for the case of p < m. Otherwise
%        the system is transposed, then reduced and back transposed.
%     2) In case of a rank-deficient D term, an epsilon regularization is
%        performed, which replaces the D during the computations with an
%        identity matrix scaled by a given epsilon.
%     3) For unstable systems, first an additive decomposition into the
%        stable and anti-stable parts is performed and then only the
%        stable part will be reduced.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   sys  - structure or state-space object, containing the standard
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | caredlopts      | structure, containing the optional parameters for |
%   |                 | the Riccati equation sign function solver, only   |
%   |                 | used if RiccatiSolver = 'sign', see               |
%   |                 | ml_caredl_sgn_fac                                 |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Epsilon         | positive scalar, used in the case of a            |
%   |                 | non-full-rank D term for epsilon regularization   |
%   |                 | by multiplying with an identity matrix of         |
%   |                 | appropriate size                                  |
%   |                 | (default 1.0e-03)                                 |
%   +-----------------+---------------------------------------------------+
%   | lyapopts        | structure, containing the optional parameters for |
%   |                 | the computation of the continuous-time algebraic  |
%   |                 | Lyapunov equation, see ml_lyap_sgn_fac            |
%   +-----------------+---------------------------------------------------+
%   | Method          | character array, determining algorithm for the    |
%   | {!}             | computation of the reduced-order model            |
%   |                 |  'sr'   - square-root method                      |
%   |                 |  'bfsr' - balancing-free square-root method       |
%   |                 | (default 'sr')                                    |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   | {!}             | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsv)) + Nu)                |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using absolute error bound         |
%   |                 | (default 'tolerance')                             |
%   +-----------------+---------------------------------------------------+
%   | pcareopts       | structure, containing the optional parameters for |
%   |                 | the computation of the continuous-time algebraic  |
%   |                 | positive Riccati equation, only used if           |
%   |                 | RiccatiSolver = 'newton', see ml_pcare_nwt_fac    |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | RiccatiSolver   | character array, determining the solver for the   |
%   |                 | dual Riccati equations                            |
%   |                 |  'newton' - Newton iteration                      |
%   |                 |  'sign'   - dual sign function method             |
%   |                 | (default 'sign')                                  |
%   +-----------------+---------------------------------------------------+
%   | stabsignmopts   | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, see ml_signm                               |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | stabsylvopts    | structure, containing the optional parameters for |
%   |                 | the Sylvester equation solver used for the        |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, see ml_sylv_sgn                            |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | StoreProjection | {0, 1}, used to disable/enable storing of the     |
%   |                 | computed projection matrices W and T              |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 | by an relative error bound if 'tolerance' is set  |
%   |                 | for OrderComputation                              |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%   | UnstabDim       | integer, dimension of the deflating anti-stable   |
%   |                 | subspace, negative if unknown                     |
%   |                 | (default -1)                                      |
%   +-----------------+---------------------------------------------------+
%
%   Note: Parameters marked with {!} may also be a cell array containing
%         multiple arguments. In this case an cell array of the same size
%         is returned with one entry computed for each input argument and
%         the marked fields of the info struct are cells as well.
%         When multiple arguments are given as cells, they are expected to
%         have the same length.
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   rom  - structure or state-space object, with the following entries:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Hsv             | a vector, containing the computed characteristic  |
%   |                 | stochastic singular values                        |
%   +-----------------+---------------------------------------------------+
%   | infoADTF        | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | stable and anti-stable parts, see ml_ct_ss_adtf   |
%   +-----------------+---------------------------------------------------+
%   | infoCAREDL      | structure, containing information about the       |
%   |                 | sign function solver for the dual Riccati         |
%   |                 | equations, see ml_caredl_sgn_fac                  |
%   +-----------------+---------------------------------------------------+
%   | infoLYAP        | structure, containing information about the       |
%   |                 | continuous-time Lyapunov equation sovler for the  |
%   |                 | controllability Gramian, see ml_lyap_sgn_fac      |
%   +-----------------+---------------------------------------------------+
%   | infoPCARE       | structure, containing information about the       |
%   |                 | continuous-time algebraic positive Riccati        |
%   |                 | equation for the observability Gramian, see       |
%   |                 | ml_pcare_nwt_fac                                  |
%   +-----------------+---------------------------------------------------+
%   | Ns              | Dimension of the stable part of the reduced-order |
%   | {!}             | model                                             |
%   +-----------------+---------------------------------------------------+
%   | Nu              | Dimension of the anti-stable part of the reduced- |
%   |                 | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | RelErrBound     | computed error bound for the relative error of the|
%   | {!}             | of the reduced-order model in H-infinity norm     |
%   +-----------------+---------------------------------------------------+
%   | T               | projection matrix used as right state-space       |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%   | W               | projection matrix used as left state-space        |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   P. Benner, E. S. Quintana-Orti, G. Quintana-Orti, Efficient numerical
%   algorithms for balanced stochastic truncation, Int. J. Appl. Math.
%   Comput. Sci. 11 (5) (2001) 1123--1150.
%
% See also ml_ct_dss_bst, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 5);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'caredlopts')
    assert(isa(opts.caredlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.caredlopts has to be a struct!');
else
    opts.caredlopts = struct();
end

if ml_field_set_to_value(opts, 'Epsilon')
    ml_assert_posscalar(opts.Epsilon, 'opts.Epsilon');
else
    opts.Epsilon = 1.0e-03;
end

if ml_field_set_to_value(opts, 'lyapopts')
    assert(isa(opts.lyapopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.lyapopts has to be a struct!');
else
    opts.lyapopts = struct();
end

opts       = ml_check_cell_param(opts, 'Method', @ml_assert_char, 'sr');
numMethods = length(opts.Method);
mor_method = cell(1, numMethods);
for k = 1:numMethods
    method = opts.Method{k};
    if strcmpi(method, 'bfsr')
        mor_method{k} = @ml_ss_bfsr;
    elseif strcmpi(method, 'sr')
        mor_method{k} = @ml_ss_sr;
    else
        error('MORLAB:data', ...
            'The given computation method is not implemented!');
    end
end

opts = ml_check_cell_param(opts, 'OrderComputation', ...
    @ml_assert_char, 'tolerance');
numOrderComp = length(opts.OrderComputation);
tselect      = cell(1, numOrderComp);
for k = 1:numOrderComp
    if strcmpi(opts.OrderComputation{k}, 'order')
        tselect{k} = 0;
    elseif strcmpi(opts.OrderComputation{k}, 'tolerance')
        tselect{k} = 4;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
end

if ml_field_set_to_value(opts, 'pcareopts')
    assert(isa(opts.pcareopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.pcareopts has to be a struct!');
else
    opts.pcareopts = struct();
end

if ml_field_set_to_value(opts, 'RiccatiSolver')
    assert(strcmpi(opts.RiccatiSolver, 'newton') ...
        || strcmpi(opts.RiccatiSolver, 'sign'), ...
        'MORLAB:data', ...
        'The desired Riccati equation solver is not implemented!');
else
    opts.RiccatiSolver = 'sign';
end

if ml_field_set_to_value(opts, 'StoreProjection')
    ml_assert_boolean(opts.StoreProjection, 'opts.StoreProjection');
else
    opts.StoreProjection = false;
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into the stable and anti-stable parts.
[sys, infoADTF] = ml_ct_ss_adtf(sys, opts);

% Assign information about additive decomposition.
info.infoADTF = rmfield(infoADTF, {'T'; 'W'});


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SOLVE MATRIX EQUATIONS.                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

m = size(sys.B, 2);
p = size(sys.C, 1);

% Transpose system, if there are more outputs than inputs.
if m < p
    sys            = structfun(@ctranspose, sys, 'UniformOutput', 0);
    [sys.C, sys.B] = deal(sys.B, sys.C);
end

% Computation of the low-rank factors of the system Gramians.
if size(sys.A, 1) > 0
    [R, infoLYAP] = ml_lyap_sgn_fac(sys.A, sys.B, [], opts.lyapopts);
    
    % Assign information about Lyapunov solver.
    info.infoLYAP  = infoLYAP;
    
    % Epsilon regularization.
    if (rank(sys.D) < min(m, p)) || (norm(sys.D, 'fro') < 1.0e+02 * eps)
        D  = opts.Epsilon * eye(min(m, p));
        Bw = sys.B * opts.Epsilon * eye(min(m, p), max(m, p))' + ...
            R * (R' * sys.C');
    else
        Bw     = sys.B * sys.D' + R * (R' * sys.C');
        [~, D] = qr(sys.D', 0);
    end
    
    switch lower(opts.RiccatiSolver)
        case 'newton'
            [L, infoPCARE] = ml_pcare_nwt_fac(sys.A - Bw * ((D' * D) ...
                \ sys.C), Bw / D, (D' \ sys.C), [], opts.pcareopts);
            
            % Assign information about Riccati solver.
            info.infoPCARE = infoPCARE;
            
        case 'sign'
            % Compute the two system Gramians.
            opts.caredlopts.EqnType = 'primal';
            [L, ~, infoCAREDL]      = ml_caredl_sgn_fac(sys.A ...
                - Bw * ((D' * D) \ sys.C), Bw / D, (D' \ sys.C), [], 1, ...
                opts.caredlopts);
            
            % Assign information about Riccati equation solver.
            info.infoCAREDL = infoCAREDL;
    end
else
    [R, L] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE REDUCED-ORDER MODEL.                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if numMethods == 1
    [roms, hsv, T, W] = mor_method{1}(sys, R, L, tselect, opts);
    if not(isa(roms, 'cell'))
        roms = {roms};
        T    = {T};
        W    = {W};
    end
else
    roms = cell(1, numMethods);
    T    = cell(1, numMethods);
    W    = cell(1, numMethods);
    for k = 1:numMethods
        opts.vecInd = k;
        opts.vecMax = numMethods;
        [roms{k}, hsv, T{k}, W{k}] = mor_method{k}(sys, R, L, tselect, ...
            opts);
    end
end

% Transpose system back, if it was transposed before.
if m < p
    for k = 1:length(roms)
        roms{k}                = structfun(@ctranspose, roms{k}, ... 
            'UniformOutput', 0);
        [roms{k}.C, roms{k}.B] = deal(roms{k}.B, roms{k}.C);
        [T{k}, W{k}]           = deal(W{k}', T{k}');
    end
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign output information.
numSys      = length(roms);
n           = cell(1, numSys);
relErrBound = cell(1, numSys);
for k = 1:numSys
    n{k}           = size(roms{k}.A, 1);
    relErrBound{k} = prod((1 + hsv(n{k}+1:end)) ...
        ./ (1 - hsv(n{k}+1:end))) - 1;
end

info.Hsv         = hsv;
info.Ns          = n;
info.Nu          = size(sys.Au, 1);
info.RelErrBound = relErrBound;

if opts.StoreProjection
    for k = 1:numSys
        info.T{k} = infoADTF.T * blkdiag(T{k}, eye(info.Nu));
        info.W{k} = blkdiag(W{k}, eye(info.Nu)) * infoADTF.W;
    end
else
    info.T = [];
    info.W = [];
end

[varargout{1:nargout}] = ml_ss_format_output(roms, ioformat, info);
