function [varargout] = ml_ct_ss_flbt(varargin)
%ML_CT_SS_FLBT Frequency-limited balanced truncation for standard systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_FLBT(A, B, C, D)
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_FLBT(A, B, C, D, opts)
%
%   [rom, info] = ML_CT_SS_FLBT(sys)
%   [rom, info] = ML_CT_SS_FLBT(sys, opts)
%
% DESCRIPTION:
%   This function computes the frequency-limited balanced truncation for a
%   standard system of the form
%
%       x'(t) = A*x(t) + B*u(t),                                        (1)
%        y(t) = C*x(t) + D*u(t).                                        (2)
%
%   Therefore, the two standard Lyapunov equations
%
%       A*P  + P*A' + BF*B' + B*BF' = 0,
%       A'*Q + Q*A  + CF'*C + C'*CF = 0,
%
%   where BF and CF are frequency-dependent matrices, are solved for the
%   frequency-limited Gramians P and Q. As result, a reduced-order
%   system of the form
%
%       x'(t) = Ar*x(t) + Br*u(t),                                      (3)
%        y(t) = Cr*x(t) + Dr*u(t)                                       (4)
%
%   is computed. For enforcing stability in the reduced-order model, the
%   modified Gramian approach can be used, which also gives a global error
%   bound of the form
%
%       ||G - Gr||_{\infty} <= 2*||JB||*||JC||(Hsv(r+1) + ... + Hsv(n)),
%
%   with Hsv, a vector containing the freuqency-limited Hankel singular
%   values of the system.
%
%   Note: For unstable systems, first an additive decomposition into the
%         stable and anti-stable parts is performed and then only the
%         stable part will be reduced.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   sys  - structure or state-space object, containing the standard
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | FreqRange       | nonnegative vector, frequency intervals such that |
%   |                 | [w(1), w(2)] ... [w(2k-1), w(2k)] are approximated|
%   |                 | (default [0, 1.0e+03])                            |
%   +-----------------+---------------------------------------------------+
%   | lyapdlopts      | structure, containing the optional parameters for |
%   |                 | the computation of the generalized continuous-time|
%   |                 | Lyapunov equations, see ml_lyapdl_sgn_ldl if      |
%   |                 | ModGramian = 0 and ml_lyapdl_sgn_fac if           |
%   |                 | ModGramian = 1                                    |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Method          | character array, determining algorithm for the    |
%   | {!}             | computation of the reduced-order model            |
%   |                 |  'sr'   - square-root method                      |
%   |                 |  'bfsr' - balancing-free square-root method       |
%   |                 | (default 'sr')                                    |
%   +-----------------+---------------------------------------------------+
%   | ModGramian      | {0, 1}, used to disable/enable the modified       |
%   |                 | Gramian approach                                  |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   | {!}             | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsv)) + Nu)                |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using rel. tolerance for the hsv   |
%   |                 |  'sum'       - using rel. tolerance for sum of hsv|
%   |                 | (default 'sum')                                   |
%   +-----------------+---------------------------------------------------+
%   | stabsignmopts   | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, see ml_signm                               |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | stabsylvopts    | structure, containing the optional parameters for |
%   |                 | the Sylvester equation solver used for the        |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, see ml_sylv_sgn                            |
%   +-----------------+---------------------------------------------------+
%   | StoreProjection | {0, 1}, used to disable/enable storing of the     |
%   |                 | computed projection matrices W and T              |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 | if 'tolerance' or 'sum' is set for                |
%   |                 | OrderComputation                                  |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%   | UnstabDim       | integer, dimension of the deflating anti-stable   |
%   |                 | subspace, negative if unknown                     |
%   |                 | (default -1)                                      |
%   +-----------------+---------------------------------------------------+
%
%   Note: Parameters marked with {!} may also be a cell array containing
%         multiple arguments. In this case an cell array of the same size
%         is returned with one entry computed for each input argument and
%         the marked fields of the info struct are cells as well.
%         When multiple arguments are given as cells, they are expected to
%         have the same length.
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   rom  - structure or state-space object, with the following entries:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | AbsErrBound     | computed error bound for the absolute error of the|
%   | {!}             | reduced-order model in H-infinity norm, only for  |
%   |                 | the modified Gramian approach                     |
%   +-----------------+---------------------------------------------------+
%   | Hsv             | a vector, containing the computed Hankel singular |
%   |                 | values                                            |
%   +-----------------+---------------------------------------------------+
%   | infoADTF        | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | stable and anti-stable parts, see ml_ct_ss_adtf   |
%   +-----------------+---------------------------------------------------+
%   | infoLYAPDL      | structure, containing information about the       |
%   |                 | continuous-time dual Lyapunov equations solver,   |
%   |                 | see ml_lyapdl_sgn_ldl or ml_lyapdl_sgn_fac        |
%   +-----------------+---------------------------------------------------+
%   | Ns              | Dimension of the stable part of the reduced-order |
%   | {!}             | model                                             |
%   +-----------------+---------------------------------------------------+
%   | Nu              | Dimension of the anti-stable part of the reduced- |
%   |                 | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | T               | projection matrix used as right state-space       |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%   | W               | projection matrix used as left state-space        |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   P. Benner, P. Kurschner, J. Saak, Frequency-limited balanced truncation
%   with low-rank approximations, SIAM J. Sci. Comput. 38 (1) (2016)
%   A471--A499. https://doi.org/ 10.1137/15M1030911
%
% See also ml_ct_ss_bt, ml_ct_dss_flbt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 5);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'FreqRange')
    ml_assert_nonnegvector(opts.FreqRange, 'opts.FreqRange');
    
    assert(mod(length(opts.FreqRange), 2) == 0, ...
        'MORLAB:data', ...
        'The parameter opts.FreqRange has to be an even vector!');
    
    opts.FreqRange = sort(opts.FreqRange);
else
    opts.FreqRange = [0, 1.0e+03];
end

if ml_field_set_to_value(opts, 'lyapdlopts')
    assert(isa(opts.lyapdlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.lyapdlopts has to be a struct!');
else
    opts.lyapdlopts = struct();
end

opts       = ml_check_cell_param(opts, 'Method', @ml_assert_char, 'sr');
numMethods = length(opts.Method);
mor_method = cell(1, numMethods);
for k = 1:numMethods
    method = opts.Method{k};
    if strcmpi(method, 'bfsr')
        mor_method{k} = @ml_ss_bfsr;
    elseif strcmpi(method, 'sr')
        mor_method{k} = @ml_ss_sr;
    else
        error('MORLAB:data', ...
            'The given computation method is not implemented!');
    end
end

if ml_field_set_to_value(opts, 'ModGramian')
    ml_assert_boolean(opts.ModGramian, 'opts.ModGramian');
else
    opts.ModGramian = 0;
end

opts = ml_check_cell_param(opts, 'OrderComputation', ...
    @ml_assert_char, 'tolerance');
numOrderComp = length(opts.OrderComputation);
tselect      = cell(1, numOrderComp);
for k = 1:numOrderComp
    if strcmpi(opts.OrderComputation{k}, 'order')
        tselect{k} = 0;
    elseif strcmpi(opts.OrderComputation{k}, 'tolerance')
        tselect{k} = 1;
    elseif strcmpi(opts.OrderComputation{k}, 'sum')
        tselect{k} = 2;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
end

if ml_field_set_to_value(opts, 'StoreProjection')
    ml_assert_boolean(opts.StoreProjection, 'opts.StoreProjection');
else
    opts.StoreProjection = false;
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into the stable and anti-stable parts.
[sys, infoADTF] = ml_ct_ss_adtf(sys, opts);

% Assign information about additive decomposition.
info.infoADTF = rmfield(infoADTF, {'T'; 'W'});


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SOLVE MATRIX EQUATIONS.                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = size(sys.A, 1);
m = size(sys.B, 2);
p = size(sys.C, 1);
E = eye(n);

% Computation of the full-rank factors of the proper Gramians.
if n > 0
    % Compute frequency-dependent right-hand side.
    if opts.FreqRange(1) == 0
        F      = -sys.A - 1i * opts.FreqRange(2) * eye(n);
        kstart = 2;
    else
        F      = eye(n);
        kstart = 1;
    end
    
    for k = kstart:1:length(opts.FreqRange)/2
        F = F * ((sys.A + 1i * opts.FreqRange(2*k-1) * E) ...
            \ (sys.A + 1i * opts.FreqRange(2*k) * E));
    end
    
    F  = (1i / pi) * logm(F);
    BF = real(F * sys.B);
    CF = real(sys.C * F);
    BQ = [zeros(m), eye(m); eye(m), zeros(m)];
    CQ = [zeros(p), eye(p); eye(p), zeros(p)];
    
    if opts.ModGramian
        % Modified Gramian approach.
        [BF, Y] = ml_compress_ldl([sys.B, BF], BQ, eps, 'col');
        Y       = abs(diag(Y));
        JB      = norm(diag(1 ./ sqrt(Y)) * BF' * sys.B, 2);
        BF      = BF * diag(sqrt(Y));
        
        [CF, Y] = ml_compress_ldl([sys.C; CF], CQ, eps, 'row');
        Y       = abs(diag(Y));
        JC      = norm(sys.C * CF' * diag(1 ./ sqrt(Y)), 2);
        CF      = diag(sqrt(Y)) * CF;
        
        [R, L, infoLYAPDL] = ml_lyapdl_sgn_fac(sys.A, BF, CF, [], ...
            opts.lyapdlopts);
    else
        % Classical method.
        [R, RY, L, LY, infoLYAPDL] = ml_lyapdl_sgn_ldl(sys.A, ...
            [sys.B, BF], BQ, [sys.C; CF], CQ, [], opts.lyapdlopts);
        
        RY  = diag(RY);
        idx = (RY > 0);
        R   = R(:, idx) * diag(sqrt(RY(idx)));
        
        LY  = diag(LY);
        idx = (LY > 0);
        L   = L(:, idx) * diag(sqrt(LY(idx)));
    end
    
    % Assig information about Lyapunov equation solver.
    info.infoLYAPDL = infoLYAPDL;
else
    [R, L] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE REDUCED-ORDER MODEL.                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if numMethods == 1
    [roms, hsv, T, W] = mor_method{1}(sys, R, L, tselect, opts);
    if not(isa(roms, 'cell'))
        roms = {roms};
        T    = {T};
        W    = {W};
    end
else
    roms = cell(1, numMethods);
    T    = cell(1, numMethods);
    W    = cell(1, numMethods);
    for k = 1:numMethods
        opts.vecInd = k;
        opts.vecMax = numMethods;
        [roms{k}, hsv, T{k}, W{k}] = mor_method{k}(sys, R, L, tselect, ...
            opts);
    end
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign output information.
numSys      = length(roms);
n           = cell(1, numSys);
for k = 1:numSys
    n{k} = size(roms{k}.A, 1);
end
if opts.ModGramian
    absErrBound = cell(1, numSys);
    for k = 1:numSys
        absErrBound{k} = 2 * JB * JC * sum(hsv(n{k}+1:end));
    end
    info.AbsErrBound = absErrBound;
end

info.Hsv = hsv;
info.Ns  = n;
info.Nu  = size(sys.Au, 1);

if opts.StoreProjection
    for k = 1:numSys
        info.T{k} = infoADTF.T * blkdiag(T{k}, eye(info.Nu));
        info.W{k} = blkdiag(W{k}, eye(info.Nu)) * infoADTF.W;
    end
else
    info.T = [];
    info.W = [];
end

[varargout{1:nargout}] = ml_ss_format_output(roms, ioformat, info);
