function [varargout] = ml_ct_ss_hinfbt(varargin)
%ML_CT_SS_HINFBT H-infinity balanced truncation for standard systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_HINFBT(A, B, C, D)
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_HINFBT(A, B, C, D, opts)
%
%   [rom, info] = ML_CT_SS_HINFBT(sys)
%   [rom, info] = ML_CT_SS_HINFBT(sys, opts)
%
% DESCRIPTION:
%   This function computes the H-infinity balanced truncation for a
%   standard system of the form
%
%       x'(t) = A*x(t) + B*u(t),                                        (1)
%        y(t) = C*x(t) + D*u(t).                                        (2)
%
%   Therefor, the two algebraic Riccati equations
%
%       A*P  + P*A' + B*B'
%           - beta * (P*C' + B*D') * inv(Rb) * (P*C' + B*D')' = 0,
%       A'*Q + Q*A  + C'*C
%           - beta * (B'*Q + D'*C)' * inv(Rc) * (B'*Q + D'*C) = 0,
%
%   are solved for the Gramians P and Q, with beta = (1 - GAM^(-2)) and
%
%       Rb = I + D*D',
%       Rc = I + D'*D.
%
%   As result, a reduced-order system of the form
%
%       x'(t) = Ar*x(t) + Br*u(t),                                      (3)
%        y(t) = Cr*x(t) + Dr*u(t)                                       (4)
%
%   is computed, such that for the original transfer function G with the
%   (right) coprime factorization sqrt(beta)*G = N*inv(M) and the r-th
%   order transfer function Gr with the (right) coprime factorization
%   sqrt(beta)*Gr = Nr*inv(Mr) it holds
%
%       1/sqrt(beta) * ||[N; M] - [Nr; Mr]||_{\infty} <=
%           2 * (Hsv(r+1) / sqrt(1 + (beta*Hsv(r+1))^2) + ...
%           + Hsv(n)/sqrt(1 + (beta*Hsv(n))^2)),
%
%   with Hsv, a vector containing the characteristic H-infinity singular
%   values of the system.
%
%   Note:
%     1) The parameter GAM refers to the optimal cost gamma computed during
%        the H-infinity controller construction of the LQG normalized
%        system with feed-through term
%
%           [ A | B 0 B ]
%           [ --------- ]
%           [ C | 0 0 0 ],
%           [ 0 | 0 0 I ]
%           [ C | 0 I D ]
%
%        which can be constructed by
%
%           sys_normal = ss(A, ...
%                           [B, zeros(n,p), B], ...
%                           [C; zeros(m,n); C], ...
%                           [zeros(p,m), zeros(p,p), zeros(p,m);
%                            zeros(m,m), zeros(m,p), eye(m,m);
%                            zeros(p,m), eye(p,p), D]).
%
%        Then an optimal parameter GAM_OPT can be computed by
%
%           [K, CL, GAM_OPT, INFO] = hinfsyn(sys_normal, p, m).
%
%        In this case, the parameter GAM > GAM_OPT can be chosen
%        arbitrarily.
%        By default, the maximum of 1 and the given GAM is disturbed by
%        0.01% before used for computations.
%     2) In the case of GAM == Inf, the H-infinity balanced truncation is
%        identical to the LQG balanced truncation.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   sys  - structure or state-space object, containing the standard
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Beta            | nonnegative scalar, used as shift of the          |
%   |                 | in Bass' algorithm for better conditioning if     |
%   |                 | StabMethod == 'lyap' is chosen only used if       |
%   |                 | RiccatiSolver = 'newton'                          |
%   |                 | (default 0.1)                                     |
%   +-----------------+---------------------------------------------------+
%   | caredlopts      | structure, containing the optional parameters for |
%   |                 | the Riccati equation sign function solver, only   |
%   |                 | used if RiccatiSolver = 'sign', see               |
%   |                 | ml_caredl_sgn_fac                                 |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | careopts        | structure, containing the optional parameters for |
%   |                 | the computation of the continuous-time algebraic  |
%   |                 | Riccati equations, only used if                   |
%   |                 | RiccatiSolver = 'newton', see ml_care_nwt_fac     |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, used for the scaling of the      |
%   |                 | Riccati equations, see GAM above                  |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | Method          | character array, determining algorithm for the    |
%   | {!}             | computation of the reduced-order model            |
%   |                 |  'sr'   - square-root method                      |
%   |                 |  'bfsr' - balancing-free square-root method       |
%   |                 | (default 'sr')                                    |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   | {!}             | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsv)) + Nu)                |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using absolute error bound         |
%   |                 | (default 'tolerance')                             |
%   +-----------------+---------------------------------------------------+
%   | RiccatiSolver   | character array, determining the solver for the   |
%   |                 | dual Riccati equations                            |
%   |                 |  'newton' - Newton iteration                      |
%   |                 |  'sign'   - dual sign function method             |
%   |                 | (default 'sign')                                  |
%   +-----------------+---------------------------------------------------+
%   | StabMethod      | character array, determining the method of        |
%   |                 | stabilization of the system, only used if         |
%   |                 | RiccatiSolver = 'newton'                          |
%   |                 |   'cabe'  - partial stabilization with the        |
%   |                 |            algebraic Bernoulli equation           |
%   |                 |   'lyap' - partial stabilization with Bass'       |
%   |                 |            algorithm                              |
%   |                 | (default 'cabe')                                  |
%   +-----------------+---------------------------------------------------+
%   | stabmethodopts  | structure, containing the optional parameters for |
%   |                 | the sign function based Lyapunov or Bernoulli     |
%   |                 | equation solver used for the stabilization, only  |
%   |                 | used if RiccatiSolver = 'newton', see ml_cabe_sgn |
%   |                 | or ml_lyap_sgn                                    |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | stabsignmopts   | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, only used if RiccatiSolver = 'newton',     |
%   |                 | see ml_signm                                      |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | StoreProjection | {0, 1}, used to disable/enable storing of the     |
%   |                 | computed projection matrices W and T              |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 | by an absolute error bound if 'tolerance' is set  |
%   |                 | for OrderComputation                              |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%   | UnstabDim       | integer, dimension of the deflating anti-stable   |
%   |                 | subspace in the control and filter Riccati        |
%   |                 | equations, only used if RiccatiSolver = 'newton', |
%   |                 | negative if unknown                               |
%   |                 | (default -1)                                      |
%   +-----------------+---------------------------------------------------+
%
%   Note: Parameters marked with {!} may also be a cell array containing
%         multiple arguments. In this case an cell array of the same size
%         is returned with one entry computed for each input argument and
%         the marked fields of the info struct are cells as well.
%         When multiple arguments are given as cells, they are expected to
%         have the same length.
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   rom  - structure or state-space object, with the following entries:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | AbsErrBound     | computed error bound for the absolute error of the|
%   | {!}             | (right) coprime factorization in H-infinity norm  |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, the disturbed scaling parameter  |
%   |                 | of the computations                               |
%   +-----------------+---------------------------------------------------+
%   | Hsv             | a vector, containing the computed characteristic  |
%   |                 | H-infinity singular values                        |
%   +-----------------+---------------------------------------------------+
%   | infoCARE_C      | structure, containing information about the       |
%   |                 | Newton solver for the regulator Riccati equation, |
%   |                 | see ml_care_nwt_fac                               |
%   +-----------------+---------------------------------------------------+
%   | infoCARE_O      | structure, containing information about the       |
%   |                 | Newton solver for the filter Riccati equation,    |
%   |                 | see ml_care_nwt_fac                               |
%   +-----------------+---------------------------------------------------+
%   | infoCAREDL      | structure, containing information about the       |
%   |                 | sign function solver for the dual Riccati         |
%   |                 | equations, see ml_caredl_sgn_fac                  |
%   +-----------------+---------------------------------------------------+
%   | infoPARTSTAB_C  | structure, containing information about the       |
%   |                 | partial stabilization used for the controllability|
%   |                 | Riccati equation, see ml_ct_ss_partstab           |
%   +-----------------+---------------------------------------------------+
%   | infoPARTSTAB_O  | structure, containing information about the       |
%   |                 | partial stabilization used for the observability  |
%   |                 | Riccati equation, see ml_ct_ss_partstab           |
%   +-----------------+---------------------------------------------------+
%   | N {!}           | Dimension of the reduced-order model              |
%   +-----------------+---------------------------------------------------+
%   | T               | projection matrix used as right state-space       |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%   | W               | projection matrix used as left state-space        |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   D. Mustafa, K. Glover, Controller reduction by Hinf-balanced
%   truncation, IEEE Trans. Autom. Control 36 (6) (1991) 668--682.
%   https://doi.org/10.1109/9.86941
%
% See also ml_ct_dss_hinfbt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 5);
else
    nargoutchk(0, 2);
end

% Fill the D matrix.
if isempty(sys.D)
    sys.D = zeros(size(sys.C, 1), size(sys.B, 2));
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'caredlopts')
    assert(isa(opts.caredlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.careopts has to be a struct!');
else
    opts.caredlopts = struct();
end

if ml_field_set_to_value(opts, 'careopts')
    assert(isa(opts.careopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.careopts has to be a struct!');
else
    opts.careopts = struct();
end

if ml_field_set_to_value(opts, 'Gamma')
    ml_assert_posscalar(opts.Gamma, 'opts.Gamma');
else
    opts.Gamma = Inf;
end

opts       = ml_check_cell_param(opts, 'Method', @ml_assert_char, 'sr');
numMethods = length(opts.Method);
mor_method = cell(1, numMethods);
for k = 1:numMethods
    method = opts.Method{k};
    if strcmpi(method, 'bfsr')
        mor_method{k} = @ml_ss_bfsr;
    elseif strcmpi(method, 'sr')
        mor_method{k} = @ml_ss_sr;
    else
        error('MORLAB:data', ...
            'The given computation method is not implemented!');
    end
end

opts = ml_check_cell_param(opts, 'OrderComputation', ...
    @ml_assert_char, 'tolerance');
numOrderComp = length(opts.OrderComputation);
tselect      = cell(1, numOrderComp);
for k = 1:numOrderComp
    if strcmpi(opts.OrderComputation{k}, 'order')
        tselect{k} = 0;
    elseif strcmpi(opts.OrderComputation{k}, 'tolerance')
        tselect{k} = 6;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
end

if ml_field_set_to_value(opts, 'RiccatiSolver')
    assert(strcmpi(opts.RiccatiSolver, 'newton') ...
        || strcmpi(opts.RiccatiSolver, 'sign'), ...
        'MORLAB:data', ...
        'The desired Riccati equation solver is not implemented!');
else
    opts.RiccatiSolver = 'sign';
end

if ml_field_set_to_value(opts, 'StoreProjection')
    ml_assert_boolean(opts.StoreProjection, 'opts.StoreProjection');
else
    opts.StoreProjection = false;
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARTIAL STABILIZATION AND SOLVE MATRIX EQUATIONS.                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Disturbance of scaling parameter.
opts.Gamma = 1.0001 * max(1.0, opts.Gamma);
gam        = sqrt(1 - 1 / opts.Gamma^2);

% Computation of matrices for Riccati equations.
Rb = eye(size(sys.C, 1)) + sys.D * sys.D';
Lb = chol(Rb);
Rc = eye(size(sys.B, 2)) + sys.D' * sys.D;
Lc = chol(Rc);

A = sys.A - sys.B * sys.D' * (Rb \ sys.C);
B = sys.B / Lc;
C = Lb' \ sys.C;

switch lower(opts.RiccatiSolver)
    case 'newton'
        % Partial stabilization.
        [K1, infoPARTSTAB_C] = ml_ct_ss_partstab(A', gam * C', opts);
        [K2, infoPARTSTAB_O] = ml_ct_ss_partstab(A, gam * B, opts);
        
        % Computation of the full-rank factors of the system Gramians.
        opts.careopts.K0 = K1;
        [R, infoCARE_C]  = ml_care_nwt_fac(A', gam * C', B', [], ...
            opts.careopts);
        
        opts.careopts.K0 = K2;
        [L, infoCARE_O]  = ml_care_nwt_fac(A, gam * B, C, [], ...
            opts.careopts);
        
        info.infoCARE_C     = infoCARE_C;
        info.infoCARE_O     = infoCARE_O;
        info.infoPARTSTAB_C = infoPARTSTAB_C;
        info.infoPARTSTAB_O = infoPARTSTAB_O;
        
    case 'sign'
        % Compute the two system Gramians.
        [L, R, infoCAREDL] = ml_caredl_sgn_fac(A, B, C, [], -gam^2, ...
            opts.caredlopts);
        
        info.infoCAREDL = infoCAREDL;
        
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE REDUCED-ORDER MODEL.                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if numMethods == 1
    [roms, hsv, T, W] = mor_method{1}(sys, R, L, tselect, opts);
    if not(isa(roms, 'cell'))
        roms = {roms};
        T    = {T};
        W    = {W};
    end
else
    roms = cell(1, numMethods);
    T    = cell(1, numMethods);
    W    = cell(1, numMethods);
    for k = 1:numMethods
        opts.vecInd = k;
        opts.vecMax = numMethods;
        [roms{k}, hsv, T{k}, W{k}] = mor_method{k}(sys, R, L, tselect, ...
            opts);
    end
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign output information.
numSys      = length(roms);
n           = cell(1, numSys);
absErrBound = cell(1, numSys);
for k = 1:numSys
    n{k}           = size(roms{k}.A, 1);
    absErrBound{k} = 2 * sum(hsv(n{k}+1:end) ...
        ./ sqrt(1+(gam*hsv(n{k}+1:end)).^2));
end

info.AbsErrBound = absErrBound;
info.Gamma       = opts.Gamma;
info.Hsv         = hsv;
info.N           = n;

if opts.StoreProjection
    for k = 1:numSys
        info.T{k} = T{k};
        info.W{k} = W{k};
    end
else
    info.T = [];
    info.W = [];
end

[varargout{1:nargout}] = ml_ss_format_output(roms, ioformat, info);
