function [varargout] = ml_dt_dss_bt(varargin)
%ML_DT_DSS_BT Balanced truncation for discrete-time descriptor systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, Er, info] = ML_DT_DSS_BT(A, B, C, D, E)
%   [Ar, Br, Cr, Dr, Er, info] = ML_DT_DSS_BT(A, B, C, D, E, opts)
%
%   [rom, info] = ML_DT_DSS_BT(sys)
%   [rom, info] = ML_DT_DSS_BT(sys, opts)
%
% DESCRIPTION:
%   This function computes the generalized balanced truncation for a
%   discrete-time descriptor system of the form
%
%       E*x(t+1) = A*x(t) + B*u(t),                                     (1)
%           y(t) = C*x(t) + D*u(t).                                     (2)
%
%   Therefore, first an additive decomposition of the system is performed
%   using the matrix disk function, such that
%
%            [ Ei  0 ]       [ Ai  0 ]                        [ Ci ]
%       E2 = [       ], A2 = [       ], B2 = [ Bi, Bp ], C2 = [    ],
%            [ 0  Ep ]       [ 0  Ap ]                        [ Cp ]
%
%   with (Ei, Ai, Bi, Ci, D) belonging to the polynomial part and
%   (Ep, Ap, Bp, Cp, 0) belonging to the strictly proper part.
%   Now, the two generalized discrete-time Lyapunov equations
%
%       Ap*Pp*Ap' - Ep*Pp*Ep' + Bp*Bp' = 0,
%       Ap'*Qp*Ap - Ep'*Qp*Ep + Cp'*Cp = 0
%
%   are solved for the reduction of the strictly proper part, and the two
%   generalized discrete-time Lyapunov equations
%
%       Ai*Pi*Ai' - Ei*Pi*Ei' - Bi*Bi' = 0,
%       Ai'*Qi*Ai - Ei'*Qi*Ei - Ci'*Ci = 0
%
%   are solved for the reduction of the polynomial part. As result, a
%   reduced-order system of the form
%
%       Er*x(t+1) = Ar*x(t) + Br*u(t),                                  (3)
%            y(t) = Cr*x(t) + Dr*u(t)                                   (4)
%
%   is computed, such that for the original transfer function G and the
%   reduced-order transfer function Gr with an r-th order strictly proper
%   part it holds
%
%       ||G - Gr||_{\infty} <= 2 * (Hsvp(r+1) + ... + Hsvp(n)),
%
%   with Hsvp, a vector containing the proper Hankel singular values of the
%   system.
%
%   Note: For unstable systems, an additional additive decomposition into
%         the stable and anti-stable parts is performed and then only the
%         stable part will be reduced.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   E    - matrix from (1) with dimensions n x n
%   sys  - structure or state-space object, containing the descriptor
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | DecompEig       | positive scalar, overestimation of the absolute   |
%   |                 | value of the largest finite eigenvalue of s*E - A,|
%   |                 | if set, replaces the computation with DecompTol   |
%   |                 | (default [])                                      |
%   +-----------------+---------------------------------------------------+
%   | DecompTol       | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest singular value of E to determine the      |
%   |                 | smallest non-quasi-zero singular value of E       |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | dlyapdlopts     | structure, containing the optional parameters for |
%   |                 | the computation of the discrete-time Lyapunov     |
%   |                 | equations, see ml_dlyapdl_smith_fac               |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | gdlyapdlopts    | structure, containing the optional parameters for |
%   |                 | the computation of the generalized discrete-time  |
%   |                 | Lyapunov equations, see ml_gdlyapdl_smith_fac     |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | ImproperTrunc   | nonnegative scalar, tolerance multiplied with the |
%   | {!}             | largest proper Hankel singular value of the       |
%   |                 | system to truncate the improper part              |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | Index           | nonnegative integer, index of the descriptor      |
%   |                 | system used to set an upper bound on the size of  |
%   |                 | the reduced improper part, Inf if unknown         |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | infdecopts      | structure, containing the optional parameters for |
%   |                 | the decomposition of the finite and infinite parts|
%   |                 | of the system using the disk function and subspace|
%   |                 | extraction method, see ml_disk and ml_getqz       |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Method          | character array, determining algorithm for the    |
%   | {!}             | computation of the reduced-order model            |
%   |                 |  'sr'   - square-root method                      |
%   |                 |  'bfsr' - balancing-free square-root method       |
%   |                 | (default 'sr')                                    |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   | {!}             | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsvp)) + Nu + Ni)          |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using absolute error bound         |
%   |                 | (default 'tolerance')                             |
%   +-----------------+---------------------------------------------------+
%   | stabdecopts     | structure, containing the optional parameters for |
%   |                 | the decomposition of the stable and unstable parts|
%   |                 | of the system using the sign function and subspace|
%   |                 | extraction method, see ml_signm and ml_getqz      |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | StoreProjection | {0, 1}, used to disable/enable storing of the     |
%   |                 | computed projection matrices W and T              |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   | {!}             | computation of the size of the reduced-order model|
%   |                 | by an absolute error bound if 'tolerance' is set  |
%   |                 | for OrderComputation                              |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
%   Note: Parameters marked with {!} may also be a cell array containing
%         multiple arguments. In this case an cell array of the same size
%         is returned with one entry computed for each input argument and
%         the marked fields of the info struct are cells as well.
%         When multiple arguments are given as cells, they are expected to
%         have the same length.
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   Er   - matrix of (3) with dimensions r x r
%   rom  - structure or state-space object, containing the reduced-order
%          descriptor system:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | AbsErrBound     | computed error bound for the absolute error of the|
%   | {!}             | reduced-order model in H-infinity norm            |
%   +-----------------+---------------------------------------------------+
%   | Hsvi            | a vector, containing the computed Hankel singular |
%   |                 | values of the improper part of the system         |
%   +-----------------+---------------------------------------------------+
%   | Hsvp            | a vector, containing the computed Hankel singular |
%   |                 | values of the proper part of the system           |
%   +-----------------+---------------------------------------------------+
%   | infoDLYAPDL     | structure, containing information about the       |
%   |                 | discrete-time dual Lyapunov equations solver,     |
%   |                 | see ml_dlyapdl_smith_fac                          |
%   +-----------------+---------------------------------------------------+
%   | infoADTF        | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | infinite, finite stable and finite anti-stable    |
%   |                 | parts, see ml_dt_dss_adtf                         |
%   +-----------------+---------------------------------------------------+
%   | infoGDLYAPDL    | structure, containing information about the       |
%   |                 | generalized discrete-time Lyapunov equation solver|
%   |                 | for the improper Gramians, see                    |
%   |                 | ml_gdlyapdl_smith_fac                             |
%   +-----------------+---------------------------------------------------+
%   | Ni              | Dimension of the improper part in the reduced-    |
%   | {!}             | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | Np              | Dimension of the proper part in the reduced-order |
%   | {!}             | model                                             |
%   +-----------------+---------------------------------------------------+
%   | Nu              | Dimension of the unstable part in the reduced-    |
%   |                 | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | T               | projection matrix used as right state-space       |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%   | W               | projection matrix used as left state-space        |
%   | {!}             | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   V. Mehrmann, T. Stykel, Balanced truncation model reduction for
%   large-scale systems in descriptor form, in: P. Benner, V. Mehrmann,
%   D. C. Sorensen (Eds.), Dimension Reduction of Large-Scale Systems,
%   Vol. 45 of Lect. Notes Comput. Sci. Eng., Springer-Verlag,
%   Berlin/Heidelberg, Germany, 2005, pp. 83--115.
%   https://doi.org/10.1007/3-540-27909-1_3
%
% See also ml_dt_ss_bt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_dss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 6);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'gdlyapdlopts')
    assert(isa(opts.gdlyapdlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.gdlyapdlopts has to be a struct!');
else
    opts.gdlyapdlopts = struct();
end

if ml_field_set_to_value(opts, 'dlyapdlopts')
    assert(isa(opts.dlyapdlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.dlyapdlopts has to be a struct!');
else
    opts.dlyapdlopts = struct();
end

opts       = ml_check_cell_param(opts, 'Method', @ml_assert_char, 'sr');
numMethods = length(opts.Method);
mor_method = cell(1, numMethods);
for k = 1:numMethods
    method = opts.Method{k};
    if strcmpi(method, 'bfsr')
        mor_method{k} = @ml_dss_bfsr;
    elseif strcmpi(method, 'sr')
        mor_method{k} = @ml_dss_sr;
    else
        error('MORLAB:data', ...
            'The given computation method is not implemented!');
    end
end

opts = ml_check_cell_param(opts, 'OrderComputation', ...
    @ml_assert_char, 'tolerance');
numOrderComp = length(opts.OrderComputation);
tselect      = cell(1, numOrderComp);
for k = 1:numOrderComp
    if strcmpi(opts.OrderComputation{k}, 'order')
        tselect{k} = 0;
    elseif strcmpi(opts.OrderComputation{k}, 'tolerance')
        tselect{k} = 3;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
end

if ml_field_set_to_value(opts, 'StoreProjection')
    ml_assert_boolean(opts.StoreProjection, 'opts.StoreProjection');
else
    opts.StoreProjection = false;
end

% Initial info structure.
info = struct();

% Save sampling time.
if isfield(sys, 'Ts')
    Ts    = sys.Ts;
    hasTs = 1;
else
    hasTs = 0;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into unstable, finite and infinite parts.
[sys, infoADTF] = ml_dt_dss_adtf(sys, opts);

ninf = size(sys.Ainf, 1);

% Assign information about additive decomposition.
info.infoADTF = rmfield(infoADTF, {'T'; 'W'});


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SOLVE MATRIX EQUATIONS.                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computation of the full-rank factors of the proper Gramians.
if size(sys.A, 1) > 0
    [R, L, infoDLYAPDL] = ml_dlyapdl_smith_fac(sys.A, sys.B, sys.C, ...
        sys.E, opts.dlyapdlopts);
    
    % Assig information about Lyapunov equation solver.
    info.infoDLYAPDL = infoDLYAPDL;
else
    [R, L] = deal([]);
end

% Computation of the full-rank factors of the improper Gramians.
if ninf > 0
    [Rinf, Linf, infoGDLYAPDL] = ml_gdlyapdl_smith_fac( ...
        sys.Ainf, sys.Binf, sys.Cinf, sys.Einf, opts.gdlyapdlopts);
    
    % Assign information about improper Lyapunov equation solver.
    info.infoGDLYAPDL = infoGDLYAPDL;
else
    [Rinf, Linf] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE REDUCED-ORDER MODEL.                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if numMethods == 1
    [roms, hsvp, hsvi, prj] = mor_method{1}(sys, R, Rinf, L, Linf, ...
        tselect, opts);
    
    T    = prj.T;
    Tinf = prj.Tinf;
    W    = prj.W;
    Winf = prj.Winf;
    
    if not(isa(roms, 'cell'))
        roms = {roms};
        T    = {T};
        Tinf = {Tinf};
        W    = {W};
        Winf = {Winf};
    end
else
    roms = cell(1, numMethods);
    T    = cell(1, numMethods);
    Tinf = cell(1, numMethods);
    W    = cell(1, numMethods);
    Winf = cell(1, numMethods);
    for k = 1:numMethods
        opts.vecInd = k;
        opts.vecMax = numMethods;
        [roms{k}, hsvp, hsvi, prj] = mor_method{k}(sys, R, Rinf, L, ...
            Linf, tselect, opts);
        
        T{k}    = prj.T;
        Tinf{k} = prj.Tinf;
        W{k}    = prj.W;
        Winf{k} = prj.Winf;
    end
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign output information.
numSys      = length(roms);
nf          = cell(1, numSys);
absErrBound = cell(1, numSys);
for k = 1:numSys
    nf{k}          = size(roms{k}.A, 1);
    absErrBound{k} = 2 * sum(hsvp(nf{k}+1:end));
    
    if hasTs
        roms{k}.Ts = Ts;
    end
end

if ml_field_set_to_value(opts, 'ImproperTrunc') ...
        && isa(opts.ImproperTrunc, 'cell')
    ni = cell(1, numSys);
    for k = 1:numSys
        ni{k} = size(roms{k}.Ainf, 1);
    end
else
    ni = size(roms{1}.Ainf, 1);
end

info.AbsErrBound = absErrBound;
info.Hsvi        = hsvi;
info.Hsvp        = hsvp;
info.Ni          = ni;
info.Np          = nf;
info.Nu          = size(roms{1}.Au, 1);

if opts.StoreProjection
    for k = 1:numSys
        info.T{k} = infoADTF.T * blkdiag(T{k}, eye(info.Nu), Tinf{k});
        info.W{k} = blkdiag(W{k}, eye(info.Nu), Winf{k}) * infoADTF.W;
    end
else
    info.T = [];
    info.W = [];
end

[varargout{1:nargout}] = ml_dss_format_output(roms, ioformat, info);
