function [varargout] = ml_dt_ss_mt(varargin)
%ML_DT_SS_MT Modal truncation for standard discrete-time systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, info] = ML_DT_SS_MT(A, B, C, D)
%   [Ar, Br, Cr, Dr, info] = ML_DT_SS_MT(A, B, C, D, opts)
%
%   [rom, info] = ML_DT_SS_MT(sys)
%   [rom, info] = ML_DT_SS_MT(sys, opts)
%
% DESCRIPTION:
%   This function computes the modal truncation for a standard
%   discrete-time system of the form
%
%       x(t+1) = A*x(t) + B*u(t),                                       (1)
%         y(t) = C*x(t) + D*u(t).                                       (2)
%
%   Therefore, a block diagonalization of the matrix A is performed using
%   the matrix sign function and a Sylvester equation, such that
%
%            [ Ar  0 ]                        [ Cr ]
%       A2 = [       ], B2 = [ Br, B1 ], C2 = [    ], Dr = D,
%            [ 0  A1 ]                        [ C1 ]
%
%   where Ar contains all the eigenvalues of A, which have an absolute
%   value larger than a given alpha. As result, the reduced-order system is
%   given by
%
%       x(t+1) = Ar*x(t) + Br*u(t),                                     (3)
%         y(t) = Cr*x(t) + Dr*u(t).                                     (4)
%
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   sys  - structure or state-space object, containing the standard
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Alpha           | real scalar, such that all finite eigenvalues with|
%   | {!}             | the real part smaller than Alpha are truncated    |
%   |                 | (default -1.0)                                    |
%   +-----------------+---------------------------------------------------+
%   | dsylvopts       | structure, containing the optional parameters for |
%   |                 | the Sylvester equation solver used for the        |
%   |                 | decomposition of the eigenvalues,                 |
%   |                 | see ml_dsylv_smith                                |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | signmopts       | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition of the eigenvalues, see ml_signm    |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | StoreProjection | {0, 1}, used to disable/enable storing of the     |
%   |                 | computed projection matrices W and T              |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%
%   Note: Parameters marked with {!} may also be a cell array containing
%         multiple arguments. In this case an cell array of the same size
%         is returned with one entry computed for each input argument and
%         the marked fields of the info struct are cells as well.
%         When multiple arguments are given as cells, they are expected to
%         have the same length.
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   rom  - structure or state-space object, with the following entries:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | infoSIGNM       | structure, containing information about the       |
%   | {!}             | matrix sign function method, see ml_signm         |
%   +-----------------+---------------------------------------------------+
%   | infoDSYLV       | structure, containing information about the       |
%   | {!}             | Sylvester equation solver, see ml_dsylv_smith     |
%   +-----------------+---------------------------------------------------+
%   | N {!}           | Dimension of the reduced-order model              |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   P. Benner, E. S. Quintana-Orti, Model reduction based on spectral
%   projection methods, in: P. Benner, V. Mehrmann, D. Sorensen (Eds.),
%   Dimension Reduction of Large-Scale Systems, Vol. 45 of Lect. Notes
%   Comput. Sci. Eng., Springer, Berlin/Heidelberg, Germany, 2005,
%   pp. 5--45. https://doi.org/10.1007/3-540-27909-1_1
%
% See also ml_dt_dss_mt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 5);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
opts = ml_check_cell_param(opts, 'Alpha', @ml_assert_nonnegscalar, 0.5);
[opts.Alpha, Inds] = sort(cell2mat(opts.Alpha));
numAlpha           = length(opts.Alpha);

if ml_field_set_to_value(opts, 'dsylvopts')
    assert(isa(opts.dsylvopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.dsylvopts has to be a struct!');
else
    opts.dsylvopts = struct();
end

if ml_field_set_to_value(opts, 'signmopts')
    assert(isa(opts.signmopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.signmopts has to be a struct!');
else
    opts.signmopts = struct();
end

if ml_field_set_to_value(opts, 'StoreProjection')
    ml_assert_boolean(opts.StoreProjection, 'opts.StoreProjection');
else
    opts.StoreProjection = false;
end

% Initial info structure.
info = struct();
info.infoSIGNM = cell(1, numAlpha);
info.infoDSYLV = cell(1, numAlpha);
info.N         = cell(1, numAlpha);

if opts.StoreProjection
    info.T = cell(1, numAlpha);
    info.W = cell(1, numAlpha);
else
    info.T = [];
    info.W = [];
end

roms = cell(1, numAlpha);

% Save sampling time.
if isfield(sys, 'Ts')
    Ts    = sys.Ts;
    hasTs = 1;
else
    hasTs = 0;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for k = 1:numAlpha
    % Decomposition of desired and undesired eigenvalues.
    n              = size(sys.A, 1);
    [Z, infoSIGNM] = ml_signm((sys.A + opts.Alpha(k) * eye(n)) ...
        \ (sys.A - opts.Alpha(k) * eye(n)), [], opts.signmopts);
    
    % Assign information about sign function.
    info.infoSIGNM{Inds(k)} = infoSIGNM;
    
    n1 = (n + round(trace(Z))) / 2;
    
    if (n1 > 0) && (n1 < n)
        [Q, ~, ~] = qr(eye(n) + Z);
        
        A1     = Q(: , n1+1:end)' * sys.A * Q(: , n1+1:end);
        Wa     = Q(: , 1:n1)' * sys.A * Q(: , n1+1:end);
        sys.A  = Q(: , 1:n1)' * sys.A * Q(: , 1:n1);
        Ainv   = sys.A \ eye(n1);
        
        [Y, infoDSYLV] = ml_dsylv_smith(1 / opts.Alpha(k) * Ainv, ...
            opts.Alpha(k) * A1, -Ainv * Wa, [], [], opts.dsylvopts);
        
        sys.B = Q' * sys.B;
        sys.B = sys.B(1:n1, :) - Y * sys.B(n1+1:end, :);
        
        sys.C = sys.C * Q(: , 1:n1);
        
        % Assign information about Sylvester equation solver.
        info.infoDSYLV{Inds(k)} = infoDSYLV;
        
        if opts.StoreProjection
            if k == 1
                info.T{k} = Q(: , 1:n1);
                info.W{k} = [eye(n1), -Y] * Q';
            else
                info.T{k} = info.T{k-1} * Q(: , 1:n1);
                info.W{k} = [eye(n1), -Y] * (Q' * info.W{k-1});
            end
        end
    elseif n1 == 0
        [sys.A, sys.B, sys.C] = deal([]);
        
        if opts.StoreProjection
            info.T{k} = zeros(n, 0);
            info.W{k} = zeros(0, n);
        end
    else
        if opts.StoreProjection
            info.T{k} = eye(n);
            info.W{k} = eye(n);
        end
    end
    
    roms{Inds(k)} = sys;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign output information.
for k = 1:numAlpha
    info.N{k}  = size(roms{k}.A, 1);
    
    if hasTs
        roms{k}.Ts = Ts;
    end
end

[varargout{1:nargout}] = ml_ss_format_output(roms, ioformat, info);
