function [Z] = ml_compress_fac(Z, tol, type)
%ML_COMPRESS_FAC SVD-based column/row compression.
%
% SYNTAX:
%   Z = ML_COMPRESS_FAC(Z, tol, type)
%
% DESCRIPTION:
%   Computes a column or row compression of the matrix Z using the SVD.
%   Usually used to approximate the products Z'*Z or Z*Z' via a low-rank
%   factor.
%
% INPUT:
%   Z    - matrix of dimensions n x m
%   tol  - nonnegative scalar, tolerance multiplied with the largest
%          singular value to determine the rank of the approximation
%   type - string, switching between row and column compression
%            'c', 'col', 'column' - for column compression
%            'r', 'row'           - for row compression
%
% OUTPUT:
%   Z    - matrix of dimensions n x r in case of column compression and
%          r x m in case of row compression
%
% See also ml_compress_ldl.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(3, 3);

assert(isa(Z, 'double'), ...
    'MORLAB:data', ...
    'The input argument Z has to be a matrix!');

ml_assert_nonnegscalar(tol, 'tol');

assert(ischar(type), ...
    'MORLAB:data', ...
    'The input argument type has to be a string!');


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COLUMN/ROW COMPRESSION.                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch lower(type)
    case {'c', 'col', 'column'}
        [U, S, ~] = svd(Z, 'econ');
        r         = sum(diag(S) > S(1) * tol);
        Z         = U(:, 1:r) * S(1:r, 1:r);
        
    case {'r', 'row'}
        [~, S, V] = svd(Z, 'econ');
        r         = sum(diag(S) > S(1) * tol);
        Z         = S(1:r, 1:r) * V(:, 1:r)';
        
    otherwise
        error( ...
            'MORLAB:data', ...
            'This type of compression is not implemented!');
end
