function [Z, Y] = ml_compress_ldl(Z, Y, tol, type)
%ML_COMPRESS_LDL LDL^T row/column compression.
%
% SYNTAX:
%   [Z, Y] = ML_COMPRESS_LDL(Z, Y, tol, type)
%
% DESCRIPTION:
%   Computes a column or row compression of the matrices Z and Y using an
%   eigenvalue decomposition. Usually used to approximate the products
%   Z'*Y*Z (row compression) or Z*Y*Z' (column compression) via low-rank
%   factors.
%
% INPUT:
%   Z    - matrix of dimensions n x m (column compression) or m x n
%          (row compression)
%   Y    - symmetric matrix of dimensions m x m
%   tol  - nonnegative scalar, tolerance multiplied with the largest
%          singular value to determine the rank of the approximation
%   type - string, switching between row and column compression
%            'c', 'col', 'column' - for column compression
%            'r', 'row'           - for row compression
%
% OUTPUT:
%   Z    - matrix of dimensions n x r in case of column compression and
%          r x n in case of row compression
%   Y    - matrix of dimensions r x r
%
% See also ml_compress_fac.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(4, 4);

assert(isa(Z, 'double'), ...
    'MORLAB:data', ...
    'The input argument Z has to be a matrix!');

assert(isa(Y, 'double'), ...
    'MORLAB:data', ...
    'The input argument Y has to be a matrix!');

ml_assert_nonnegscalar(tol, 'tol');

assert(ischar(type), ...
    'MORLAB:data', ...
    'The input argument type has to be a string!');


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% LDL^T COLUMN/ROW COMPRESSION.                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch lower(type)
    case {'c', 'col', 'column'}
        assert(isequal(size(Y), [size(Z, 2), size(Z, 2)]), ...
            'MORLAB:data', ...
            'Wrong dimensions of matrices Z and Y!');
        
        [Q, R] = qr(Z, 0);
        RYR    = R * Y * R';
        [V, Y] = eig(RYR);
        r      = abs(diag(Y)) > max(abs(diag(Y))) * tol;
        Y      = Y(r, r);
        Z      = Q * V(:, r);
        
    case {'r', 'row'}
        assert(isequal(size(Y), [size(Z, 1), size(Z, 1)]), ...
            'MORLAB:data', ...
            'Wrong dimensions of matrices Z and Y!');
        
        [Q, R] = qr(Z', 0);
        RYR    = R * Y * R';
        [V, Y] = eig(RYR);
        r      = abs(diag(Y)) > max(abs(diag(Y))) * tol;
        Y      = Y(r, r);
        Z      = (Q * V(:, r))';
        
    otherwise
        error( ...
            'MORLAB:data', ...
            'This type of compression is not implemented!');
end
