function [varargout] = ml_dss_format_output(systems, ioformat, info)
%ML_DSS_FORMAT_OUTPUT Converts a cell array of systems to another format.
%
% SYNTAX:
%   [A, B, C, D, E, info] = ML_DSS_FORMAT_OUTPUT(systems, 0, info)
%   [sys, info]           = ML_DSS_FORMAT_OUTPUT(systems, 1, info)
%   [sys, info]           = ML_DSS_FORMAT_OUTPUT(systems, 2, info)
%
% DESCRIPTION:
%   This function takes an array of system-structs and returns cell arrays
%   of the requested format. Unstable and infinite parts of the models are
%   merged when available.
%   If only a single system is given no cell array is created and single
%   element cell arrays in the info struct are unpacked.
%
% INPUT:
%   systems  - cell-array of structs containing the descriptor system's
%              matrices for stable, unstable and infinite parts
%   ioformat - integer, describing the output format.
%                0 - for input type matrices
%                1 - for input type struct
%                2 - for input type state-space object
%   info     - struct, containing additional information
%
% OUTPUT:
%   A    - cell array of matrices with dimensions n x n
%   B    - cell array of matrices with dimensions n x m
%   C    - cell array of matrices with dimensions p x n
%   D    - cell array of matrices with dimensions p x m
%   E    - cell array of matrices with dimensions n x n
%   sys  - cell array of structures or state-space objects, containing the
%          descriptor system's matrices
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrixwith dimensions n x m                       |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix with dimensions p x n                      |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix with dimensions p x m                      |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   info - struct, with single-cell-array fields unpacked
%
% See also ml_ss_format_output, ml_soss_format_output.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%

narginchk(3, 3);

assert(isa(systems, 'cell') ...
    && isvector(systems), ...
    'MORLAB:data', ...
    'The input argument systems has to be a cell vector!');

assert(isa(info, 'struct'), ...
    'MORLAB:data', ...
    'The input argument info has to be a struct!');

% Sort info fields.
[~, perm] = sort(lower(fieldnames(info)));
info      = orderfields(info, perm);

% Format output ROMs.
numSys = length(systems);

for k = numSys:-1:1
    % Merge stable and unstable parts if available.
    sys = structfun(@full, systems{k}, 'UniformOutput', 0);
    if isfield(sys, 'Au')
        sys.A = blkdiag(sys.A, sys.Au, sys.Ainf);
        sys.B = [sys.B; sys.Bu; sys.Binf];
        sys.C = [sys.C,  sys.Cu, sys.Cinf];
        sys.D = sys.D;
        sys.E = blkdiag(sys.E, sys.Eu, sys.Einf);
    end
    
    % Pack arrays.
    switch ioformat
        case 0
            varargout{1}{k} = sys.A;
            varargout{2}{k} = sys.B;
            varargout{3}{k} = sys.C;
            varargout{4}{k} = sys.D;
            varargout{5}{k} = sys.E;
            
        case 1
            varargout{1}{k} = struct(...
                'A', sys.A, ...
                'B', sys.B, ...
                'C', sys.C, ...
                'D', sys.D, ...
                'E', sys.E);
            
            if isfield(sys, 'Ts')
                varargout{1}{k}.Ts = sys.Ts;
            end
            
        case 2
            if isfield(sys, 'Ts')
                Ts = sys.Ts;
            else
                Ts = 0;
            end
            
            varargout{1}{k} = dss(sys.A, sys.B, sys.C, sys.D, sys.E, Ts);
            
        otherwise
            error('MORLAB:data', ...
                'The input argument ioformat has to be 0, 1 or 2!');
    end
end

% A single result is unpacked.
if numSys == 1
    for k = 1:length(varargout)
        varargout{k} = varargout{k}{1};
    end
    
    % Iterate fields of the info struct.
    fields = fieldnames(info);
    for field = fields'
        f = info.(field{1});
        if isa(f, 'cell') && (length(f) == 1)
            info.(field{1}) = f{1};
        end
    end
end

% Forward info structure.
len              = length(varargout);
varargout{len+1} = info;
