function [sys, opts, ioformat] = ml_dss_lti2struct(varargin)
%ML_DSS_LTI2STRUCT Maps given input to a descriptor LTI system struct.
%
% SYNTAX:
%   [sys, opts, ioformat] = ML_DSS_LTI2STRUCT(A, B, C, D, E)
%   [sys, opts, ioformat] = ML_DSS_LTI2STRUCT(A, B, C, D, E, opts)
%
%   [sys, opts, ioformat] = ML_DSS_LTI2STRUCT(sys)
%   [sys, opts, ioformat] = ML_DSS_LTI2STRUCT(sys, opts)
%
% DESCRIPTION:
%   This function gets the possible inputs of the model order reduction
%   methods, checks the system matrices and returns structs for the system
%   and the options in the right format
%
%       Ex'(t) = A*x(t) + B*u(t),                                       (1)
%         y(t) = C*x(t) + D*u(t).                                       (2)
%
%   Also, the input format is returned as an integer.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   E    - matrix from (1) with dimensions n x n
%   sys  - structure or state-space object, containing the descriptor
%          system's matrices
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing optional parameters
%
% OUTPUT:
%   sys      - structure, containing the descriptor system's matrices
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   opts     - structure, containing optional parameters
%   ioformat - integer, describing the input format
%                0 - for input type matrices
%                1 - for input type struct
%                2 - for input type state-space object
%
% See also ml_ss_lti2struct, ml_soss_lti2struct.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%

% Ensure existing inputs.
narginchk(1, 6);

if isa(varargin{1}, 'double') % Input case: matrices.
    narginchk(5, 6)
    
    if (nargin < 6) || isempty(varargin{6})
        opts = struct();
    else
        opts = varargin{6};
    end
    
    % Assign and check standard system.
    sys = struct(...
        'A', varargin{1}, ...
        'B', varargin{2}, ...
        'C', varargin{3}, ...
        'D', varargin{4}, ...
        'E', varargin{5});
    
    ml_assert_descsys(sys);
    
    if ml_field_set_to_value(sys, 'D')
        assert(isequal(size(sys.D), [size(sys.C, 1), size(sys.B, 2)]), ...
            'MORLAB:data', ...
            'The matrix D must have the dimensions %d x %d!', ...
            size(sys.C, 1), size(sys.B, 2));
    end
    
    % Output format.
    ioformat = 0;
elseif isa(varargin{1}, 'struct') % Input case: structs.
    narginchk(1, 2);
    
    if (nargin < 2) || isempty(varargin{2})
        opts = struct();
    else
        opts = varargin{2};
    end
    
    % Check input standard system.
    sys = varargin{1};
    
    ml_assert_descsys(sys);
    
    if ml_field_set_to_value(sys, 'D')
        assert(isequal(size(sys.D), [size(sys.C, 1), size(sys.B, 2)]), ...
            'MORLAB:data', ...
            'The matrix D must have the dimensions %d x %d!', ...
            size(sys.C, 1), size(sys.B, 2));
    else
        sys.D = [];
    end
    
    % Output format.
    ioformat = 1;
elseif isa(varargin{1}, 'ss') % Input case: ss.
    narginchk(1, 2);
    
    if (nargin < 2) || isempty(varargin{2})
        opts = struct();
    else
        opts = varargin{2};
    end
    
    % Assign input standard system.
    sys = struct(...
        'A', varargin{1}.A, ...
        'B', varargin{1}.B, ...
        'C', varargin{1}.C, ...
        'D', varargin{1}.D, ...
        'E', varargin{1}.E);
    
    % Is discrete.
    if varargin{1}.Ts ~= 0
        sys.Ts = varargin{1}.Ts;
    end
    
    % Output format.
    ioformat = 2;
else % Input case: invalid type.
    error('MORLAB:data', ...
        'Such type of input data is not implemented!');
end

% Convert to full struct.
if any(structfun(@issparse, sys, 'UniformOutput', 1))
    warning('MORLAB:lti2struct', ...
        'Converting from sparse to full system matrices!');
    sys = structfun(@full, sys, 'UniformOutput', 0);
end

% Check option struct.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');
