function [roms, hsvp, hsvi, prj] = ml_dss_sr(sys, R, Rinf, L, Linf, tselect, opts)
%ML_DSS_SR Square root method for descriptor systems.
%
% SYNTAX:
%   [sys, hsvp, hsvi, prj] = ML_DSS_SR(sys, R, Rinf, L, Linf, tselect)
%   [sys, hsvp, hsvi, prj] = ML_DSS_SR(sys, R, Rinf, L, Linf, tselect, opts)
%
% DESCRIPTION:
%   Computes the reduced-order matrices of a descriptor system by the
%   square root method. Therefore, transformation matrices of the form
%
%       W    = L * U * S^(-1/2),
%       Winf = Linf * Uinf * Sinf^(-1/2),
%       T    = R * V * S^(-1/2),
%       Tinf = Rinf * Vinf * Sinf^(-1/2)
%
%   are computed, where U, Uinf, S, Sinf, V, Vinf result from singular
%   value decompositions. The computation of the reduced-order model is
%   done by
%
%            [ W' * A * T     0  ]
%       Ar = [                   ],
%            [     0          I  ]
%
%            [ W' * B       ]
%       Br = [              ],
%            [ Winf' * Binf ]
%
%       Cr = [ C * T, Cinf * Tinf ],
%
%            [ I              0            ]
%       Er = [                             ].
%            [ 0       Winf' * Einf * Tinf ]
%
% INPUT:
%   sys     - structure, containing the descriptor system in the form:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | E               | matrix with dimensions nf x nf                    |
%   +-----------------+---------------------------------------------------+
%   | Einf            | matrix with dimensions ninf x ninf                |
%   +-----------------+---------------------------------------------------+
%   | A               | matrix with dimensions nf x nf                    |
%   +-----------------+---------------------------------------------------+
%   | Ainf            | matrix with dimensions ninf x ninf                |
%   +-----------------+---------------------------------------------------+
%   | B               | matrix with dimensions nf x m                     |
%   +-----------------+---------------------------------------------------+
%   | Binf            | matrix with dimensions ninf x m                   |
%   +-----------------+---------------------------------------------------+
%   | C               | matrix with dimensions p x nf                     |
%   +-----------------+---------------------------------------------------+
%   | Cinf            | matrix with dimensions p x ninf                   |
%   +-----------------+---------------------------------------------------+
%   R       - Cholesky factor of the proper controllability Gramian with
%             dimensions nf x nr1
%   Rinf    - Cholesky factor of the improper controllability Gramian with
%             dimensions ninf x nr2
%   L       - Cholesky factor of the proper observability Gramian with
%             dimensions nf x nl1
%   Linf    - Cholesky factor of the improper observability Gramian with
%             dimensions ninf x nl2
%   tselect - an integer, used to determine the computation method for the
%             order of the reduced-order model
%               0 - order is directly given by user
%               1 - computed by a relative tolerance for the hsv
%               2 - computed by a relative tolerance on the sum of hsv
%               3 - computed by absolute error bound of BT
%               4 - computed by relative error bound of BST
%               5 - computed by absolute error bound of LQGBT
%               6 - computed by absolute error bound of HinfBT
%   opts    - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, scaling term from the H-infinity |
%   |                 | balanced truncation                               |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | ImproperTrunc   | nonnegative scalar, tolerance multiplied with the |
%   | {!}             | largest proper Hankel singular value of the       |
%   |                 | system to truncate the improper part, if 0 no     |
%   |                 | improper balanced truncation is performed         |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | Index           | nonnegative integer, index of nilpotency of the   |
%   |                 | matrix Einf used to set an upper bound on the size|
%   |                 | of the reduced improper part, if the index is     |
%   |                 | unknown Inf is set                                |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   | {!}             | reduced-order model chosen by the user if         |
%   |                 | tselect == 1                                      |
%   |                 | (default min(10,length(hsvp)) + nu + ni)          |
%   +-----------------+---------------------------------------------------+
%   | StoreProjection | {0, 1}, used to disable/enable storing of the     |
%   |                 | computed projection matrices W and T              |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used in the         |
%   | {!}             | different error formulas                          |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
%   Note: Parameters marked with {!} may also be a cell array containing
%         multiple arguments. In this case an cell array of the same size
%         is returned with one entry computed for each input argument and
%         the marked fields of the info struct are cells as well.
%         When multiple arguments are given as cells, they are expected to
%         have the same length.
%
% OUTPUT:
%   sys  - struct, containing the transformed system matrices
%   hsvp - vector, containing the characteristic proper singular values
%   hsvi - vector, containing the improper Hankel singular values
%   prj  - struct, containing the left and right projection matrices T,
%          Tinf, W, Winf, if opts.StoreProjection == 1
%
%
% REFERENCE:
%   V. Mehrmann, T. Stykel, Balanced truncation model reduction for
%   large-scale systems in descriptor form, in: P. Benner, V. Mehrmann,
%   D. C. Sorensen (Eds.), Dimension Reduction of Large-Scale Systems,
%   Vol. 45 of Lect. Notes Comput. Sci. Eng., Springer-Verlag,
%   Berlin/Heidelberg, Germany, 2005, pp. 83--115.
%   https://doi.org/10.1007/3-540-27909-1_3
%
% See also ml_ss_sr, ml_dss_bfsr, ml_order.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(6, 7);

if (nargin < 7) || isempty(opts)
    opts = struct();
end

% Check of input descriptor system.
ml_assert_descsys(sys);
ml_assert_infdescsys(sys);

nf   = size(sys.A, 1);
ninf = size(sys.Ainf, 1);

assert(size(R, 1) == nf, ...
    'MORLAB:data', ...
    'The matrix R must have the same number of rows as sys.A!');

assert(size(Rinf, 1) == ninf, ...
    'MORLAB:data', ...
    'The matrix Rinf must have the same number of rows as sys.Ainf!');

assert(size(L, 1) == nf, ...
    'MORLAB:data', ...
    'The matrix L must have the same number of rows as sys.A!');

assert(size(Linf, 1) == ninf, ...
    'MORLAB:data', ...
    'The matrix Linf must have the same number of rows as sys.Ainf!');

if issparse(R), R = full(R); end
if issparse(L), L = full(L); end
if issparse(Rinf), Rinf = full(Rinf); end
if issparse(Linf), Linf = full(Linf); end

% Check and assign optional parameters.
opts = ml_check_cell_param(opts, 'ImproperTrunc', ...
    @ml_assert_nonnegscalar, log(nf + ninf) * eps);

if ml_field_set_to_value(opts, 'Index')
    ml_assert_nonnegintinf(opts.Index, 'opts.Index');
else
    opts.Index = Inf;
end

if ml_field_set_to_value(opts, 'StoreProjection')
    ml_assert_boolean(opts.StoreProjection, 'opts.StoreProjection');
else
    opts.StoreProjection = false;
end

numVariants = length(opts.ImproperTrunc);
% vecInd is set by a calling routine to signal that only one variant should
% be computed even when some arguments are cell arrays.
if ml_field_set_to_value(opts, 'vecInd')
    hasInd = 1;
    if numVariants > 1
        assert(numVariants >= opts.vecInd ...
            && numVariants <= opts.vecMax, ...
            'MORLAB:data', ...
            'Inconsistent length of vector parameters.');
        
        % Only compute for param at that index.
        indBegin = opts.vecInd;
        indEnd   = opts.vecInd;
    end
else
    hasInd = 0;
    if numVariants > 1
        opts.vecMax = numVariants;
    end
    indBegin = 1;
    indEnd   = numVariants;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTATION OF CHARACTERISTIC SINGULAR VALUES.                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[U, S, V]          = svd(L' * sys.E * R, 'econ');
[Uinf, Sinf, Vinf] = svd(Linf' * sys.Ainf * Rinf, 'econ');

hsvp = nonzeros(diag(S));
hsvi = nonzeros(diag(Sinf));


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTATION OF REDUCED-ORDER MODEL.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Makes implementation of compute_reduced_model simpler.
opts.noUnpack = 1;
% No vectorization on this level, just forward results.
if numVariants == 1
    [roms, Ts, Tinfs, Ws, Winfs] = compute_reduced_model(sys, hsvi, ...
        hsvp, opts.ImproperTrunc{1}, tselect, R, Rinf, L, Linf, ...
        U, Uinf, V, Vinf, opts);
else
    roms  = cell(1, numVariants);
    Ts    = cell(1, numVariants);
    Ws    = cell(1, numVariants);
    Tinfs = cell(1, numVariants);
    Winfs = cell(1, numVariants);
    for k = indBegin:indEnd
        % Introduce index for ml_order, when already set by the calling
        % routine this has no effect since k == opts.vecInd.
        opts.vecInd = k;
        
        [rom, T, Tinf, W, Winf] = compute_reduced_model(sys, hsvi, ...
            hsvp, opts.ImproperTrunc{k}, tselect, R, Rinf, L, Linf, ...
            U, Uinf, V, Vinf, opts);
        roms{k}  = rom{1};
        Ts{k}    = T{1};
        Tinfs{k} = Tinf{1};
        Ws{k}    = W{1};
        Winfs{k} = Winf{1};
    end
end

if not(opts.StoreProjection)
    Ts    = [];
    Tinfs = [];
    Ws    = [];
    Winfs = [];
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% HANDLE OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

len = length(roms);
% Unpack single result.
if len == 1
    roms = roms{1};
    
    if opts.StoreProjection
        Ts    = Ts{1};
        Tinfs = Tinfs{1};
        Ws    = Ws{1};
        Winfs = Winfs{1};
    end
elseif hasInd
    roms = roms{opts.vecInd};
    
    if opts.StoreProjection
        Ts    = Ts{opts.vecInd};
        Tinfs = Tinfs{opts.vecInd};
        Ws    = Ws{opts.vecInd};
        Winfs = Winfs{opts.vecInd};
    end
end

prj = struct( ...
    'T'   , {Ts}, ...
    'Tinf', {Tinfs}, ...
    'W'   , {Ws}, ...
    'Winf', {Winfs});

end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% HELPER FUNCTION.                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [roms, Ts, Tinfs, Ws, Winfs]  = compute_reduced_model(sys, ....
    hsvi, hsvp, improperTrunc, tselect, R, Rinf, L, Linf, U, Uinf, V, ...
    Vinf, opts)
%COMPUTE_REDUCED_MODEL Compute projection matrices and reduced model.
%
% SYNTAX:
%   sys = COMPUTE_REDUCED_MODEL(sys, hsvi, hsvp, improperTrunc, ...
%       tselect, R, Rinf, L, Linf, U, Uinf, V, Vinf, opts)
%
% DESCRIPTION:
%   Sets up the projection matrices and computes the reduced-order model.
%
% INPUT:
%   sys            - structure, containing the descriptor system in the
%                    form:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | E               | matrix with dimensions nf x nf                    |
%   +-----------------+---------------------------------------------------+
%   | Einf            | matrix with dimensions ninf x ninf                |
%   +-----------------+---------------------------------------------------+
%   | A               | matrix with dimensions nf x nf                    |
%   +-----------------+---------------------------------------------------+
%   | Ainf            | matrix with dimensions ninf x ninf                |
%   +-----------------+---------------------------------------------------+
%   | B               | matrix with dimensions nf x m                     |
%   +-----------------+---------------------------------------------------+
%   | Binf            | matrix with dimensions ninf x m                   |
%   +-----------------+---------------------------------------------------+
%   | C               | matrix with dimensions p x nf                     |
%   +-----------------+---------------------------------------------------+
%   | Cinf            | matrix with dimensions p x ninf                   |
%   +-----------------+---------------------------------------------------+
%   hsvi          - improper Hankel singular values
%   hsvp          - proper Hankel singular values
%   ImproperTrunc - tolerance multiplied with the largest proper Hankel
%                   singular value of the system to truncate the improper
%                   part, if 0 no improper balanced truncation is performed
%   tselect       - an integer, used to determine the computation method
%                   for the order of the reduced-order model
%                     0 - order is directly given by user
%                     1 - computed by a relative tolerance for the hsv
%                     2 - computed by a relative tolerance on the sum of
%                         hsv
%                     3 - computed by absolute error bound of BT
%                     4 - computed by relative error bound of BST
%                     5 - computed by absolute error bound of LQGBT
%                     6 - computed by absolute error bound of HinfBT
%   R             - Cholesky factor of the proper controllability Gramian
%                   with dimensions nf x nr1
%   Rinf          - Cholesky factor of the improper controllability Gramian
%                   with dimensions ninf x nr2
%   L             - Cholesky factor of the proper observability Gramian
%                   with dimensions nf x nl1
%   Linf          - Cholesky factor of the improper observability Gramian
%                   with dimensions ninf x nl2
%   U             - left singular vectors, such that U*S*V = L'*E*R
%   Uinf          - left singular vectors, such that
%                   Uinf*Sinf*Vinf = Linf'*Ainf*Rinf
%   V             - right singular vectors, such that U*S*V = L'*E*R
%   Vinf          - right singular vectors, such that
%                   Uinf*Sinf*Vinf = Linf'*Ainf*Rinf
%   opts          - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, scaling term from the H-infinity |
%   |                 | balanced truncation                               |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | Index           | nonnegative integer, index of nilpotency of the   |
%   |                 | matrix Einf used to set an upper bound on the size|
%   |                 | of the reduced improper part, if the index is     |
%   |                 | unknown Inf is set                                |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | tselect == 1                                      |
%   |                 | (default min(10,length(hsvp)) + nu + ni)          |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used in the         |
%   |                 | different error formulas                          |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   sys            - struct, containing the transformed system matrices

% Computation of the minimal size of the improper part.
if isempty(hsvi) || (improperTrunc == 0) || (hsvi(1) <= improperTrunc)
    ninf = 0;
else
    ninf = sum(hsvi >= improperTrunc * hsvi(1));
end

% Application of rank bound on the improper part.
ninf = min([ninf, size(sys.Ainf, 1), size(sys.B, 2) * opts.Index, ...
    size(sys.C, 1) * opts.Index]);

% Computation of the order of the proper part.
if ml_field_set_to_value(sys, 'Au')
    nfs = ml_order(hsvp, size(sys.Au, 1), ninf, tselect, opts);
else
    nfs = ml_order(hsvp, 0, ninf, tselect, opts);
end

len   = length(nfs);
roms  = cell(1, len);
Ts    = cell(1, len);
Tinfs = cell(1, len);
Ws    = cell(1, len);
Winfs = cell(1, len);
for k = 1:len
    nf  = nfs{k};
    rom = sys;
    
    % Computation of improper matrices.
    if ninf > 0
        Sinf = sparse(1:ninf, 1:ninf, 1./ sqrt(hsvi(1:ninf)));
        Winf = Sinf * (Linf * Uinf(: , 1:ninf))';
        Tinf = (Rinf * Vinf(: , 1:ninf)) * Sinf;
        
        rom.Ainf = eye(ninf);
        rom.Binf = Winf * sys.Binf;
        rom.Cinf = sys.Cinf * Tinf;
        rom.Einf = Winf * sys.Einf * Tinf;
        
        Tinfs{k} = Tinf;
        Winfs{k} = Winf;
    elseif (opts.Index == 0) || (improperTrunc > 0)
        rom.Ainf = [];
        rom.Binf = [];
        rom.Cinf = [];
        rom.Einf = [];
        Tinfs{k} = zeros(size(sys.Ainf, 1), 0);
        Winfs{k} = zeros(0, size(sys.Ainf, 1));
    elseif improperTrunc == 0
        Tinfs{k} = eye(size(sys.Ainf));
        Winfs{k} = eye(size(sys.Ainf));
    else
        Tinfs{k} = zeros(size(sys.Ainf, 1), 0);
        Winfs{k} = zeros(0, size(sys.Ainf, 1));
    end
    
    % Computation of proper matrices.
    S = sparse(1:nf, 1:nf, 1 ./ sqrt(hsvp(1:nf)));
    W = S * (L * U(: , 1:nf))';
    T = (R * V(: , 1:nf)) * S;

    rom.A = W * sys.A * T;
    rom.B = W * sys.B;
    rom.C = sys.C * T;
    rom.E = eye(nf);
    
    roms{k} = rom;
    Ts{k}   = T;
    Ws{k}   = W;
end

end
