function [sys, info] = ml_dt_ss_adtf(sys, opts)
%ML_DT_SS_ADTF Additive decomposition of standard system's transfer function.
%
% SYNTAX:
%   [sys, info] = ML_DT_SS_ADTF(sys)
%   [sys, info] = ML_DT_SS_ADTF(sys, opts)
%
% DESCRIPTION:
%   Consider a standard discrete-time system of the form
%
%       x(t+1) = A*x(t) + B*u(t),                                       (1)
%        y(t)  = C*x(t).                                                (2)
%
%   This function computes an additive decomposition of the corresponding
%   transfer function by a block diagonalization of the matrix A. The
%   system is transformed, such that
%
%                [ As     ]       [ Bs ]
%       z(t+1) = [        ]z(t) + [    ]u(t),                           (3)
%                [     Au ]       [ Bu ]
%
%         y(t) = [ Cs, Cu ]z(t),                                        (4)
%
%   where As contains the eigenvalues with absolute values smaller than 1
%   and Au all the eigenvalues with absolute values larger than 1.
%
% INPUT:
%   sys  - structure, containing the descriptor system in the form:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | A               | matrix with dimensions n x n in (1)               |
%   +-----------------+---------------------------------------------------+
%   | B               | matrix with dimensions n x m in (1)               |
%   +-----------------+---------------------------------------------------+
%   | C               | matrix with dimensions p x n in (2)               |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | stabsignmopts   | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, see ml_signm                               |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | stabdsylvopts   | structure, containing the optional parameters for |
%   |                 | the Sylvester equation solver used for the        |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, see ml_dsylv_smith                         |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | StoreProjection | {0, 1}, used to disable/enable storing of the     |
%   |                 | computed projection matrices W and T              |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | UnstabDim       | integer, dimension of the deflating anti-stable   |
%   |                 | subspace, negative if unknown                     |
%   |                 | (default -1)                                      |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   sys - structure, containing the transformed descriptor system:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | A               | matrix with dimensions ns x ns, see As in (3)     |
%   +-----------------+---------------------------------------------------+
%   | Au              | matrix with dimensions nu x nu in (3)             |
%   +-----------------+---------------------------------------------------+
%   | B               | matrix with dimensions ns x m, see Bs in (3)      |
%   +-----------------+---------------------------------------------------+
%   | Bu              | matrix with dimensions nu x m in (3)              |
%   +-----------------+---------------------------------------------------+
%   | C               | matrix with dimensions p x ns, see Cs in (4)      |
%   +-----------------+---------------------------------------------------+
%   | Cu              | matrix with dimensions p x nu in (4)              |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information about the
%          algorithms used for the additive decomposition
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | infoSTABSIGNM   | structure, containing information about the       |
%   |                 | matrix sign function method, see ml_signm         |
%   +-----------------+---------------------------------------------------+
%   | infoSTABDSYLV   | structure, containing information about the       |
%   |                 | Sylvester equation solver, see ml_dsylv_sgn       |
%   +-----------------+---------------------------------------------------+
%   | Ns              | Number of identified stable eigenvalues           |
%   +-----------------+---------------------------------------------------+
%   | Nu              | Number of identified anti-stable eigenvalues      |
%   +-----------------+---------------------------------------------------+
%   | T               | projection matrix used as right state-space       |
%   |                 | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%   | W               | projection matrix used as left state-space        |
%   |                 | transformation to obtain the resulting block      |
%   |                 | system, if opts.StoreProjection == 1              |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   S. Werner, Hankel-norm approximation of descriptor systems, Master's
%   thesis, Otto von Guericke University, Magdeburg, Germany (2016).
%   http://nbn-resolving.de/urn:nbn:de:gbv:ma9:1-8845
%
% See also ml_dt_dss_adtf, ml_ss_lti2struct.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(1, 2);

if (nargin < 2) || isempty(opts)
    opts = struct();
end

% Check input standard system.
ml_assert_stdsys(sys);

n = size(sys.A, 1);

% Check and assign optional parameters.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');

if ml_field_set_to_value(opts, 'stabsignmopts')
    assert(isa(opts.stabsignmopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.stabsignmopts has to be a struct!');
else
    opts.stabsignmopts = struct();
end

if ml_field_set_to_value(opts, 'stabdsylvopts')
    assert(isa(opts.stabdsylvopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.stabdsylvopts has to be a struct!');
else
    opts.stabdsylvopts = struct();
end

if ml_field_set_to_value(opts, 'StoreProjection')
    ml_assert_boolean(opts.StoreProjection, 'opts.StoreProjection');
else
    opts.StoreProjection = false;
end

if ml_field_set_to_value(opts, 'UnstabDim')
    ml_assert_integer(opts.UnstabDim, 'UnstabDim');
else
    opts.UnstabDim = -1;
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION: ANTI-STABLE PART.                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computation deflating subspaces using the matrix sign function.
if opts.UnstabDim == 0
    nu = 0;
else
    [Z, infoSTABSIGNM] = ml_signm((sys.A + eye(n)) \ (sys.A - eye(n)), ...
        [], opts.stabsignmopts);
    
    if opts.UnstabDim > 0
        nu = opts.UnstabDim;
    else
        nu = (n + round(trace(Z))) / 2;
    end
    
    info.infoSTABSIGNM = infoSTABSIGNM;
end

% Block diagonalization of the system matrices.
if (nu > 0) && (nu < n)
    [Q, ~, ~] = qr(eye(n) + Z);
    
    sys.Au = Q(: , 1:nu)' * sys.A * Q(: , 1:nu);
    Wa     = Q(: , 1:nu)' * sys.A * Q(: , nu+1:end);
    sys.A  = Q(: , nu+1:end)' * sys.A * Q(: , nu+1:end);
    Auinv  = sys.Au \ eye(nu);
    
    [Y, infoSTABDSYLV] = ml_dsylv_smith(Auinv, sys.A, -Auinv * Wa, ...
        [], [], opts.stabdsylvopts);
    
    sys.B  = Q' * sys.B;
    sys.Bu = sys.B(1:nu, :) - Y * sys.B(nu+1:end, :);
    sys.B  = sys.B(nu+1:end, :);
    
    sys.Cu = sys.C * Q;
    sys.C  = sys.Cu(: , nu+1:end) + sys.Cu(: , 1:nu) * Y;
    sys.Cu = sys.Cu(: , 1:nu);
    
    info.infoSTABDSYLV = infoSTABDSYLV;
    
    if opts.StoreProjection
        T = Q * [Y, eye(nu); eye(n - nu), zeros(size(Y'))];
        W = [zeros(size(Y')), eye(n - nu); eye(nu), -Y] * Q';
    else
        W = [];
        T = [];
    end
elseif nu == n
    sys.Au = sys.A;
    sys.Bu = sys.B;
    sys.Cu = sys.C;
    
    [sys.A, sys.B, sys.C] = deal([]);
    
    if opts.StoreProjection
        T = eye(n);
        W = eye(n);
    else
        W = [];
        T = [];
    end
else
    [sys.Au, sys.Bu, sys.Cu] = deal([]);
    
    if opts.StoreProjection
        T = eye(n);
        W = eye(n);
    else
        W = [];
        T = [];
    end
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN INFORMATION.                                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Collection of all information.
info.Ns = n - nu;
info.Nu = nu;
info.T  = T;
info.W  = W;

[~, perm] = sort(lower(fieldnames(info)));
info      = orderfields(info, perm);
