function [opts] = ml_morlabopts(fname)
%ML_MORLABOPTS Option struct generator for MORLAB functions.
%
% SYNTAX:
%   ML_MORLABOPTS(fname)
%   opts = ML_MORLABOPTS(fname)
%
% DESCRIPTION:
%   This function can be used to generate an option structure for the
%   MORLAB function specified by fname. The structure is filled with empty
%   arrays such that the default values of the MORLAB function are called.
%   For the choice of the default values, call this function without an
%   output argument or see the documentation of the corresponding function.
%
% INPUT:
%   fname - character array, the name of the function for which the option
%           struct is requested
%
% OUTPUT:
%   opts - struct, the desired option structure for a MORLAB function
%
% See also morlab.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUT.                                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(1, 1);

assert(ischar(fname), ...
    'MORLAB:data', ...
    'The given function name has to be a character array.');

% Check if hyperlinks can be used.
hasHyperlink = usejava('desktop');

% Simplify case checks by restricting to lower case letters.
fname = lower(fname);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHOOSE FUNCTION.                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign option structs.
switch fname
    case 'ml_assert_nonnegintinf'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_nonnegvector'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_posinteger'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_posscalar'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_sosys'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_stdsys'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_check_cell_param'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_vector'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_boolean'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_char'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_infdescsys'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_integer'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_nonnegscalar'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_scalar'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_field_set_to_value'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_check_cell_vector'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_descsys'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_assert_nonnegint'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'morlab_data_desc_infdunstab'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_data_desc_br'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_data_desc_infunstab'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_data_desc_pr'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_data_so_stab'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_data_std_br'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_data_std_pr'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_data_std_unstab'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_data_std_dunstab'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_data_std_dstab'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_data_desc_dstab'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_demo_equation_solvers'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_demo_partial_stabilization'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_demo_mor_continuous_standard'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_demo_mor_continuous_second_order'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_demo_mor_discrete_standard'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_demo_mor_discrete_descriptor'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_demo_morlabopts'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_demo_time_evaluation'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_demo_additive_decomposition'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_demo_frequency_evaluation'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'morlab_demo_mor_continuous_descriptor'
        if nargout == 0
            fprintf(1, ' Scripts cannot use option structs!\n');
        else
            opts = struct([]);
        end

    case 'ml_dlyap_smith'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_dlyap_smith_fac'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {1.0e-02*sqrt(n*eps)} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_dlyapdl_smith_fac'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {1.0e-02*sqrt(n*eps)} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_sylv_sgn'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_cabe_sgn'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_gdlyapdl_smith_fac'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '   Index: [ nonnegative integer | {Inf} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'Index' , [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_dlyapdl_smith'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_dlyap_smith_ldl'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {sqrt(n)*eps} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_dlyapdl_smith_ldl'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {sqrt(n)*eps} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_dsylv_smith'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_dsylv_smith_fac'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {sqrt(n)*eps} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_care_nwt_fac'
        if nargout == 0
            fprintf(1, '   AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '     Info: [ 1 | {0} ]\n');
            fprintf(1, '       K0: [ matrix with dimensions m x n | {zeros(m, n)} ]\n');
            if hasHyperlink
                fprintf(1, ' lyapopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, ' lyapopts: [ 1x1 struct | {ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n');
            end
            fprintf(1, '  MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '   RelTol: [ nonnegative scalar | {1.0e+02*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol' , [], ...
                'Info'   , [], ...
                'K0'     , [], ...
                'lyapopts', ml_morlabopts('ml_lyap_sgn_fac'), ...
                'MaxIter', [], ...
                'RelTol' , []);
        end

    case 'ml_caredl_sgn'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' EqnType: [ ''primal'' | ''dual'' | {''both''} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+02*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'EqnType', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_caredl_sgn_fac'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {1.0e-02*sqrt(n*eps)} ]\n');
            fprintf(1, ' EqnType: [ ''primal'' | ''dual'' | {''both''} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+02*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'EqnType', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_dare_nwt_fac'
        if nargout == 0
            fprintf(1, '    AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '      Info: [ 1 | {0} ]\n');
            fprintf(1, '        K0: [ matrix with dimensions m x n | {zeros(m, n)} ]\n');
            if hasHyperlink
                fprintf(1, ' dlyapopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dlyap_smith_fac">ml_morlabopts(''ml_dlyap_smith_fac'')</a>} ]\n');
            else
                fprintf(1, ' dlyapopts: [ 1x1 struct | {ml_morlabopts(''ml_dlyap_smith_fac'')} ]\n');
            end
            fprintf(1, '   MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '    RelTol: [ nonnegative scalar | {1.0e+02*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol'  , [], ...
                'Info'    , [], ...
                'K0'      , [], ...
                'dlyapopts', ml_morlabopts('ml_dlyap_smith_fac'), ...
                'MaxIter' , [], ...
                'RelTol'  , []);
        end

    case 'ml_daredl_sda'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' EqnType: [ ''primal'' | ''dual'' | {''both''} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+02*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'EqnType', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_daredl_sda_fac'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {1.0e-02*sqrt(n*eps)} ]\n');
            fprintf(1, ' EqnType: [ ''primal'' | ''dual'' | {''both''} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+02*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'EqnType', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_lyapdl_sgn'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_icare_ric_fac'
        if nargout == 0
            fprintf(1, '   AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '  CompTol: [ nonnegative scalar | {1.0e-02*sqrt(n*eps)} ]\n');
            fprintf(1, '     Info: [ 1 | {0} ]\n');
            if hasHyperlink
                fprintf(1, ' careopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_care_nwt_fac">ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, ' careopts: [ 1x1 struct | {ml_morlabopts(''ml_care_nwt_fac'')} ]\n');
            end
            fprintf(1, '  MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '   RelTol: [ nonnegative scalar | {1.0e+02*n*eps} ]\n');
            fprintf(1, ' StoreLQG: [ 1 | {0} ]\n');
            fprintf(1, '       Z0: [ matrix of dimensions n x k | {[]} ]\n');
        else
            opts = struct( ...
                'AbsTol' , [], ...
                'CompTol', [], ...
                'Info'   , [], ...
                'careopts', ml_morlabopts('ml_care_nwt_fac'), ...
                'MaxIter', [], ...
                'RelTol' , [], ...
                'StoreLQG', [], ...
                'Z0'     , []);
        end

    case 'ml_lyap_sgn'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_lyap_sgn_fac'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {1.0e-02*sqrt(n*eps)} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_lyap_sgn_ldl'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {sqrt(n)*eps} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_lyapdl_sgn_fac'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {1.0e-02*sqrt(n*eps)} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_lyapdl_sgn_ldl'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {sqrt(n)*eps} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_pcare_nwt_fac'
        if nargout == 0
            fprintf(1, '   AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '  CompTol: [ nonnegative scalar | {1.0e-02*sqrt(n*eps)} ]\n');
            fprintf(1, '     Info: [ 1 | {0} ]\n');
            if hasHyperlink
                fprintf(1, ' lyapopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, ' lyapopts: [ 1x1 struct | {ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n');
            end
            fprintf(1, '  MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '   RelTol: [ nonnegative scalar | {1.0e+02*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol' , [], ...
                'CompTol', [], ...
                'Info'   , [], ...
                'lyapopts', ml_morlabopts('ml_lyap_sgn_fac'), ...
                'MaxIter', [], ...
                'RelTol' , []);
        end

    case 'ml_sylv_sgn_fac'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, ' CompTol: [ nonnegative scalar | {sqrt(n)*eps} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+01*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'CompTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_ct_ss_mt'
        if nargout == 0
            fprintf(1, '           Alpha: [ real scalar | {-1.0} ]\n');
            if hasHyperlink
                fprintf(1, '       signmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '       signmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
            if hasHyperlink
                fprintf(1, '        sylvopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_sylv_sgn">ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n');
            else
                fprintf(1, '        sylvopts: [ 1x1 struct | {ml_morlabopts(''ml_sylv_sgn'')} ]\n');
            end
        else
            opts = struct( ...
                'Alpha'         , [], ...
                'signmopts'     , ml_morlabopts('ml_signm'), ...
                'StoreProjection', [], ...
                'sylvopts'      , ml_morlabopts('ml_sylv_sgn'));
        end

    case 'ml_ct_ss_prbt'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '       caredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_caredl_sgn_fac">ml_morlabopts(''ml_caredl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       caredlopts: [ 1x1 struct | {ml_morlabopts(''ml_caredl_sgn_fac'')} ]\n');
            end
            fprintf(1, '          Epsilon: [ positive scalar | {1.0e-03} ]\n');
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv))} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '        pcareopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '        pcareopts: [ 1x1 struct | {ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n');
            end
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sign''} ]\n');
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'caredlopts'     , ml_morlabopts('ml_caredl_sgn_fac'), ...
                'Epsilon'        , [], ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'pcareopts'      , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'RiccatiSolver'  , [], ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_dt_ss_mt'
        if nargout == 0
            fprintf(1, '           Alpha: [ real scalar | {-1.0} ]\n');
            if hasHyperlink
                fprintf(1, '       dsylvopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dsylv_smith">ml_morlabopts(''ml_dsylv_smith'')</a>} ]\n');
            else
                fprintf(1, '       dsylvopts: [ 1x1 struct | {ml_morlabopts(''ml_dsylv_smith'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '       signmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '       signmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
        else
            opts = struct( ...
                'Alpha'         , [], ...
                'dsylvopts'     , ml_morlabopts('ml_dsylv_smith'), ...
                'signmopts'     , ml_morlabopts('ml_signm'), ...
                'StoreProjection', []);
        end

    case 'ml_ct_ss_bt'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv)) + Nu} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '     stabsylvopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_sylv_sgn">ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n');
            else
                fprintf(1, '     stabsylvopts: [ 1x1 struct | {ml_morlabopts(''ml_sylv_sgn'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
            fprintf(1, '        UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'lyapdlopts'     , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'  , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'   , ml_morlabopts('ml_sylv_sgn'), ...
                'StoreProjection', [], ...
                'Tolerance'      , [], ...
                'UnstabDim'      , []);
        end

    case 'ml_ct_ss_hna'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '  hankelsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '  hankelsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '   hankelsylvopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_sylv_sgn">ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n');
            else
                fprintf(1, '   hankelsylvopts: [ 1x1 struct | {ml_morlabopts(''ml_sylv_sgn'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n');
            end
            fprintf(1, '        MinRelTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv)) + Nu} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '     stabsylvopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_sylv_sgn">ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n');
            else
                fprintf(1, '     stabsylvopts: [ 1x1 struct | {ml_morlabopts(''ml_sylv_sgn'')} ]\n');
            end
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
            fprintf(1, '        UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'hankelsignmopts', ml_morlabopts('ml_signm'), ...
                'hankelsylvopts' , ml_morlabopts('ml_sylv_sgn'), ...
                'lyapdlopts'     , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'MinRelTol'      , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'  , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'   , ml_morlabopts('ml_sylv_sgn'), ...
                'Tolerance'      , [], ...
                'UnstabDim'      , []);
        end

    case 'ml_dt_ss_bt'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '      dlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dlyapdl_smith_fac">ml_morlabopts(''ml_dlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '      dlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_dlyapdl_smith_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv)) + Nu} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '    stabdsylvopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dsylv_smith">ml_morlabopts(''ml_dsylv_smith'')</a>} ]\n');
            else
                fprintf(1, '    stabdsylvopts: [ 1x1 struct | {ml_morlabopts(''ml_dsylv_smith'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
            fprintf(1, '        UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'dlyapdlopts'    , ml_morlabopts('ml_dlyapdl_smith_fac'), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'  , ml_morlabopts('ml_signm'), ...
                'stabdsylvopts'  , ml_morlabopts('ml_dsylv_smith'), ...
                'StoreProjection', [], ...
                'Tolerance'      , [], ...
                'UnstabDim'      , []);
        end

    case 'ml_ct_dss_bst'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '       caredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_caredl_sgn_fac">ml_morlabopts(''ml_caredl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       caredlopts: [ 1x1 struct | {ml_morlabopts(''ml_caredl_sgn_fac'')} ]\n');
            end
            fprintf(1, '        DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '        DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '          Epsilon: [ positive scalar | {1.0e-03} ]\n');
            if hasHyperlink
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_gdlyapdl_smith_fac">ml_morlabopts(''ml_gdlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_gdlyapdl_smith_fac'')} ]\n');
            end
            fprintf(1, '    ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Index: [ nonnegative integer | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '       infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '       infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '         lyapopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '         lyapopts: [ 1x1 struct | {ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsvp)) + Nu + Ni} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '        pcareopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '        pcareopts: [ 1x1 struct | {ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n');
            end
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sign''} ]\n');
            if hasHyperlink
                fprintf(1, '      stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'caredlopts'     , ml_morlabopts('ml_caredl_sgn_fac'), ...
                'DecompEig'      , [], ...
                'DecompTol'      , [], ...
                'Epsilon'        , [], ...
                'gdlyapdlopts'   , ml_morlabopts('ml_gdlyapdl_smith_fac'), ...
                'ImproperTrunc'  , [], ...
                'Index'          , [], ...
                'infdecopts'     , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'lyapopts'       , ml_morlabopts('ml_lyap_sgn_fac'), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'pcareopts'      , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'RiccatiSolver'  , [], ...
                'stabdecopts'    , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_ct_dss_flbt'
        if nargout == 0
            fprintf(1, '        DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '        DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '        FreqRange: [ nonnegative vector | {[0, 1.0e+03]} ]\n');
            if hasHyperlink
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_gdlyapdl_smith_fac">ml_morlabopts(''ml_gdlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_gdlyapdl_smith_fac'')} ]\n');
            end
            fprintf(1, '    ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Index: [ nonnegative integer | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '       infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '       infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or <a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_lyapdl_sgn_ldl'') or ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '       ModGramian: [ 1 | {0} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsvp)) + Nu + Ni} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | ''tolerance'' | {''sum''} ]\n');
            if hasHyperlink
                fprintf(1, '      stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'DecompEig'      , [], ...
                'DecompTol'      , [], ...
                'FreqRange'      , [], ...
                'gdlyapdlopts'   , ml_morlabopts('ml_gdlyapdl_smith_fac'), ...
                'ImproperTrunc'  , [], ...
                'Index'          , [], ...
                'infdecopts'     , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'lyapdlopts'     , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'         , [], ...
                'ModGramian'     , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'stabdecopts'    , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_ct_dss_mt'
        if nargout == 0
            fprintf(1, '           Alpha: [ real scalar | {-1.0} ]\n');
            fprintf(1, '       DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '       DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '       signmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '       signmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '      infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '         RankTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
        else
            opts = struct( ...
                'Alpha'         , [], ...
                'DecompEig'     , [], ...
                'DecompTol'     , [], ...
                'signmopts'     , ml_morlabopts('ml_signm'), ...
                'infdecopts'    , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'RankTol'       , [], ...
                'StoreProjection', []);
        end

    case 'ml_ct_dss_tlbt'
        if nargout == 0
            fprintf(1, '        DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '        DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_gdlyapdl_smith_fac">ml_morlabopts(''ml_gdlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_gdlyapdl_smith_fac'')} ]\n');
            end
            fprintf(1, '    ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Index: [ nonnegative integer | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '       infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '       infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or <a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_lyapdl_sgn_ldl'') or ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '       ModGramian: [ 1 | {0} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsvp)) + Nu + Ni} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | ''tolerance'' | {''sum''} ]\n');
            if hasHyperlink
                fprintf(1, '      stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        TimeRange: [ nonnegative vector | {[0, 10]} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'DecompEig'      , [], ...
                'DecompTol'      , [], ...
                'gdlyapdlopts'   , ml_morlabopts('ml_gdlyapdl_smith_fac'), ...
                'ImproperTrunc'  , [], ...
                'Index'          , [], ...
                'infdecopts'     , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'lyapdlopts'     , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'         , [], ...
                'ModGramian'     , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'stabdecopts'    , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'StoreProjection', [], ...
                'TimeRange'      , [], ...
                'Tolerance'      , []);
        end

    case 'ml_ct_ss_lqgbt'
        if nargout == 0
            fprintf(1, '             Beta: [ nonnegative scalar | {0.1} ]\n');
            if hasHyperlink
                fprintf(1, '       caredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_caredl_sgn_fac">ml_morlabopts(''ml_caredl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       caredlopts: [ 1x1 struct | {ml_morlabopts(''ml_caredl_sgn_fac'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '         careopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_care_nwt_fac">ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '         careopts: [ 1x1 struct | {ml_morlabopts(''ml_care_nwt_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv)) + Nu} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sign''} ]\n');
            fprintf(1, '       StabMethod: [ ''lyap'' | {''cabe''} ]\n');
            if hasHyperlink
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_cabe_sgn">ml_morlabopts(''ml_cabe_sgn'')</a> or <a href="matlab:ml_morlabopts ml_lyap_sgn">ml_morlabopts(''ml_lyap_sgn'')</a>} ]\n');
            else
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {ml_morlabopts(''ml_cabe_sgn'') or ml_morlabopts(''ml_lyap_sgn'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
            fprintf(1, '        UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'Beta'           , [], ...
                'caredlopts'     , ml_morlabopts('ml_caredl_sgn_fac'), ...
                'careopts'       , ml_morlabopts('ml_care_nwt_fac'), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'RiccatiSolver'  , [], ...
                'StabMethod'     , [], ...
                'stabmethodopts' , ml_morlabopts('ml_cabe_sgn'), ...
                'stabsignmopts'  , ml_morlabopts('ml_signm'), ...
                'StoreProjection', [], ...
                'Tolerance'      , [], ...
                'UnstabDim'      , []);
        end

    case 'ml_dt_dss_lqgbt'
        if nargout == 0
            fprintf(1, '             Beta: [ nonnegative scalar | {0.1} ]\n');
            if hasHyperlink
                fprintf(1, '       daredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_daredl_sda_fac">ml_morlabopts(''ml_daredl_sda_fac'')</a>} ]\n');
            else
                fprintf(1, '       daredlopts: [ 1x1 struct | {ml_morlabopts(''ml_daredl_sda_fac'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '         dareopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dare_nwt_fac">ml_morlabopts(''ml_dare_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '         dareopts: [ 1x1 struct | {ml_morlabopts(''ml_dare_nwt_fac'')} ]\n');
            end
            fprintf(1, '        DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '        DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_gdlyapdl_smith_fac">ml_morlabopts(''ml_gdlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_gdlyapdl_smith_fac'')} ]\n');
            end
            fprintf(1, '    ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Index: [ nonnegative integer | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '       infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '       infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsvp)) + Nu + Ni} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sda''} ]\n');
            if hasHyperlink
                fprintf(1, '      stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dlyap_smith">ml_morlabopts(''ml_dlyap_smith'')</a>} ]\n');
            else
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {ml_morlabopts(''ml_dlyap_smith'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'Beta'           , [], ...
                'daredlopts'     , ml_morlabopts('ml_daredl_sda_fac'), ...
                'dareopts'       , ml_morlabopts('ml_dare_nwt_fac'), ...
                'DecompEig'      , [], ...
                'DecompTol'      , [], ...
                'gdlyapdlopts'   , ml_morlabopts('ml_gdlyapdl_smith_fac'), ...
                'ImproperTrunc'  , [], ...
                'Index'          , [], ...
                'infdecopts'     , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'RiccatiSolver'  , [], ...
                'stabdecopts'    , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'stabmethodopts' , ml_morlabopts('ml_dlyap_smith'), ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_ct_dss_hna'
        if nargout == 0
            fprintf(1, '        DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '        DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_gdlyapdl_smith_fac">ml_morlabopts(''ml_gdlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_gdlyapdl_smith_fac'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '    hankeldecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '    hankeldecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '    ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Index: [ nonnegative integer | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '       infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '       infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n');
            end
            fprintf(1, '        MinRelTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsvp)) + Nu + Ni} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '      stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'DecompEig'      , [], ...
                'DecompTol'      , [], ...
                'gdlyapdlopts'   , ml_morlabopts('ml_gdlyapdl_smith_fac'), ...
                'hankeldecopts'  , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'ImproperTrunc'  , [], ...
                'Index'          , [], ...
                'infdecopts'     , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'lyapdlopts'     , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'MinRelTol'      , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'stabdecopts'    , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'Tolerance'      , []);
        end

    case 'ml_ct_soss_bt'
        if nargout == 0
            fprintf(1, '      BalanceType: [ ''p'' | ''pm'' | ''pv'' | ''vp'' | ''vpm'' | ''v'' | ''fv'' | {''so''} ]\n');
            if hasHyperlink
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv))} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | ''tolerance'' | {''sum''} ]\n');
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'BalanceType'    , [], ...
                'lyapdlopts'     , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_ct_soss_flbt'
        if nargout == 0
            fprintf(1, '      BalanceType: [ ''p'' | ''pm'' | ''pv'' | ''vp'' | ''vpm'' | ''v'' | ''fv'' | {''so''} ]\n');
            fprintf(1, '        FreqRange: [ nonnegative vector | {[0, 1.0e+03]} ]\n');
            if hasHyperlink
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or <a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_lyapdl_sgn_ldl'') or ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '       ModGramian: [ 1 | {0} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv))} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | ''tolerance'' | {''sum''} ]\n');
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'BalanceType'    , [], ...
                'FreqRange'      , [], ...
                'lyapdlopts'     , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'         , [], ...
                'ModGramian'     , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_ct_soss_tlbt'
        if nargout == 0
            fprintf(1, '      BalanceType: [ ''p'' | ''pm'' | ''pv'' | ''vp'' | ''vpm'' | ''v'' | ''fv'' | {''so''} ]\n');
            if hasHyperlink
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or <a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_lyapdl_sgn_ldl'') or ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '       ModGramian: [ 1 | {0} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv))} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | ''tolerance'' | {''sum''} ]\n');
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        TimeRange: [ nonnegative vector | {[0, 10]} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'BalanceType'    , [], ...
                'lyapdlopts'     , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'         , [], ...
                'ModGramian'     , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'StoreProjection', [], ...
                'TimeRange'      , [], ...
                'Tolerance'      , []);
        end

    case 'ml_dt_ss_lqgbt'
        if nargout == 0
            fprintf(1, '             Beta: [ nonnegative scalar | {0.1} ]\n');
            if hasHyperlink
                fprintf(1, '       daredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_daredl_sda_fac">ml_morlabopts(''ml_daredl_sda_fac'')</a>} ]\n');
            else
                fprintf(1, '       daredlopts: [ 1x1 struct | {ml_morlabopts(''ml_daredl_sda_fac'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '         dareopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dare_nwt_fac">ml_morlabopts(''ml_dare_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '         dareopts: [ 1x1 struct | {ml_morlabopts(''ml_dare_nwt_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv)) + Nu} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sda''} ]\n');
            if hasHyperlink
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dlyap_smith">ml_morlabopts(''ml_dlyap_smith'')</a>} ]\n');
            else
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {ml_morlabopts(''ml_dlyap_smith'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
            fprintf(1, '        UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'Beta'           , [], ...
                'daredlopts'     , ml_morlabopts('ml_daredl_sda_fac'), ...
                'dareopts'       , ml_morlabopts('ml_dare_nwt_fac'), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'RiccatiSolver'  , [], ...
                'stabmethodopts' , ml_morlabopts('ml_dlyap_smith'), ...
                'stabsignmopts'  , ml_morlabopts('ml_signm'), ...
                'StoreProjection', [], ...
                'Tolerance'      , [], ...
                'UnstabDim'      , []);
        end

    case 'ml_ct_ss_bst'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '       caredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_caredl_sgn_fac">ml_morlabopts(''ml_caredl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       caredlopts: [ 1x1 struct | {ml_morlabopts(''ml_caredl_sgn_fac'')} ]\n');
            end
            fprintf(1, '          Epsilon: [ positive scalar | {1.0e-03} ]\n');
            if hasHyperlink
                fprintf(1, '         lyapopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '         lyapopts: [ 1x1 struct | {ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv)) + Nu} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '        pcareopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '        pcareopts: [ 1x1 struct | {ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n');
            end
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sign''} ]\n');
            if hasHyperlink
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '     stabsylvopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_sylv_sgn">ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n');
            else
                fprintf(1, '     stabsylvopts: [ 1x1 struct | {ml_morlabopts(''ml_sylv_sgn'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
            fprintf(1, '        UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'caredlopts'     , ml_morlabopts('ml_caredl_sgn_fac'), ...
                'Epsilon'        , [], ...
                'lyapopts'       , ml_morlabopts('ml_lyap_sgn_fac'), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'pcareopts'      , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'RiccatiSolver'  , [], ...
                'stabsignmopts'  , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'   , ml_morlabopts('ml_sylv_sgn'), ...
                'StoreProjection', [], ...
                'Tolerance'      , [], ...
                'UnstabDim'      , []);
        end

    case 'ml_ct_ss_flbt'
        if nargout == 0
            fprintf(1, '        FreqRange: [ nonnegative vector | {[0, 1.0e+03]} ]\n');
            if hasHyperlink
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or <a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_lyapdl_sgn_ldl'') or ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '       ModGramian: [ 1 | {0} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv)) + Nu} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | ''tolerance'' | {''sum''} ]\n');
            if hasHyperlink
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '     stabsylvopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_sylv_sgn">ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n');
            else
                fprintf(1, '     stabsylvopts: [ 1x1 struct | {ml_morlabopts(''ml_sylv_sgn'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
            fprintf(1, '        UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'FreqRange'      , [], ...
                'lyapdlopts'     , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'         , [], ...
                'ModGramian'     , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'  , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'   , ml_morlabopts('ml_sylv_sgn'), ...
                'StoreProjection', [], ...
                'Tolerance'      , [], ...
                'UnstabDim'      , []);
        end

    case 'ml_ct_ss_tlbt'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or <a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_lyapdl_sgn_ldl'') or ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '       ModGramian: [ 1 | {0} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv)) + Nu} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | ''tolerance'' | {''sum''} ]\n');
            if hasHyperlink
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '     stabsylvopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_sylv_sgn">ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n');
            else
                fprintf(1, '     stabsylvopts: [ 1x1 struct | {ml_morlabopts(''ml_sylv_sgn'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        TimeRange: [ nonnegative vector | {[0, 10]} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
            fprintf(1, '        UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'lyapdlopts'     , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'         , [], ...
                'ModGramian'     , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'  , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'   , ml_morlabopts('ml_sylv_sgn'), ...
                'StoreProjection', [], ...
                'TimeRange'      , [], ...
                'Tolerance'      , [], ...
                'UnstabDim'      , []);
        end

    case 'ml_ct_dss_brbt'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '       caredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_caredl_sgn_fac">ml_morlabopts(''ml_caredl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       caredlopts: [ 1x1 struct | {ml_morlabopts(''ml_caredl_sgn_fac'')} ]\n');
            end
            fprintf(1, '        DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '        DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_gdlyapdl_smith_fac">ml_morlabopts(''ml_gdlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_gdlyapdl_smith_fac'')} ]\n');
            end
            fprintf(1, '    ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Index: [ nonnegative integer | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '       infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '       infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsvp)) + Ni} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '        pcareopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '        pcareopts: [ 1x1 struct | {ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n');
            end
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sign''} ]\n');
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'caredlopts'     , ml_morlabopts('ml_caredl_sgn_fac'), ...
                'DecompEig'      , [], ...
                'DecompTol'      , [], ...
                'gdlyapdlopts'   , ml_morlabopts('ml_gdlyapdl_smith_fac'), ...
                'ImproperTrunc'  , [], ...
                'Index'          , [], ...
                'infdecopts'     , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'pcareopts'      , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'RiccatiSolver'  , [], ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_ct_dss_bt'
        if nargout == 0
            fprintf(1, '        DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '        DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_gdlyapdl_smith_fac">ml_morlabopts(''ml_gdlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_gdlyapdl_smith_fac'')} ]\n');
            end
            fprintf(1, '    ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Index: [ nonnegative integer | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '       infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '       infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       lyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsvp)) + Nu + Ni} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '      stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'DecompEig'      , [], ...
                'DecompTol'      , [], ...
                'gdlyapdlopts'   , ml_morlabopts('ml_gdlyapdl_smith_fac'), ...
                'ImproperTrunc'  , [], ...
                'Index'          , [], ...
                'infdecopts'     , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'lyapdlopts'     , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'stabdecopts'    , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_ct_dss_hinfbt'
        if nargout == 0
            fprintf(1, '             Beta: [ nonnegative scalar | {0.1} ]\n');
            if hasHyperlink
                fprintf(1, '       caredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_caredl_sgn_fac">ml_morlabopts(''ml_caredl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       caredlopts: [ 1x1 struct | {ml_morlabopts(''ml_caredl_sgn_fac'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '         careopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_care_nwt_fac">ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '         careopts: [ 1x1 struct | {ml_morlabopts(''ml_care_nwt_fac'')} ]\n');
            end
            fprintf(1, '        DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '        DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Gamma: [ positive scalar | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_gdlyapdl_smith_fac">ml_morlabopts(''ml_gdlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_gdlyapdl_smith_fac'')} ]\n');
            end
            fprintf(1, '    ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Index: [ nonnegative integer | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '       infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '       infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsvp)) + Nu + Ni} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sign''} ]\n');
            if hasHyperlink
                fprintf(1, '      stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '       StabMethod: [ ''lyap'' | {''cabe''} ]\n');
            if hasHyperlink
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_cabe_sgn">ml_morlabopts(''ml_cabe_sgn'')</a> or <a href="matlab:ml_morlabopts ml_lyap_sgn">ml_morlabopts(''ml_lyap_sgn'')</a>} ]\n');
            else
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {ml_morlabopts(''ml_cabe_sgn'') or ml_morlabopts(''ml_lyap_sgn'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'Beta'           , [], ...
                'caredlopts'     , ml_morlabopts('ml_caredl_sgn_fac'), ...
                'careopts'       , ml_morlabopts('ml_care_nwt_fac'), ...
                'DecompEig'      , [], ...
                'DecompTol'      , [], ...
                'Gamma'          , [], ...
                'gdlyapdlopts'   , ml_morlabopts('ml_gdlyapdl_smith_fac'), ...
                'ImproperTrunc'  , [], ...
                'Index'          , [], ...
                'infdecopts'     , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'RiccatiSolver'  , [], ...
                'stabdecopts'    , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'StabMethod'     , [], ...
                'stabmethodopts' , ml_morlabopts('ml_cabe_sgn'), ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_ct_dss_lqgbt'
        if nargout == 0
            fprintf(1, '             Beta: [ nonnegative scalar | {0.1} ]\n');
            if hasHyperlink
                fprintf(1, '       caredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_caredl_sgn_fac">ml_morlabopts(''ml_caredl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       caredlopts: [ 1x1 struct | {ml_morlabopts(''ml_caredl_sgn_fac'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '         careopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_care_nwt_fac">ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '         careopts: [ 1x1 struct | {ml_morlabopts(''ml_care_nwt_fac'')} ]\n');
            end
            fprintf(1, '        DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '        DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_gdlyapdl_smith_fac">ml_morlabopts(''ml_gdlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_gdlyapdl_smith_fac'')} ]\n');
            end
            fprintf(1, '    ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Index: [ nonnegative integer | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '       infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '       infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsvp)) + Nu + Ni} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sign''} ]\n');
            if hasHyperlink
                fprintf(1, '      stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '       StabMethod: [ ''lyap'' | {''cabe''} ]\n');
            if hasHyperlink
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_cabe_sgn">ml_morlabopts(''ml_cabe_sgn'')</a> or <a href="matlab:ml_morlabopts ml_lyap_sgn">ml_morlabopts(''ml_lyap_sgn'')</a>} ]\n');
            else
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {ml_morlabopts(''ml_cabe_sgn'') or ml_morlabopts(''ml_lyap_sgn'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'Beta'           , [], ...
                'caredlopts'     , ml_morlabopts('ml_caredl_sgn_fac'), ...
                'careopts'       , ml_morlabopts('ml_care_nwt_fac'), ...
                'DecompEig'      , [], ...
                'DecompTol'      , [], ...
                'gdlyapdlopts'   , ml_morlabopts('ml_gdlyapdl_smith_fac'), ...
                'ImproperTrunc'  , [], ...
                'Index'          , [], ...
                'infdecopts'     , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'RiccatiSolver'  , [], ...
                'stabdecopts'    , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'StabMethod'     , [], ...
                'stabmethodopts' , ml_morlabopts('ml_cabe_sgn'), ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_ct_dss_prbt'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '       caredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_caredl_sgn_fac">ml_morlabopts(''ml_caredl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       caredlopts: [ 1x1 struct | {ml_morlabopts(''ml_caredl_sgn_fac'')} ]\n');
            end
            fprintf(1, '        DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '        DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '          Epsilon: [ positive scalar | {1.0e-03} ]\n');
            if hasHyperlink
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_gdlyapdl_smith_fac">ml_morlabopts(''ml_gdlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_gdlyapdl_smith_fac'')} ]\n');
            end
            fprintf(1, '    ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Index: [ nonnegative integer | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '       infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '       infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsvp)) + Ni} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '        pcareopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '        pcareopts: [ 1x1 struct | {ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n');
            end
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sign''} ]\n');
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'caredlopts'     , ml_morlabopts('ml_caredl_sgn_fac'), ...
                'DecompEig'      , [], ...
                'DecompTol'      , [], ...
                'Epsilon'        , [], ...
                'gdlyapdlopts'   , ml_morlabopts('ml_gdlyapdl_smith_fac'), ...
                'ImproperTrunc'  , [], ...
                'Index'          , [], ...
                'infdecopts'     , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'pcareopts'      , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'RiccatiSolver'  , [], ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_ct_ss_hinfbt'
        if nargout == 0
            fprintf(1, '             Beta: [ nonnegative scalar | {0.1} ]\n');
            if hasHyperlink
                fprintf(1, '       caredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_caredl_sgn_fac">ml_morlabopts(''ml_caredl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       caredlopts: [ 1x1 struct | {ml_morlabopts(''ml_caredl_sgn_fac'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '         careopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_care_nwt_fac">ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '         careopts: [ 1x1 struct | {ml_morlabopts(''ml_care_nwt_fac'')} ]\n');
            end
            fprintf(1, '            Gamma: [ positive scalar | {Inf} ]\n');
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv)) + Nu} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sign''} ]\n');
            fprintf(1, '       StabMethod: [ ''lyap'' | {''cabe''} ]\n');
            if hasHyperlink
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_cabe_sgn">ml_morlabopts(''ml_cabe_sgn'')</a> or <a href="matlab:ml_morlabopts ml_lyap_sgn">ml_morlabopts(''ml_lyap_sgn'')</a>} ]\n');
            else
                fprintf(1, '   stabmethodopts: [ 1x1 struct | {ml_morlabopts(''ml_cabe_sgn'') or ml_morlabopts(''ml_lyap_sgn'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '    stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
            fprintf(1, '        UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'Beta'           , [], ...
                'caredlopts'     , ml_morlabopts('ml_caredl_sgn_fac'), ...
                'careopts'       , ml_morlabopts('ml_care_nwt_fac'), ...
                'Gamma'          , [], ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'RiccatiSolver'  , [], ...
                'StabMethod'     , [], ...
                'stabmethodopts' , ml_morlabopts('ml_cabe_sgn'), ...
                'stabsignmopts'  , ml_morlabopts('ml_signm'), ...
                'StoreProjection', [], ...
                'Tolerance'      , [], ...
                'UnstabDim'      , []);
        end

    case 'ml_dt_dss_bt'
        if nargout == 0
            fprintf(1, '        DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '        DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '      dlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dlyapdl_smith_fac">ml_morlabopts(''ml_dlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '      dlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_dlyapdl_smith_fac'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_gdlyapdl_smith_fac">ml_morlabopts(''ml_gdlyapdl_smith_fac'')</a>} ]\n');
            else
                fprintf(1, '     gdlyapdlopts: [ 1x1 struct | {ml_morlabopts(''ml_gdlyapdl_smith_fac'')} ]\n');
            end
            fprintf(1, '    ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '            Index: [ nonnegative integer | {Inf} ]\n');
            if hasHyperlink
                fprintf(1, '       infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '       infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsvp)) + Nu + Ni} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '      stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'DecompEig'      , [], ...
                'DecompTol'      , [], ...
                'dlyapdlopts'    , ml_morlabopts('ml_dlyapdl_smith_fac'), ...
                'gdlyapdlopts'   , ml_morlabopts('ml_gdlyapdl_smith_fac'), ...
                'ImproperTrunc'  , [], ...
                'Index'          , [], ...
                'infdecopts'     , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'stabdecopts'    , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_dt_dss_mt'
        if nargout == 0
            fprintf(1, '           Alpha: [ real scalar | {-1.0} ]\n');
            fprintf(1, '       DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '       DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '       signmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '       signmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '      infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '         RankTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
        else
            opts = struct( ...
                'Alpha'         , [], ...
                'DecompEig'     , [], ...
                'DecompTol'     , [], ...
                'signmopts'     , ml_morlabopts('ml_signm'), ...
                'infdecopts'    , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'RankTol'       , [], ...
                'StoreProjection', []);
        end

    case 'ml_ct_ss_brbt'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '       caredlopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_caredl_sgn_fac">ml_morlabopts(''ml_caredl_sgn_fac'')</a>} ]\n');
            else
                fprintf(1, '       caredlopts: [ 1x1 struct | {ml_morlabopts(''ml_caredl_sgn_fac'')} ]\n');
            end
            fprintf(1, '           Method: [ ''bfsr'' | {''sr''} ]\n');
            fprintf(1, '            Order: [ positive integer | {min(10,length(Hsv)) + Nu} ]\n');
            fprintf(1, ' OrderComputation: [ ''order'' | {''tolerance''} ]\n');
            if hasHyperlink
                fprintf(1, '        pcareopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n');
            else
                fprintf(1, '        pcareopts: [ 1x1 struct | {ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n');
            end
            fprintf(1, '    RiccatiSolver: [ ''newton'' | {''sign''} ]\n');
            fprintf(1, '  StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '        Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'caredlopts'     , ml_morlabopts('ml_caredl_sgn_fac'), ...
                'Method'         , [], ...
                'Order'          , [], ...
                'OrderComputation', [], ...
                'pcareopts'      , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'RiccatiSolver'  , [], ...
                'StoreProjection', [], ...
                'Tolerance'      , []);
        end

    case 'ml_compress_fac'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_ct_dss_adtf'
        if nargout == 0
            fprintf(1, '       DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '       DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '      infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '     stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '     stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
        else
            opts = struct( ...
                'DecompEig'     , [], ...
                'DecompTol'     , [], ...
                'infdecopts'    , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'stabdecopts'   , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'StoreProjection', []);
        end

    case 'ml_ct_ss_adtf'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '   stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '   stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '    stabsylvopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_sylv_sgn">ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n');
            else
                fprintf(1, '    stabsylvopts: [ 1x1 struct | {ml_morlabopts(''ml_sylv_sgn'')} ]\n');
            end
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '       UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'stabsignmopts' , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'  , ml_morlabopts('ml_sylv_sgn'), ...
                'StoreProjection', [], ...
                'UnstabDim'     , []);
        end

    case 'ml_dt_dss_partstab'
        if nargout == 0
            fprintf(1, '           Beta: [ positive scalar < 1 | {0.1} ]\n');
            fprintf(1, '      DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '      DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '     infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '     infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '    stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '    stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, ' stabmethodopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dlyap_smith">ml_morlabopts(''ml_dlyap_smith'')</a>} ]\n');
            else
                fprintf(1, ' stabmethodopts: [ 1x1 struct | {ml_morlabopts(''ml_dlyap_smith'')} ]\n');
            end
        else
            opts = struct( ...
                'Beta'         , [], ...
                'DecompEig'    , [], ...
                'DecompTol'    , [], ...
                'infdecopts'   , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'stabdecopts'  , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'stabmethodopts', ml_morlabopts('ml_dlyap_smith'));
        end

    case 'ml_soss_bfsr'
        if nargout == 0
            fprintf(1, '     BalanceType: [ ''p'' | ''pm'' | ''pv'' | ''vp'' | ''vpm'' | ''v'' | ''fv'' | {''so''} ]\n');
            fprintf(1, '           Gamma: [ positive scalar | {Inf} ]\n');
            fprintf(1, '           Order: [ positive integer | {min(10,length(hsv))} ]\n');
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '       Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'BalanceType'   , [], ...
                'Gamma'         , [], ...
                'Order'         , [], ...
                'StoreProjection', [], ...
                'Tolerance'     , []);
        end

    case 'ml_ss_bfsr'
        if nargout == 0
            fprintf(1, '           Gamma: [ positive scalar | {Inf} ]\n');
            fprintf(1, '           Order: [ positive integer | {min(10,length(hsv)) + nu} ]\n');
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '       Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'Gamma'         , [], ...
                'Order'         , [], ...
                'StoreProjection', [], ...
                'Tolerance'     , []);
        end

    case 'ml_ct_dss_partstab'
        if nargout == 0
            fprintf(1, '           Beta: [ nonnegative scalar | {0.1} ]\n');
            fprintf(1, '      DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '      DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '     infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '     infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '    stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '    stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, '     StabMethod: [ ''lyap'' | {''cabe''} ]\n');
            if hasHyperlink
                fprintf(1, ' stabmethodopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_cabe_sgn">ml_morlabopts(''ml_cabe_sgn'')</a> or <a href="matlab:ml_morlabopts ml_lyap_sgn">ml_morlabopts(''ml_lyap_sgn'')</a>} ]\n');
            else
                fprintf(1, ' stabmethodopts: [ 1x1 struct | {ml_morlabopts(''ml_cabe_sgn'') or ml_morlabopts(''ml_lyap_sgn'')} ]\n');
            end
        else
            opts = struct( ...
                'Beta'         , [], ...
                'DecompEig'    , [], ...
                'DecompTol'    , [], ...
                'infdecopts'   , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'stabdecopts'  , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'StabMethod'   , [], ...
                'stabmethodopts', ml_morlabopts('ml_cabe_sgn'));
        end

    case 'ml_dss_sr'
        if nargout == 0
            fprintf(1, '           Gamma: [ positive scalar | {Inf} ]\n');
            fprintf(1, '   ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '           Index: [ nonnegative integer | {Inf} ]\n');
            fprintf(1, '           Order: [ positive integer | {min(10,length(hsvp)) + nu + ni} ]\n');
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '       Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'Gamma'         , [], ...
                'ImproperTrunc' , [], ...
                'Index'         , [], ...
                'Order'         , [], ...
                'StoreProjection', [], ...
                'Tolerance'     , []);
        end

    case 'ml_compress_ldl'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_disk'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+02*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_getqz'
        if nargout == 0
            fprintf(1, ' Dimension: [ integer | {-1} ]\n');
            fprintf(1, '   RankTol: [ nonnegative scalar | {log(n)*eps} ]\n');
        else
            opts = struct( ...
                'Dimension', [], ...
                'RankTol' , []);
        end

    case 'ml_order'
        if nargout == 0
            fprintf(1, '     Gamma: [ positive scalar | {Inf} ]\n');
            fprintf(1, '     Order: [ positive integer | {min(10,length(hsv)) + nu + ni} ]\n');
            fprintf(1, ' Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'Gamma'   , [], ...
                'Order'   , [], ...
                'Tolerance', []);
        end

    case 'ml_dss_format_output'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_dt_ss_adtf'
        if nargout == 0
            if hasHyperlink
                fprintf(1, '   stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '   stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '   stabdsylvopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dsylv_smith">ml_morlabopts(''ml_dsylv_smith'')</a>} ]\n');
            else
                fprintf(1, '   stabdsylvopts: [ 1x1 struct | {ml_morlabopts(''ml_dsylv_smith'')} ]\n');
            end
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '       UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'stabsignmopts' , ml_morlabopts('ml_signm'), ...
                'stabdsylvopts' , ml_morlabopts('ml_dsylv_smith'), ...
                'StoreProjection', [], ...
                'UnstabDim'     , []);
        end

    case 'ml_soss_format_output'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_ss_sr'
        if nargout == 0
            fprintf(1, '           Gamma: [ positive scalar | {Inf} ]\n');
            fprintf(1, '           Order: [ positive integer | {min(10,length(hsv)) + nu} ]\n');
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '       Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'Gamma'         , [], ...
                'Order'         , [], ...
                'StoreProjection', [], ...
                'Tolerance'     , []);
        end

    case 'ml_ct_ss_partstab'
        if nargout == 0
            fprintf(1, '           Beta: [ nonnegative scalar | {0.1} ]\n');
            fprintf(1, '     StabMethod: [ ''lyap'' | {''cabe''} ]\n');
            if hasHyperlink
                fprintf(1, ' stabmethodopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_cabe_sgn">ml_morlabopts(''ml_cabe_sgn'')</a> or <a href="matlab:ml_morlabopts ml_lyap_sgn">ml_morlabopts(''ml_lyap_sgn'')</a>} ]\n');
            else
                fprintf(1, ' stabmethodopts: [ 1x1 struct | {ml_morlabopts(''ml_cabe_sgn'') or ml_morlabopts(''ml_lyap_sgn'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '  stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '  stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            fprintf(1, '      UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'Beta'         , [], ...
                'StabMethod'   , [], ...
                'stabmethodopts', ml_morlabopts('ml_cabe_sgn'), ...
                'stabsignmopts', ml_morlabopts('ml_signm'), ...
                'UnstabDim'    , []);
        end

    case 'ml_morlabopts'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_signm'
        if nargout == 0
            fprintf(1, '  AbsTol: [ nonnegative scalar | {0} ]\n');
            fprintf(1, '    Info: [ 1 | {0} ]\n');
            fprintf(1, ' MaxIter: [ positive integer | {100} ]\n');
            fprintf(1, '  RelTol: [ nonnegative scalar | {1.0e+02*n*eps} ]\n');
        else
            opts = struct( ...
                'AbsTol', [], ...
                'Info'  , [], ...
                'MaxIter', [], ...
                'RelTol', []);
        end

    case 'ml_extend_cell'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_dss_lti2struct'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_soss_lti2struct'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_ss_lti2struct'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_dt_ss_partstab'
        if nargout == 0
            fprintf(1, '           Beta: [ positive scalar < 1 | {0.1} ]\n');
            if hasHyperlink
                fprintf(1, ' stabmethodopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_dlyap_smith">ml_morlabopts(''ml_dlyap_smith'')</a>} ]\n');
            else
                fprintf(1, ' stabmethodopts: [ 1x1 struct | {ml_morlabopts(''ml_dlyap_smith'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '  stabsignmopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>} ]\n');
            else
                fprintf(1, '  stabsignmopts: [ 1x1 struct | {ml_morlabopts(''ml_signm'')} ]\n');
            end
            fprintf(1, '      UnstabDim: [ integer | {-1} ]\n');
        else
            opts = struct( ...
                'Beta'         , [], ...
                'stabmethodopts', ml_morlabopts('ml_dlyap_smith'), ...
                'stabsignmopts', ml_morlabopts('ml_signm'), ...
                'UnstabDim'    , []);
        end

    case 'ml_dt_dss_adtf'
        if nargout == 0
            fprintf(1, '       DecompEig: [ positive scalar | {[]} ]\n');
            fprintf(1, '       DecompTol: [ nonnegative scalar | {log(n)*eps} ]\n');
            if hasHyperlink
                fprintf(1, '      infdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_disk">ml_morlabopts(''ml_disk'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '      infdecopts: [ 1x1 struct | {ml_morlabopts(''ml_disk''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            if hasHyperlink
                fprintf(1, '     stabdecopts: [ 1x1 struct | {<a href="matlab:ml_morlabopts ml_signm">ml_morlabopts(''ml_signm'')</a>, <a href="matlab:ml_morlabopts ml_getqz">ml_morlabopts(''ml_getqz'')</a>} ]\n');
            else
                fprintf(1, '     stabdecopts: [ 1x1 struct | {ml_morlabopts(''ml_signm''), ml_morlabopts(''ml_getqz'')} ]\n');
            end
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
        else
            opts = struct( ...
                'DecompEig'     , [], ...
                'DecompTol'     , [], ...
                'infdecopts'    , merge_structs(ml_morlabopts('ml_disk'), ml_morlabopts('ml_getqz')), ...
                'stabdecopts'   , merge_structs(ml_morlabopts('ml_signm'), ml_morlabopts('ml_getqz')), ...
                'StoreProjection', []);
        end

    case 'ml_soss_sr'
        if nargout == 0
            fprintf(1, '     BalanceType: [ ''p'' | ''pm'' | ''pv'' | ''vp'' | ''vpm'' | ''v'' | ''fv'' | {''so''} ]\n');
            fprintf(1, '           Gamma: [ positive scalar | {Inf} ]\n');
            fprintf(1, '           Order: [ positive integer | {min(10,length(hsv))} ]\n');
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '       Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'BalanceType'   , [], ...
                'Gamma'         , [], ...
                'Order'         , [], ...
                'StoreProjection', [], ...
                'Tolerance'     , []);
        end

    case 'ml_ss_format_output'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    case 'ml_dss_bfsr'
        if nargout == 0
            fprintf(1, '           Gamma: [ positive scalar | {Inf} ]\n');
            fprintf(1, '   ImproperTrunc: [ nonnegative scalar | {log(n)*eps} ]\n');
            fprintf(1, '           Index: [ nonnegative integer | {Inf} ]\n');
            fprintf(1, '           Order: [ positive integer | {min(10,length(hsvp)) + nu + ni} ]\n');
            fprintf(1, ' StoreProjection: [ 1 | {0} ]\n');
            fprintf(1, '       Tolerance: [ nonnegative scalar | {1.0e-02} ]\n');
        else
            opts = struct( ...
                'Gamma'         , [], ...
                'ImproperTrunc' , [], ...
                'Index'         , [], ...
                'Order'         , [], ...
                'StoreProjection', [], ...
                'Tolerance'     , []);
        end

    case 'ml_ct_dss_simulate_ie'
        if nargout == 0
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '    InputFcn: [ function handle | {ones(m, 1)} ]\n');
            fprintf(1, '  InputRange: [ vector of length <= 2 | {opts.TimeRange} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
            fprintf(1, ' StoreStates: [ 1 | {0} ]\n');
            fprintf(1, '  TimePoints: [ positive integer | {500} ]\n');
            fprintf(1, '   TimeRange: [ vector of length <= 2 | {[0 10]} ]\n');
            fprintf(1, '  TimeSample: [ ''equalstep'' | [numeric] | {''equalpts''} ]\n');
            fprintf(1, '    TimeStep: [ scalar | {(tf - t0) / 500} ]\n');
            fprintf(1, '          X0: [ vector | {zeros(n, 1)} ]\n');
        else
            opts = struct( ...
                'DiffMode'  , [], ...
                'Info'      , [], ...
                'InputFcn'  , [], ...
                'InputRange', [], ...
                'LineSpecs' , [], ...
                'ShowPlot'  , [], ...
                'StoreStates', [], ...
                'TimePoints', [], ...
                'TimeRange' , [], ...
                'TimeSample', [], ...
                'TimeStep'  , [], ...
                'X0'        , []);
        end

    case 'ml_ct_soss_simulate_ie'
        if nargout == 0
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '    InputFcn: [ function handle | {ones(m, 1)} ]\n');
            fprintf(1, '  InputRange: [ vector of length <= 2 | {opts.TimeRange} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
            fprintf(1, ' StoreStates: [ 1 | {0} ]\n');
            fprintf(1, '  TimePoints: [ positive integer | {500} ]\n');
            fprintf(1, '   TimeRange: [ vector of length <= 2 | {[0 10]} ]\n');
            fprintf(1, '  TimeSample: [ ''equalstep'' | [numeric] | {''equalpts''} ]\n');
            fprintf(1, '    TimeStep: [ scalar | {(tf - t0) / 500} ]\n');
            fprintf(1, '         Xp0: [ vector | {zeros(n, 1)} ]\n');
            fprintf(1, '         Xv0: [ vector | {zeros(n, 1)} ]\n');
        else
            opts = struct( ...
                'DiffMode'  , [], ...
                'Info'      , [], ...
                'InputFcn'  , [], ...
                'InputRange', [], ...
                'LineSpecs' , [], ...
                'ShowPlot'  , [], ...
                'StoreStates', [], ...
                'TimePoints', [], ...
                'TimeRange' , [], ...
                'TimeSample', [], ...
                'TimeStep'  , [], ...
                'Xp0'       , [], ...
                'Xv0'       , []);
        end

    case 'ml_ct_dss_simulate_ss11'
        if nargout == 0
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '    InputFcn: [ function handle | {ones(m, 1)} ]\n');
            fprintf(1, '  InputRange: [ vector of length <= 2 | {opts.TimeRange} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
            fprintf(1, ' StoreStates: [ 1 | {0} ]\n');
            fprintf(1, '       Theta: [ nonnegative scalar | {0.5} ]\n');
            fprintf(1, '  TimePoints: [ positive integer | {500} ]\n');
            fprintf(1, '   TimeRange: [ vector of length <= 2 | {[0 10]} ]\n');
            fprintf(1, '  TimeSample: [ ''equalstep'' | [numeric] | {''equalpts''} ]\n');
            fprintf(1, '    TimeStep: [ scalar | {(tf - t0) / 500} ]\n');
            fprintf(1, '          X0: [ vector | {zeros(n, 1)} ]\n');
        else
            opts = struct( ...
                'DiffMode'  , [], ...
                'Info'      , [], ...
                'InputFcn'  , [], ...
                'InputRange', [], ...
                'LineSpecs' , [], ...
                'ShowPlot'  , [], ...
                'StoreStates', [], ...
                'Theta'     , [], ...
                'TimePoints', [], ...
                'TimeRange' , [], ...
                'TimeSample', [], ...
                'TimeStep'  , [], ...
                'X0'        , []);
        end

    case 'ml_ct_dss_simulate_ss21'
        if nargout == 0
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '    InputFcn: [ function handle | {ones(m, 1)} ]\n');
            fprintf(1, '  InputRange: [ vector of length <= 2 | {opts.TimeRange} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
            fprintf(1, ' StoreStates: [ 1 | {0} ]\n');
            fprintf(1, '      Theta1: [ nonnegative scalar | {0.5} ]\n');
            fprintf(1, '      Theta2: [ nonnegative scalar | {0.5} ]\n');
            fprintf(1, '  TimePoints: [ positive integer | {500} ]\n');
            fprintf(1, '   TimeRange: [ vector of length <= 2 | {[0 10]} ]\n');
            fprintf(1, '  TimeSample: [ ''equalstep'' | [numeric] | {''equalpts''} ]\n');
            fprintf(1, '    TimeStep: [ scalar | {(tf - t0) / 500} ]\n');
            fprintf(1, '          X0: [ vector | {zeros(n, 1)} ]\n');
        else
            opts = struct( ...
                'DiffMode'  , [], ...
                'Info'      , [], ...
                'InputFcn'  , [], ...
                'InputRange', [], ...
                'LineSpecs' , [], ...
                'ShowPlot'  , [], ...
                'StoreStates', [], ...
                'Theta1'    , [], ...
                'Theta2'    , [], ...
                'TimePoints', [], ...
                'TimeRange' , [], ...
                'TimeSample', [], ...
                'TimeStep'  , [], ...
                'X0'        , []);
        end

    case 'ml_ct_soss_simulate_ss22'
        if nargout == 0
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '    InputFcn: [ function handle | {ones(m, 1)} ]\n');
            fprintf(1, '  InputRange: [ vector of length <= 2 | {opts.TimeRange} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
            fprintf(1, ' StoreStates: [ 1 | {0} ]\n');
            fprintf(1, '      Theta1: [ nonnegative scalar | {0.5} ]\n');
            fprintf(1, '      Theta2: [ nonnegative scalar | {0.5} ]\n');
            fprintf(1, '  TimePoints: [ positive integer | {500} ]\n');
            fprintf(1, '   TimeRange: [ vector of length <= 2 | {[0 10]} ]\n');
            fprintf(1, '  TimeSample: [ ''equalstep'' | [numeric] | {''equalpts''} ]\n');
            fprintf(1, '    TimeStep: [ scalar | {(tf - t0) / 500} ]\n');
            fprintf(1, '         Xp0: [ vector | {zeros(n, 1)} ]\n');
            fprintf(1, '         Xv0: [ vector | {zeros(n, 1)} ]\n');
        else
            opts = struct( ...
                'DiffMode'  , [], ...
                'Info'      , [], ...
                'InputFcn'  , [], ...
                'InputRange', [], ...
                'LineSpecs' , [], ...
                'ShowPlot'  , [], ...
                'StoreStates', [], ...
                'Theta1'    , [], ...
                'Theta2'    , [], ...
                'TimePoints', [], ...
                'TimeRange' , [], ...
                'TimeSample', [], ...
                'TimeStep'  , [], ...
                'Xp0'       , [], ...
                'Xv0'       , []);
        end

    case 'ml_ct_ss_simulate_ie'
        if nargout == 0
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '    InputFcn: [ function handle | {ones(m, 1)} ]\n');
            fprintf(1, '  InputRange: [ vector of length <= 2 | {opts.TimeRange} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
            fprintf(1, ' StoreStates: [ 1 | {0} ]\n');
            fprintf(1, '  TimePoints: [ positive integer | {500} ]\n');
            fprintf(1, '   TimeRange: [ vector of length <= 2 | {[0 10]} ]\n');
            fprintf(1, '  TimeSample: [ ''equalstep'' | [numeric] | {''equalpts''} ]\n');
            fprintf(1, '    TimeStep: [ scalar | {(tf - t0) / 500} ]\n');
            fprintf(1, '          X0: [ vector | {zeros(n, 1)} ]\n');
        else
            opts = struct( ...
                'DiffMode'  , [], ...
                'Info'      , [], ...
                'InputFcn'  , [], ...
                'InputRange', [], ...
                'LineSpecs' , [], ...
                'ShowPlot'  , [], ...
                'StoreStates', [], ...
                'TimePoints', [], ...
                'TimeRange' , [], ...
                'TimeSample', [], ...
                'TimeStep'  , [], ...
                'X0'        , []);
        end

    case 'ml_ct_ss_simulate_ss11'
        if nargout == 0
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '    InputFcn: [ function handle | {ones(m, 1)} ]\n');
            fprintf(1, '  InputRange: [ vector of length <= 2 | {opts.TimeRange} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
            fprintf(1, ' StoreStates: [ 1 | {0} ]\n');
            fprintf(1, '       Theta: [ nonnegative scalar | {0.5} ]\n');
            fprintf(1, '  TimePoints: [ positive integer | {500} ]\n');
            fprintf(1, '   TimeRange: [ vector of length <= 2 | {[0 10]} ]\n');
            fprintf(1, '  TimeSample: [ ''equalstep'' | [numeric] | {''equalpts''} ]\n');
            fprintf(1, '    TimeStep: [ scalar | {(tf - t0) / 500} ]\n');
            fprintf(1, '          X0: [ vector | {zeros(n, 1)} ]\n');
        else
            opts = struct( ...
                'DiffMode'  , [], ...
                'Info'      , [], ...
                'InputFcn'  , [], ...
                'InputRange', [], ...
                'LineSpecs' , [], ...
                'ShowPlot'  , [], ...
                'StoreStates', [], ...
                'Theta'     , [], ...
                'TimePoints', [], ...
                'TimeRange' , [], ...
                'TimeSample', [], ...
                'TimeStep'  , [], ...
                'X0'        , []);
        end

    case 'ml_ct_ss_simulate_ss21'
        if nargout == 0
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '    InputFcn: [ function handle | {ones(m, 1)} ]\n');
            fprintf(1, '  InputRange: [ vector of length <= 2 | {opts.TimeRange} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
            fprintf(1, ' StoreStates: [ 1 | {0} ]\n');
            fprintf(1, '      Theta1: [ nonnegative scalar | {0.5} ]\n');
            fprintf(1, '      Theta2: [ nonnegative scalar | {0.5} ]\n');
            fprintf(1, '  TimePoints: [ positive integer | {500} ]\n');
            fprintf(1, '   TimeRange: [ vector of length <= 2 | {[0 10]} ]\n');
            fprintf(1, '  TimeSample: [ ''equalstep'' | [numeric] | {''equalpts''} ]\n');
            fprintf(1, '    TimeStep: [ scalar | {(tf - t0) / 500} ]\n');
            fprintf(1, '          X0: [ vector | {zeros(n, 1)} ]\n');
        else
            opts = struct( ...
                'DiffMode'  , [], ...
                'Info'      , [], ...
                'InputFcn'  , [], ...
                'InputRange', [], ...
                'LineSpecs' , [], ...
                'ShowPlot'  , [], ...
                'StoreStates', [], ...
                'Theta1'    , [], ...
                'Theta2'    , [], ...
                'TimePoints', [], ...
                'TimeRange' , [], ...
                'TimeSample', [], ...
                'TimeStep'  , [], ...
                'X0'        , []);
        end

    case 'ml_dt_dss_simulate'
        if nargout == 0
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '    InputFcn: [ function handle | {ones(m, 1)} ]\n');
            fprintf(1, '  InputRange: [ vector of length <= 2 | {opts.TimeRange} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
            fprintf(1, ' StoreStates: [ 1 | {0} ]\n');
            fprintf(1, '   TimeRange: [ nonnegative vector of length <= 2 | {[0 10]} ]\n');
            fprintf(1, '          X0: [ vector | {zeros(n, 1)} ]\n');
        else
            opts = struct( ...
                'DiffMode'  , [], ...
                'Info'      , [], ...
                'InputFcn'  , [], ...
                'InputRange', [], ...
                'LineSpecs' , [], ...
                'ShowPlot'  , [], ...
                'StoreStates', [], ...
                'TimeRange' , [], ...
                'X0'        , []);
        end

    case 'ml_dt_ss_simulate'
        if nargout == 0
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '    InputFcn: [ function handle | {ones(m, 1)} ]\n');
            fprintf(1, '  InputRange: [ vector of length <= 2 | {opts.TimeRange} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
            fprintf(1, ' StoreStates: [ 1 | {0} ]\n');
            fprintf(1, '   TimeRange: [ nonnegative vector of length <= 2 | {[0 10]} ]\n');
            fprintf(1, '          X0: [ vector | {zeros(n, 1)} ]\n');
        else
            opts = struct( ...
                'DiffMode'  , [], ...
                'Info'      , [], ...
                'InputFcn'  , [], ...
                'InputRange', [], ...
                'LineSpecs' , [], ...
                'ShowPlot'  , [], ...
                'StoreStates', [], ...
                'TimeRange' , [], ...
                'X0'        , []);
        end

    case 'ml_bodemag'
        if nargout == 0
            fprintf(1, ' AccuracyTol: [ nonnegative scalar | {0.1} ]\n');
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '   FreqRange: [ vector of length 2 | {[-4, 4]} ]\n');
            fprintf(1, '  FreqSample: [ ''equal'' | [numeric] | {''adapt''} ]\n');
            fprintf(1, '    FreqUnit: [ ''Hz'' | {''rad/sec''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '  InitPoints: [ positive integer >= 3 or vector of length >= 3 | {3} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    MagScale: [ ''linear'' | {''log''} ]\n');
            fprintf(1, '     MagUnit: [ ''dB'' | {''abs''} ]\n');
            fprintf(1, '   MaxPoints: [ positive integer | {500} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
        else
            opts = struct( ...
                'AccuracyTol', [], ...
                'DiffMode'  , [], ...
                'FreqRange' , [], ...
                'FreqSample', [], ...
                'FreqUnit'  , [], ...
                'Info'      , [], ...
                'InitPoints', [], ...
                'LineSpecs' , [], ...
                'MagScale'  , [], ...
                'MagUnit'   , [], ...
                'MaxPoints' , [], ...
                'ShowPlot'  , []);
        end

    case 'ml_frobeniusplot'
        if nargout == 0
            fprintf(1, ' AccuracyTol: [ nonnegative scalar | {0.1} ]\n');
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '   FreqRange: [ vector of length 2 | {[-4, 4]} ]\n');
            fprintf(1, '  FreqSample: [ ''equal'' | [numeric] | {''adapt''} ]\n');
            fprintf(1, '    FreqUnit: [ ''Hz'' | {''rad/sec''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '  InitPoints: [ positive integer >= 3 or vector of length >= 3 | {3} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    MagScale: [ ''linear'' | {''log''} ]\n');
            fprintf(1, '     MagUnit: [ ''dB'' | {''abs''} ]\n');
            fprintf(1, '   MaxPoints: [ positive integer | {500} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
        else
            opts = struct( ...
                'AccuracyTol', [], ...
                'DiffMode'  , [], ...
                'FreqRange' , [], ...
                'FreqSample', [], ...
                'FreqUnit'  , [], ...
                'Info'      , [], ...
                'InitPoints', [], ...
                'LineSpecs' , [], ...
                'MagScale'  , [], ...
                'MagUnit'   , [], ...
                'MaxPoints' , [], ...
                'ShowPlot'  , []);
        end

    case 'ml_sigmaplot'
        if nargout == 0
            fprintf(1, ' AccuracyTol: [ nonnegative scalar | {0.1} ]\n');
            fprintf(1, '    DiffMode: [ ''abs'' | ''rel'' | ''all'' | {''off''} ]\n');
            fprintf(1, '   FreqRange: [ vector of length 2 | {[-4, 4]} ]\n');
            fprintf(1, '  FreqSample: [ ''equal'' | [numeric] | {''adapt''} ]\n');
            fprintf(1, '    FreqUnit: [ ''Hz'' | {''rad/sec''} ]\n');
            fprintf(1, '        Info: [ 1 | {0} ]\n');
            fprintf(1, '  InitPoints: [ positive integer >= 3 or vector of length >= 3 | {3} ]\n');
            fprintf(1, '   LineSpecs: [ character array | {''-''} ]\n');
            fprintf(1, '    MagScale: [ ''linear'' | {''log''} ]\n');
            fprintf(1, '     MagUnit: [ ''dB'' | {''abs''} ]\n');
            fprintf(1, '   MaxPoints: [ positive integer | {500} ]\n');
            fprintf(1, '    ShowPlot: [ 0 | {1} ]\n');
        else
            opts = struct( ...
                'AccuracyTol', [], ...
                'DiffMode'  , [], ...
                'FreqRange' , [], ...
                'FreqSample', [], ...
                'FreqUnit'  , [], ...
                'Info'      , [], ...
                'InitPoints', [], ...
                'LineSpecs' , [], ...
                'MagScale'  , [], ...
                'MagUnit'   , [], ...
                'MaxPoints' , [], ...
                'ShowPlot'  , []);
        end

    case 'ml_addpath'
        if nargout == 0
            fprintf(1, ' The requested function doesn''t use an option struct!\n');
        else
            opts = struct([]);
        end

    otherwise % Non-MORLAB functions.
        error('MORLAB:data', ...
            'The given function %s is not implemented in MORLAB.', fname);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DEFAULT HELP LINE.                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Default help link for more details.
if nargout == 0
    fprintf(1, '\n');
    if hasHyperlink
        fprintf(1, [' For more details see <a href="matlab:help ' ...
            '%s">%s</a>.\n'], fname, fname);
    else
        fprintf(1, ' For more details see %s.', fname);
    end
    fprintf(1, '\n');
end

end % end function


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% HELPER FUNCTION.                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function struct = merge_structs(varargin)
%MERGE_STRUCTS Merges structs together.
%
% SYNTAX:
%   struct = MERGE_STRUCTS(struct1, struct2, ...)
%
% DESCRIPTION:
%   Takes multiple structs and combines them into one with all fields.
%   If structs share a field with the same name, the latter argument's
%   value is used.
%
% INPUT:
%   struct1, struct2, ... - arbitrary structs
%
% OUTPUT:
%   struct - struct, containing all fields of struct1, struct2, ...

struct = varargin{1};
for k = 2:length(varargin)
    st     = varargin{k};
    fields = fieldnames(st);

    for field = transpose(fields)
        f                 = st.(field{1});
        struct.(field{1}) = f;
    end
end

end
