function [nrs] = ml_order(hsv, nu, ni, tselect, opts)
%ML_ORDER Order computation for model reduction methods.
%
% SYNTAX:
%   nr = ML_ORDER(hsv, tselect)
%   nr = ML_ORDER(hsv, tselect, opts)
%
% DESCRIPTION:
%   This function computes the size of the resulting reduced-order model
%   based on a given vector of characteristic values and appropriate error
%   formulas.
%
% INPUT:
%   hsv     - a vector, containing the characteristic values of the system
%   nu      - an integer, number of states corresponding to anti-stable
%             eigenvalues of the system's matrix pencil
%   ni      - an integer, number of states corresponding to infinite
%             eigenvalues of the system's matrix pencil
%   tselect - an integer, used to determine the computation method for the
%   {!}       order of the reduced-order model
%               0 - order is directly given by user
%               1 - computed by a relative tolerance for the hsv
%               2 - computed by a relative tolerance on the sum of hsv
%               3 - computed by absolute error bound of BT
%               4 - computed by relative error bound of BST
%               5 - computed by absolute error bound of LQGBT
%               6 - computed by absolute error bound of HinfBT
%   opts    - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, scaling term from the H-infinity |
%   |                 | balanced truncation                               |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   | {!}             | reduced-order model chosen by the user if         |
%   |                 | tselect == 0                                      |
%   |                 | (default min(10,length(hsv)) + nu + ni)           |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used in the         |
%   | {!}             | different error formulas                          |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
%   Note: Parameters marked with {!} may also be a cell array containing
%         multiple arguments. In this case an cell array of the same size
%         is returned with one entry computed for each input argument and
%         the marked fields of the info struct are cells as well.
%         When multiple arguments are given as cells, they are expected to
%         have the same length.
%
% OUTPUT:
%   nrs - size of the reduced-order model
%
% See also ml_dss_bfsr, ml_ss_bfsr, ml_dss_sr, ml_ss_sr.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(4, 5);

if (nargin < 5) || isempty(opts)
    opts = struct();
end

% Check of input arguments.
ml_assert_nonnegvector(hsv, 'hsv');

hsv = sort(hsv(hsv > 0), 'descend');

if isempty(nu)
    nu = 0;
else
    ml_assert_nonnegint(nu, 'nu');
    
    nu = floor(nu);
end

if isempty(ni)
    ni = 0;
else
    ml_assert_nonnegint(ni, 'ni');
    
    ni = floor(ni);
end

tselect = ml_check_cell_param(tselect, 'tselect', @ml_assert_nonnegint);

% Special case of no characteristic singular values.
if isempty(hsv)
    % still expect cell
    if isfield(opts, 'noUnpack')
        nrs = {0};
    else
        nrs = 0;
    end
    return;
end

% Check and assign optional parameters.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');

opts = ml_check_cell_param(opts, 'Order', @ml_assert_posinteger, ...
    min(10, length(hsv)) + nu + ni);

opts = ml_check_cell_param(opts, 'Tolerance', @ml_assert_nonnegscalar,...
    1.0e-02);

[tselect, opts.Order, opts.Tolerance] = ml_extend_cell(tselect,...
    opts.Order, ...
    opts.Tolerance);

numVariants = length(tselect);
nrs         = cell(1, numVariants);

% vecInd is set by a calling routine to signal that only one variant should
% be computed even when some arguments are cell arrays
if ml_field_set_to_value(opts, 'vecInd')
    hasInd   = 1;
    if numVariants > 1
        assert(numVariants >= opts.vecInd ...
            && numVariants <= opts.vecMax, ...
            'MORLAB:data', ...
            'Inconsistent length of vector parameters.');
        
        % only compute for param at that index
        indBegin = opts.vecInd;
        indEnd   = opts.vecInd;
    else
        indBegin = 1;
        indEnd   = 1;
    end
else
    hasInd   = 0;
    indBegin = 1;
    indEnd   = numVariants;
end

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHOOSE ERROR FORMULA.                                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for k = indBegin:indEnd
    % Computation of the order.
    switch tselect{k}
        case 0 % Selection by a given order.
            nr = opts.Order{k} - nu - ni;
            
            if nr > length(hsv)
                nr = length(hsv);
                
                warning('MORLAB:order', ...
                    ['The desired size of the reduced-order model is ' ...
                    'too large. \nSet opts.Order = %d.'], ...
                    nr + nu + ni);
            elseif nr < 0
                nr = 0;
                
                warning('MORLAB:order', ...
                    ['The desired size of the reduced-order model is ' ...
                    'too small. \nSet opts.Order = %d.'], ...
                    nr + nu + ni);
            end
            
        case 1 % Selection by a given tolerance multiplied with the largest
            % characteristic value as lower bound.
            nr = sum(hsv > hsv(1) * opts.Tolerance{k});
            
        case 2 % Selection by a given tolerance multiplied with the largest
            % characteristic value as lower bound on the sum.
            abserr = 0;
            nr     = length(hsv) + 1;
            
            while abserr <= (hsv(1) * opts.Tolerance{k})
                nr = nr - 1;
                
                if nr == 0
                    break;
                end
                
                abserr = abserr + hsv(nr);
            end
            
        case 3 % Selection by a given tolerance and the error formula of
            % the balanced truncation.
            abserr = 0;
            nr     = length(hsv) + 1;
            
            while abserr <= (opts.Tolerance{k} / 2)
                nr = nr - 1;
                
                if nr == 0
                    break;
                end
                
                abserr = abserr + hsv(nr);
            end
            
        case 4 % Selection by a given tolerance and the error formula of
            % the balanced stochastic truncation.
            relerr = 1;
            nr     = length(hsv) + 1;
            
            while relerr <= (opts.Tolerance{k} + 1)
                nr     = nr - 1;
                
                if nr == 0
                    break;
                end
                
                relerr = relerr * (1 + hsv(nr)) / (1 - hsv(nr));
            end
            
        case 5 % Selection by a given tolerance and the error formula of
            % the LQG balanced truncation.
            abserr = 0;
            nr     = length(hsv) + 1;
            
            while abserr <= (opts.Tolerance{k} / 2)
                nr = nr - 1;
                
                if nr == 0
                    break;
                end
                
                abserr = abserr + hsv(nr) / sqrt(1 + hsv(nr)^2);
            end
            
        case 6 % Selection by a given tolerance and the error formula of
            % the Hinf balanced truncation.
            if ml_field_set_to_value(opts, 'Gamma')
                ml_assert_posscalar(opts.Gamma, 'opts.Gamma');
            else
                error('MORLAB:data', ...
                    'The parameter opts.Gamma is needed!');
            end
            
            abserr = 0;
            nr     = length(hsv) + 1;
            
            while abserr <= (opts.Tolerance{k} / 2)
                nr = nr - 1;
                
                if nr == 0
                    break;
                end
                
                abserr = abserr + hsv(nr) / sqrt(1 ...
                    + ((1 - 1 / opts.Gamma^2) * hsv(nr))^2);
            end
            
        otherwise
            error('MORLAB:data', ...
                'The given parameter tselect is not implemented!');
    end
    nrs{k} = nr;
end

% Unpack.
if length(nrs) == 1
    if not(isfield(opts, 'noUnpack'))
        nrs = nrs{1};
    end
    % Only one is expected, rest is undefined.
elseif hasInd
    if isfield(opts, 'noUnpack')
        % Take single element as cell.
        nrs = nrs(opts.vecInd);
    else
        nrs = nrs{opts.vecInd};
    end
end
