function [roms, hsvs, Ts, Ws] = ml_soss_bfsr(sys, R,  L, tselect, opts)
%ML_SOSS_BFSR Balancing-free square root method for second-order systems.
%
% SYNTAX:
%   [roms, hsvs, Ts, Ws] = ML_SOSS_BFSR(sys, R, L, tselect)
%   [roms, hsvs, Ts, Ws] = ML_SOSS_BFSR(sys, R, L, tselect, opts)
%
% DESCRIPTION:
%   Computes the reduced-order matrices of a second-order system by the
%   balancing-free square root method. The different second-order
%   balancing methods can be chosen via an optional paramter.
%
% INPUT:
%   sys     - structure, containing the second-order system in the form:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | M               | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   | E               | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   | K               | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   | Bu              | matrix with dimensions n x m                      |
%   +-----------------+---------------------------------------------------+
%   | Cp              | matrix with dimensions p x n, might be empty      |
%   +-----------------+---------------------------------------------------+
%   | Cv              | matrix with dimensions p x n, might be empty      |
%   +-----------------+---------------------------------------------------+
%   R       - Cholesky factor of the controllability Gramian with
%             dimensions 2*n x nr
%   L       - Cholesky factor of the observability Gramian with
%             dimensions 2*n x nl
%   tselect - an integer, used to determine the computation method for the
%             order of the reduced-order model
%               0 - order is directly given by user
%               1 - computed by a relative tolerance for the hsv
%               2 - computed by a relative tolerance on the sum of hsv
%               3 - computed by absolute error bound of BT
%               4 - computed by relative error bound of BST
%               5 - computed by absolute error bound of LQGBT
%               6 - computed by absolute error bound of HinfBT
%   opts    - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | BalanceType     | character array, determining the type of          |
%   | {!}             | second-order balancing                            |
%   |                 |   'p'   - position balancing                      |
%   |                 |   'pm'  - position balancing (diagonalized M)     |
%   |                 |   'pv'  - position-velocity balancing             |
%   |                 |   'vp'  - velocity-position balancing             |
%   |                 |   'vpm' - velocity-position balancing (diag. M)   |
%   |                 |   'v'   - velocity balancing                      |
%   |                 |   'fv'  - free velocity balancing                 |
%   |                 |   'so'  - second-order balancing                  |
%   |                 | (default 'so')                                    |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, scaling term from the H-infinity |
%   |                 | balanced truncation                               |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   | {!}             | reduced-order model chosen by the user if         |
%   |                 | tselect == 1                                      |
%   |                 | (default min(10,length(hsv)))                     |
%   +-----------------+---------------------------------------------------+
%   | StoreProjection | {0, 1}, used to disable/enable storing of the     |
%   |                 | computed projection matrices W and T              |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used in the         |
%   | {!}             | different error formulas                          |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%   Note: Parameters marked with {!} may also be a cell array containing
%         multiple arguments. In this case an cell array of the same size
%         is returned with one entry computed for each input argument and
%         the marked fields of the info struct are cells as well.
%         When multiple arguments are given as cells, they are expected to
%         have the same length.
%
% OUTPUT:
%   roms - struct, containing the transformed system matrices
%   hsvs - vector, containing the characteristic singular values
%   Ts   - right projection matrix, if opts.StoreProjection == 1
%   Ws   - left projection matrix, if opts.StoreProjection == 1
%
%
% REFERENCE:
%   T. Reis, T. Stykel, Balanced truncation model reduction of second-order
%   systems, Math. Comput. Model. Dyn. Syst. 14 (5) (2008) 391--406.
%   https://doi.org/10.1080/13873950701844170
%
% See also ml_ss_bfsr, ml_ss_sr, ml_order.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(4, 5);

if (nargin < 5) || isempty(opts)
    opts = struct();
end

% Check of input second-order system.
ml_assert_sosys(sys);

n = size(sys.M, 1);

assert(size(R, 1) == 2 * n, ...
    'MORLAB:data', ...
    'The matrix R must have twice the number of rows as sys.M!');

assert(size(L, 1) == 2 * n, ...
    'MORLAB:data', ...
    'The matrix L must have twice the number of rows as sys.M!');

opts = ml_check_cell_param(opts, 'BalanceType', @ml_assert_char, 'so');

if issparse(R), R = full(R); end
if issparse(L), L = full(L); end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'StoreProjection')
    ml_assert_boolean(opts.StoreProjection, 'opts.StoreProjection');
else
    opts.StoreProjection = false;
end

numVariants = length(opts.BalanceType);
% vecInd is set by a calling routine to signal that only one variant should
% be computed even when some arguments are cell arrays.
if ml_field_set_to_value(opts, 'vecInd')
    hasInd = 1;
    if numVariants > 1
        assert(numVariants >= opts.vecInd ...
            && numVariants <= opts.vecMax, ...
            'MORLAB:data', ...
            'Inconsistent length of vector parameters.');
        
        % Only compute for param at that index.
        indBegin = opts.vecInd;
        indEnd   = opts.vecInd;
    end
else
    hasInd = 0;
    if numVariants > 1
        opts.vecMax = numVariants;
    end
    indBegin = 1;
    indEnd   = numVariants;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTATION OF REDUCED-ORDER MODEL.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Makes implementation of compute_reduced_model simpler.
opts.noUnpack = 1;
% No vectorization on this level, just forward results.
if numVariants == 1
    [roms, hsvs, Ts, Ws] = compute_reduced_model(sys, R, L, tselect, n, ...
        opts.BalanceType{1}, opts);
else
    roms = cell(1, numVariants);
    hsvs = cell(1, numVariants);
    Ts   = cell(1, numVariants);
    Ws   = cell(1, numVariants);
    for k = indBegin:indEnd
        balanceType = opts.BalanceType{k};
        
        % Introduce index for ml_order; when already set by the calling
        % routine this has no effect since k == opts.vecInd.
        opts.vecInd = k;
        
        [rom, hsv, T, W] = compute_reduced_model(sys, R, L, tselect, n, ...
            balanceType, opts);
        roms{k} = rom{1};
        hsvs{k} = hsv;
        Ts{k}   = T{1};
        Ws{k}   = W{1};
    end
end

if not(opts.StoreProjection)
    Ts = [];
    Ws = [];
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% HANDLE OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

len = length(roms);
% Pass D even when empty.
if isfield(sys, 'D')
    for k = 1:len
        roms{k}.D = sys.D;
    end
end

% Unpack single result.
if len == 1
    roms = roms{1};
    
    if opts.StoreProjection
        Ts = Ts{1};
        Ws = Ws{1};
    end
elseif hasInd
    roms = roms{opts.vecInd};
    hsvs = hsvs{opts.vecInd};
    
    if opts.StoreProjection
        Ts = Ts{opts.vecInd};
        Ws = Ws{opts.vecInd};
    end
end

end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% HELPER FUNCTION.                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [roms, hsv, Ts, Ws] = compute_reduced_model(sys, R, L, ...
    tselect, n, balanceType, opts)
%COMPUTE_REDUCED_MODEL Compute projection matrices and reduced model.
%
% SYNTAX:
%   sys = COMPUTE_REDUCED_MODEL(sys, R, L, tselect, n, balanceType, opts)
%
% DESCRIPTION:
%   Sets up the projection matrices and computes the reduced-order model.
%
% INPUT:
%   sys         - structure, containing the second-order system in the
%                 form:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | M               | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   | E               | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   | K               | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   | Bu              | matrix with dimensions n x m                      |
%   +-----------------+---------------------------------------------------+
%   | Cp              | matrix with dimensions p x n, might be empty      |
%   +-----------------+---------------------------------------------------+
%   | Cv              | matrix with dimensions p x n, might be empty      |
%   +-----------------+---------------------------------------------------+
%   R           - Cholesky factor of the controllability Gramian with
%                 dimensions 2*n x nr
%   L           - Cholesky factor of the observability Gramian with
%                 dimensions 2*n x nl
%   tselect     - an integer, used to determine the computation method for
%                 the order of the reduced-order model
%                   0 - order is directly given by user
%                   1 - computed by a relative tolerance for the hsv
%                   2 - computed by a relative tolerance on the sum of hsv
%                   3 - computed by absolute error bound of BT
%                   4 - computed by relative error bound of BST
%                   5 - computed by absolute error bound of LQGBT
%                   6 - computed by absolute error bound of HinfBT
%   n           - size of second-order system
%   balanceType - character array, determining the type of second-order
%                 balancing
%                   'p'   - position balancing
%                   'pm'  - position balancing (diagonalizing M)
%                   'pv'  - position-velocity balancing
%                   'vp'  - velocity-position balancing
%                   'vpm' - velocity-position balancing (diagonalizing M)
%                   'v'   - velocity balancing
%                   'fv'  - free velocity balancing
%                   'so'  - second-order balancing
%   opts        - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, scaling term from the H-infinity |
%   |                 | balanced truncation                               |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | tselect == 1                                      |
%   |                 | (default min(10,length(hsv)))                     |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used in the         |
%   |                 | different error formulas                          |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   sys         - struct, containing the transformed system matrices

switch lower(balanceType)
    case 'p' % Position Balancing.
        % Compute position Hankel singular values.
        [~, S, V] = svd(L(1:n, :)' * R(1:n, :), 'econ');
        [U, ~, ~] = svd(L(n+1:end, :)' * sys.M * R(n+1:end, :), 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        order = ml_order(hsv, 0, 0, tselect, opts);
        
        for p = length(order):-1:1
            r = order{p};
            % Compute the transformation matrices.
            [W, ~] = qr(L(n+1:end, :) * U(:, 1:r), 0);
            [T, ~] = qr(R(1:n, :) * V(:, 1:r), 0);
            
            % Compute reduced-order model.
            rom.M  = W' * sys.M * T;
            rom.E  = W' * sys.E * T;
            rom.K  = W' * sys.K * T;
            rom.Bu = W' * sys.Bu;
            
            if ml_field_set_to_value(sys, 'Cp')
                rom.Cp = sys.Cp * T;
            else
                rom.Cp = [];
            end
            if ml_field_set_to_value(sys, 'Cv')
                rom.Cv = sys.Cv * T;
            else
                rom.Cv = [];
            end
            
            Ts{p}   = T;
            Ws{p}   = W';
            roms{p} = rom;
        end
        
    case 'pm' % Position Balancing (diagonalizing M).
        % Compute position Hankel singular values.
        [U, S, V] = svd(L(1:n, :)' * R(1:n, :), 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        order = ml_order(hsv, 0, 0, tselect, opts);
        
        for p = length(order):-1:1
            r = order{p};
            % Compute the transformation matrices.
            [W, ~] = qr(sys.M' \ (L(1:n, :) * U(:, 1:r)), 0);
            [T, ~] = qr(R(1:n, :) * V(:, 1:r), 0);
            
            % Compute reduced-order model.
            rom.M  = W' * sys.M * T;
            rom.E  = W' * sys.E * T;
            rom.K  = W' * sys.K * T;
            rom.Bu = W' * sys.Bu;
            
            if ml_field_set_to_value(sys, 'Cp')
                rom.Cp = sys.Cp * T;
            else
                rom.Cp = [];
            end
            if ml_field_set_to_value(sys, 'Cv')
                rom.Cv = sys.Cv * T;
            else
                rom.Cv = [];
            end
            
            Ts{p}   = T;
            Ws{p}   = W';
            roms{p} = rom;
        end
        
    case 'pv' % Position-Velocity Balancing.
        % Compute position-velocity Hankel singular values.
        [U, S, V] = svd(L(n+1:end, :)' * sys.M * R(1:n, :), 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        order = ml_order(hsv, 0, 0, tselect, opts);
        
        for p = length(order):-1:1
            r = order{p};
            % Compute the transformation matrices.
            [W, ~] = qr(L(n+1:end, :) * U(:, 1:r), 0);
            [T, ~] = qr(R(1:n, :) * V(:, 1:r), 0);
            
            % Compute reduced-order model.
            rom.M  = W' * sys.M * T;
            rom.E  = W' * sys.E * T;
            rom.K  = W' * sys.K * T;
            rom.Bu = W' * sys.Bu;
            
            if ml_field_set_to_value(sys, 'Cp')
                rom.Cp = sys.Cp * T;
            else
                rom.Cp = [];
            end
            if ml_field_set_to_value(sys, 'Cv')
                rom.Cv = sys.Cv * T;
            else
                rom.Cv = [];
            end
            
            Ts{p}   = T;
            Ws{p}   = W';
            roms{p} = rom;
        end
        
    case 'vp' %Velocity-Position Balancing.
        % Compute velocity-position Hankel singular values.
        [~, S, V] = svd(L(1:n, :)' * R(n+1:end, :), 'econ');
        [U, ~, ~] = svd(L(n+1:end, :)' * sys.M * R(n+1:end, :), 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        order = ml_order(hsv, 0, 0, tselect, opts);
        
        for p = length(order):-1:1
            r = order{p};
            % Compute the transformation matrices.
            [W, ~] = qr(L(n+1:end, :) * U(:, 1:r), 0);
            [T, ~] = qr(R(n+1:end, :) * V(:, 1:r), 0);
            
            % Compute reduced-order model.
            rom.M  = W' * sys.M * T;
            rom.E  = W' * sys.E * T;
            rom.K  = W' * sys.K * T;
            rom.Bu = W' * sys.Bu;
            
            if ml_field_set_to_value(sys, 'Cp')
                rom.Cp = sys.Cp * T;
            else
                rom.Cp = [];
            end
            if ml_field_set_to_value(sys, 'Cv')
                rom.Cv = sys.Cv * T;
            else
                rom.Cv = [];
            end
            
            Ts{p}   = T;
            Ws{p}   = W';
            roms{p} = rom;
        end
        
    case 'vpm' % Velocity-Position Balancing (diagonalizing M).
        % Compute velocity-position Hankel singular values.
        [U, S, V] = svd(L(1:n, :)' * R(n+1:end, :), 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        order = ml_order(hsv, 0, 0, tselect, opts);
        
        for p = length(order):-1:1
            r = order{p};
            % Compute the transformation matrices.
            [W, ~] = qr(sys.M' \ (L(1:n, :) * U(:, 1:r)), 0);
            [T, ~] = qr(R(n+1:end, :) * V(:, 1:r), 0);
            
            % Compute reduced-order model.
            rom.M  = W' * sys.M * T;
            rom.E  = W' * sys.E * T;
            rom.K  = W' * sys.K * T;
            rom.Bu = W' * sys.Bu;
            
            if ml_field_set_to_value(sys, 'Cp')
                rom.Cp = sys.Cp * T;
            else
                rom.Cp = [];
            end
            if ml_field_set_to_value(sys, 'Cv')
                rom.Cv = sys.Cv * T;
            else
                rom.Cv = [];
            end
            
            Ts{p}   = T;
            Ws{p}   = W';
            roms{p} = rom;
        end
        
    case 'v' % Velocity Balancing.
        % Compute velocity Hankel singular values.
        [U, S, V] = svd(L(n+1:end, :)' * sys.M * R(n+1:end, :), 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        order = ml_order(hsv, 0, 0, tselect, opts);
        
        for p = length(order):-1:1
            r = order{p};
            % Compute the transformation matrices.
            [W, ~] = qr(L(n+1:end, :) * U(:, 1:r), 0);
            [T, ~] = qr(R(n+1:end, :) * V(:, 1:r), 0);
            
            % Compute reduced-order model.
            rom.M  = W' * sys.M * T;
            rom.E  = W' * sys.E * T;
            rom.K  = W' * sys.K * T;
            rom.Bu = W' * sys.Bu;
            
            if ml_field_set_to_value(sys, 'Cp')
                rom.Cp = sys.Cp * T;
            else
                rom.Cp = [];
            end
            if ml_field_set_to_value(sys, 'Cv')
                rom.Cv = sys.Cv * T;
            else
                rom.Cv = [];
            end
            
            Ts{p}   = T;
            Ws{p}   = W';
            roms{p} = rom;
        end
        
    case 'fv' % Free Velocity Balancing.
        % Compute the position Hankel singular values.
        [~, S, V] = svd(L(1:n, :)' * R(1:n, :), 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        order = ml_order(hsv, 0, 0, tselect, opts);
        
        for p = length(order):-1:1
            r = order{p};
            % Compute the transformation matrices.
            [T, ~] = qr(R(1:n, :) * V(:, 1:r), 0);
            W      = T;
            
            % Compute reduced-order model.
            rom.M  = W' * sys.M * T;
            rom.E  = W' * sys.E * T;
            rom.K  = W' * sys.K * T;
            rom.Bu = W' * sys.Bu;
            
            if ml_field_set_to_value(sys, 'Cp')
                rom.Cp = sys.Cp * T;
            else
                rom.Cp = [];
            end
            if ml_field_set_to_value(sys, 'Cv')
                rom.Cv = sys.Cv * T;
            else
                rom.Cv = [];
            end
            
            Ts{p}   = T;
            Ws{p}   = W';
            roms{p} = rom;
        end
        
    case 'so' % Second-Order Balancing.
        % Compute position and velocity Hankel singular values.
        [Up, Sp, Vp] = svd(L(1:n, :)' * R(1:n, :), 'econ');
        hsvp         = diag(Sp);
        hsvp         = hsvp(1:min(length(hsvp), n));
        [Uv, Sv, Vv] = svd(L(n+1:end, :)' * sys.M * R(n+1:end, :), 'econ');
        hsvv         = diag(Sv);
        hsvv         = hsvv(1:min(length(hsvv), n));
        
        % Computation of the order of the reduced-order system.
        order1 = ml_order(hsvp, 0, 0, tselect, opts);
        order2 = ml_order(hsvv, 0, 0, tselect, opts);
        
        for p = length(order1):-1:1
            r = min(order1{p}, order2{p});
            % Compute the transformation matrices.
            [Wp, ~] = qr(L(1:n, :) * Up(:, 1:r), 0);
            [Wv, ~] = qr(L(n+1:end, :) * Uv(:, 1:r), 0);
            [Tp, ~] = qr(R(1:n, :) * Vp(:, 1:r), 0);
            [Tv, ~] = qr(R(n+1:end, :) * Vv(:, 1:r), 0);
            
            S = (Wp' * Tp) \ (Wp' * Tv);
            
            % Compute reduced-order model.
            rom.M  = S * ((Wv' * sys.M * Tv) / S);
            rom.E  = S * ((Wv' * sys.E * Tv) / S);
            rom.K  = S * (Wv' * sys.K * Tp);
            rom.Bu = S * (Wv' * sys.Bu);
            
            if ml_field_set_to_value(sys, 'Cp')
                rom.Cp = sys.Cp * Tp;
            else
                rom.Cp = [];
            end
            if ml_field_set_to_value(sys, 'Cv')
                rom.Cv = (sys.Cv * Tv) / S;
            else
                rom.Cv = [];
            end
            
            Ts{p}.Tp = Tp;
            Ts{p}.Tv = Tv;
            Ws{p}.Wp = Wp';
            Ws{p}.Wv = Wv';
            roms{p}  = rom;
        end
        hsv = struct('Hsvp', hsvp, 'Hsvv', hsvv);
        
    otherwise
        error('MORLAB:data', ...
            'The requested balancing method is not implemented!');
end

end
