function [varargout] = ml_soss_format_output(systems, ioformat, info)
%ML_SOSS_FORMAT_OUTPUT Converts a cell array of systems to another format.
%
% SYNTAX:
%   [M, E, K, Bu, Cp, Cv, D, info] = ML_SOSS_FORMAT_OUTPUT(systems, 0)
%   [sys, info]                    = ML_SOSS_FORMAT_OUTPUT(systems, 1)
%
% DESCRIPTION:
%   This function takes an array of system-structs and returns cell arrays
%   of the requested format.
%   If only a single system is given no cell array is created and single
%   element cell arrays in the info struct are unpacked.
%
% INPUT:
%   systems  - cell-array of structs containing the descriptor system's
%              matrices
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        M        | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   |        K        | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   |        Bu       | matrix with dimensions n x m                      |
%   +-----------------+---------------------------------------------------+
%   |        Cp       | matrix with dimensions p x n                      |
%   +-----------------+---------------------------------------------------+
%   |        Cv       | matrix with dimensions p x n                      |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix with dimensions p x m                      |
%   +-----------------+---------------------------------------------------+
%   ioformat - integer, describing the output format.
%                0 - for input type matrices
%                1 - for input type struct
%   info     - struct, containing additional information.
%
% OUTPUT:
%   M    - cell array of matrices with dimensions n x n
%   E    - cell array of matrices with dimensions n x n
%   K    - cell array of matrices with dimensions n x n
%   Bu   - cell array of matrices with dimensions n x m
%   Cp   - cell array of matrices with dimensions p x n
%   Cv   - cell array of matrices with dimensions p x n
%   Dr   - cell array of matrices with dimensions p x m
%   sys  - cell array of structures, containing the second-order system's
%          matrices
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        M        | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   |        K        | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   |        Bu       | matrix with dimensions n x m                      |
%   +-----------------+---------------------------------------------------+
%   |        Cp       | matrix with dimensions p x n                      |
%   +-----------------+---------------------------------------------------+
%   |        Cv       | matrix with dimensions p x n                      |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix with dimensions p x m                      |
%   +-----------------+---------------------------------------------------+
%   info - info struct with obsolete cell arrays unpacked
%
% See also ml_ss_format_output, ml_dss_format_output.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%

narginchk(3,3);

assert(isa(systems, 'cell') ...
    && isvector(systems), ...
    'MORLAB:data', ...
    'The input argument systems has to be a cell vector!');

assert(isa(info, 'struct'), ...
    'MORLAB:data', ...
    'The input argument info has to be a struct!');

% Sort info fields.
[~, perm] = sort(lower(fieldnames(info)));
info      = orderfields(info, perm);

% Format output ROMs.
numSys = length(systems);

for k = numSys:-1:1
    sys = structfun(@full, systems{k}, 'UniformOutput', 0);
    switch ioformat
        case 0
            varargout{1}{k} = sys.M;
            varargout{2}{k} = sys.E;
            varargout{3}{k} = sys.K;
            varargout{4}{k} = sys.Bu;
            varargout{5}{k} = sys.Cp;
            varargout{6}{k} = sys.Cv;
            varargout{7}{k} = sys.D;
            
        case 1
            varargout{1}{k} = struct( ...
                'M' , sys.M, ...
                'E' , sys.E, ...
                'K' , sys.K, ...
                'Bu', sys.Bu, ...
                'Cp', sys.Cp, ...
                'Cv', sys.Cv, ...
                'D' , sys.D);
            
            if isfield(sys, 'Ts')
                varargout{1}{k}.Ts = sys.Ts;
            end
            
        otherwise
            error('MORLAB:data', ...
                'The input argument ioformat has to be 0 or 1!');
    end
end

% A single result is unpacked.
if numSys == 1
    for k = 1:length(varargout)
        varargout{k} = varargout{k}{1};
    end
    
    % Iterate fields of the info struct.
    fields = fieldnames(info);
    for field = fields'
        f = info.(field{1});
        if isa(f, 'cell') && length(f) == 1
            info.(field{1}) = f{1};
        end
    end
end

% Forward info structure.
len              = length(varargout);
varargout{len+1} = info;
