function [sys, opts, ioformat] = ml_soss_lti2struct(varargin)
%ML_SOSS_LTI2STRUCT Maps given input to a second-order system struct.
%
% SYNTAX:
%   [sys, opts, ioformat] = ML_SOSS_LTI2STRUCT(M, E, K, Bu, Cp, Cv, D)
%   [sys, opts, ioformat] = ML_SOSS_LTI2STRUCT(M, E, K, Bu, Cp, Cv, D, opts)
%
%   [sys, opts, ioformat] = ML_SOSS_LTI2STRUCT(sys)
%   [sys, opts, ioformat] = ML_SOSS_LTI2STRUCT(sys, opts)
%
% DESCRIPTION:
%   This function gets the possible inputs of the model order reduction
%   methods, checks the system matrices and returns structs for the system
%   and the options in the right format
%
%       M*x''(t) = -K*x(t) -  E*x'(t) + Bu*u(t)                         (1)
%           y(t) = Cp*x(t) + Cv*x'(t) +  D*u(t)                         (2)
%
%   Also, the input format is returned as an integer.
%
% INPUT:
%   M    - matrix from (1) with dimensions n x n
%   E    - matrix from (1) with dimensions n x n
%   K    - matrix from (1) with dimensions n x n
%   Bu   - matrix from (1) with dimensions n x m
%   Cp   - matrix from (2) with dimensions p x n
%   Cv   - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   sys  - structure, containing the standard system's matrices
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        M        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        K        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        Bu       | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        Cp       | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        Cv       | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing optional parameters
%
% OUTPUT:
%   sys      - structure, containing the second-order system's matrices
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        M        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        K        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        Bu       | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        Cp       | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        Cv       | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   opts     - structure, containing optional parameters
%   ioformat - integer, describing the input format
%                0 - for input type matrices
%                1 - for input type struct
%
% See also ml_ss_lti2struct, ml_dss_lti2struct.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


% Ensure existing inputs.
narginchk(1, 8);

if isa(varargin{1}, 'double') % Input case: matrices.
    narginchk(7, 8)
    
    if (nargin < 8) || isempty(varargin{8})
        opts = struct();
    else
        opts = varargin{8};
    end
    
    % Assign and check standard system.
    sys = struct(...
        'M' , varargin{1}, ...
        'E' , varargin{2}, ...
        'K' , varargin{3}, ...
        'Bu', varargin{4}, ...
        'Cp', varargin{5}, ...
        'Cv', varargin{6}, ...
        'D' , varargin{7});
    
    ml_assert_sosys(sys);
    
    if ml_field_set_to_value(sys, 'D')
        if ml_field_set_to_value(sys, 'Cp')
            assert(isequal(size(sys.D), ...
                [size(sys.Cp, 1), size(sys.Bu, 2)]), ...
                'MORLAB:data', ...
                'The matrix D must have the dimensions %d x %d!', ...
                size(sys.Cp, 1), size(sys.Bu, 2));
        else
            assert(isequal(size(sys.D), ...
                [size(sys.Cv, 1), size(sys.Bu, 2)]), ...
                'MORLAB:data', ...
                'The matrix D must have the dimensions %d x %d!', ...
                size(sys.Cv, 1), size(sys.Bu, 2));
        end
    end
    
    % Output format.
    ioformat = 0;
elseif isa(varargin{1}, 'struct') % Input case: structs.
    narginchk(1, 2);
    
    if (nargin < 2) || isempty(varargin{2})
        opts = struct();
    else
        opts = varargin{2};
    end
    
    % Check input standard system.
    sys = varargin{1};
    
    ml_assert_sosys(sys);
    
    if not(ml_field_set_to_value(sys, 'Cp'))
        sys.Cp = [];
    end
    
    if not(ml_field_set_to_value(sys, 'Cv'))
        sys.Cv = [];
    end
    
    if ml_field_set_to_value(sys, 'D')
        if ml_field_set_to_value(sys, 'Cp')
            assert(isequal(size(sys.D), ...
                [size(sys.Cp, 1), size(sys.Bu, 2)]), ...
                'MORLAB:data', ...
                'The matrix D must have the dimensions %d x %d!', ...
                size(sys.Cp, 1), size(sys.Bu, 2));
        else
            assert(isequal(size(sys.D), ...
                [size(sys.Cv, 1), size(sys.Bu, 2)]), ...
                'MORLAB:data', ...
                'The matrix D must have the dimensions %d x %d!', ...
                size(sys.Cv, 1), size(sys.Bu, 2));
        end
    else
        sys.D = [];
    end
    
    % Output format.
    ioformat = 1;
end

% Convert to full struct.
if any(structfun(@issparse, sys, 'UniformOutput', 1))
    warning('MORLAB:lti2struct', ...
        'Converting from sparse to full system matrices!');
    sys = structfun(@full, sys, 'UniformOutput', 0);
end

% Check option struct.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');
