function [rom, hsv, T, W] = ml_ss_bfsr(sys, R, L, tselect, opts)
%ML_SS_BFSR Balancing-free square root method for standard systems.
%
% SYNTAX:
%   [sys, hsv, T, W] = ML_SS_BFSR(sys, R, L, tselect)
%   [sys, hsv, T, W] = ML_SS_BFSR(sys, R, L, tselect, opts)
%
% DESCRIPTION:
%   Computes the reduced-order matrices of a standard system by the
%   balancing-free square root method. Therefore, transformation matrices
%   of the form
%
%       W = (X' * Y)^(-T) * Y',
%       T = X,
%
%   are computed, where X, Y result from the QR decompositions
%
%       [X, ~]    = qr(R * V),
%       [Y, ~]    = qr(L' * U).
%
%   The computation of the reduced-order model is done by
%
%       Ar = W * A * T,
%
%       Br = W * B,
%
%       Cr = C * T.
%
% INPUT:
%   sys     - structure, containing the standard system in the form:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | A               | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   | B               | matrix with dimensions n x m                      |
%   +-----------------+---------------------------------------------------+
%   | C               | matrix with dimensions p x n                      |
%   +-----------------+---------------------------------------------------+
%   R       - Cholesky factor of the controllability Gramian with
%             dimensions n x nr
%   L       - Cholesky factor of the observability Gramian with
%             dimensions n x nl
%   tselect - an integer, used to determine the computation method for the
%             order of the reduced-order model
%               0 - order is directly given by user
%               1 - computed by a relative tolerance for the hsv
%               2 - computed by a relative tolerance on the sum of hsv
%               3 - computed by absolute error bound of BT
%               4 - computed by relative error bound of BST
%               5 - computed by absolute error bound of LQGBT
%               6 - computed by absolute error bound of HinfBT
%   opts    - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, scaling term from the H-infinity |
%   |                 | balanced truncation                               |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   | {!}             | reduced-order model chosen by the user if         |
%   |                 | tselect == 1                                      |
%   |                 | (default min(10,length(hsv)) + nu)                |
%   +-----------------+---------------------------------------------------+
%   | StoreProjection | {0, 1}, used to disable/enable storing of the     |
%   |                 | computed projection matrices W and T              |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used in the         |
%   | {!}             | different error formulas                          |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
%   Note: Parameters marked with {!} may also be a cell array containing
%         multiple arguments. In this case an cell array of the same size
%         is returned with one entry computed for each input argument and
%         the marked fields of the info struct are cells as well.
%         When multiple arguments are given as cells, they are expected to
%         have the same length.
%
% OUTPUT:
%   sys - struct, containing the transformed system matrices
%   hsv - vector, containing the characteristic singular values
%   T   - right projection matrix, if opts.StoreProjection == 1
%   W   - left projection matrix, if opts.StoreProjection == 1
%
%
% REFERENCE:
%   A. Varga, Controller reduction using accuracy-enhancing methods, in:
%   P. Benner, V. Mehrmann, D. Sorensen (Eds.), Dimension Reduction of
%   Large-Scale Systems, Vol. 45 of Lect. Notes Comput. Sci. Eng.,
%   Springer-Verlag, Berlin/Heidelberg, Germany, 2005, pp. 353--356.
%   https://doi.org/10.1007/3-540-27909-1_9
%
% See also ml_dss_bfsr, ml_ss_sr, ml_order.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2019 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(4, 5);

if (nargin < 5) || isempty(opts)
    opts = struct();
end

% Check of input standard system.
ml_assert_stdsys(sys);

n = size(sys.A, 1);

assert(size(R, 1) == n, ...
    'MORLAB:data', ...
    'The matrix R must have the same number of rows as sys.A!');

assert(size(L, 1) == n, ...
    'MORLAB:data', ...
    'The matrix L must have the same number of rows as sys.A!');

if issparse(R), R = full(R); end
if issparse(L), L = full(L); end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'StoreProjection')
    ml_assert_boolean(opts.StoreProjection, 'opts.StoreProjection');
else
    opts.StoreProjection = false;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTATION OF CHARACTERISTIC SINGULAR VALUES.                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[U, S, V] = svd(L' * R, 'econ');
hsv       = nonzeros(diag(S));


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTATION OF REDUCED-ORDER MODEL.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computation of the order of the reduced-order system.
if ml_field_set_to_value(sys, 'Au')
    num = ml_order(hsv, size(sys.Au, 1), 0, tselect, opts);
else
    num = ml_order(hsv, 0, 0, tselect, opts);
end

len = length(num);
if len == 1
    [rom, T, W] = compute_reduced_model(sys, R, L, U, V, num);
else
    rom = cell(1, length(num));
    T   = cell(1, length(num));
    W   = cell(1, length(num));
    for k = 1:length(num)
        [rom{k}, T{k}, W{k}] = compute_reduced_model(sys, R, L, U, V, ...
            num{k});
    end
end

if not(opts.StoreProjection)
    T = [];
    W = [];
end

end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% HELPER FUNCTION.                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [sys, T, W] = compute_reduced_model(sys, R, L, U, V, n)
%COMPUTE_REDUCED_MODEL Compute projection matrices and reduced model.
%
% SYNTAX:
%   sys = COMPUTE_REDUCED_MODEL(sys, R, L, U, V, n)
%
% DESCRIPTION:
%   Sets up the projection matrices and computes the reduced-order model.
%
% INPUT:
%   sys - structure, containing the standard system in the form:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | A               | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   | B               | matrix with dimensions n x m                      |
%   +-----------------+---------------------------------------------------+
%   | C               | matrix with dimensions p x n                      |
%   +-----------------+---------------------------------------------------+
%   R   - Cholesky factor of the controllability Gramian with
%         dimensions n x nr
%   L   - Cholesky factor of the observability Gramian with
%         dimensions n x nl
%   U   - left singular vector matrix, such that U*S*V = L'*R
%   V   - right singular vector matrix, such that U*S*V = L'*R
%   n   - size of reduced-order model
%
% OUTPUT:
%   sys - struct, containing the transformed system matrices
%   T   - right projection matrix, if st == 1
%   W   - left projection matrix, if st == 1

% Computation of the transformation matrices.
[T, ~] = qr(R * V(: , 1:n), 0);
[W, ~] = qr(L * U(: , 1:n), 0);
W      = (W' * T) \ W';

% Computation of the reduced-order matrices.
sys.A = W * sys.A * T;
sys.B = W * sys.B;
sys.C = sys.C * T;

end
